      subroutine bigout (nb1, nb2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  bigout   Updated     2006 June 30 18:26.
c               bigout   Originated  2005 June 13 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display big integers stored as an array of single decimal
c               digits, or single hexadecimal characters.
c
c     Logic:    Entry points:  bigout.
c bigout        Called by:  bigcopy, bigint, bigtran, copy, debugx, delete,
c               renamex, symbol.
c
c bigout        Calls:  aptbsum, aptbtod.
c
c     Biblio:   ?
c
c     Glossary:
c
c     abig      Input    Name of big digit.  One character, "a" to "z".
c
c     abigtag   Input    Tag to put on display of big digit abig(nb).
c                          32 characters.
c
c     adigit    Local    Hex character n of big number abig(nb) is
c                        adigit(idig(n,nb)).
c
c     idig      Input    Decimal digit n of big number abig(nb) is idig(n,nb).
c
c     nb        Local    Index from nb1 to nb2.
c
c     nb1       Input    Index of first big integer to display.
c
c     nb2       Input    Index of last  big integer to display.
c
c     ndig(nb)  Input    Number of digits in big integer abig(nb).
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Arguments.

      character*8 amount

c.... Local variables.

      dimension    icount(0:9)        ! Number of occurrences of each digit.
      character*1  adigit(0:35)       ! Hexadecimal characters, plus more.

c.... Initialize adigit.

      data adigit / '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
     &              'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
     &              'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
     &              'U', 'V', 'W', 'X', 'Y', 'Z'  /
cbug      amount = 'maximum'
cbugc***DEBUG ends.

cbugc***DEBUG begins.
cbug 9901 format ('bigout DEBUG.  nb1=',i3,'  nb2=',i3)
cbug      write (   6, 9901) nb1, nb2
cbug      write (iout, 9901) nb1, nb2
cbugc***DEBUG ends.

      if (nb1 .lt. 1)   nb1 = 1
      if (nb2 .gt. nbigm) nb2 = nbigm
cbugc***DEBUG begins.
cbug      write (   6, 9901) nb1, nb2
cbug      write (iout, 9901) nb1, nb2
cbugc***DEBUG ends.

      nbigs = 0

      do nb = nb1, nb2       ! Loop over big integers.
cbugc***DEBUG begins.
cbug 9902 format ('bigout DEBUG.  ndig=',i3,'  idig=',i3)
cbug      write (   6, 9902) ndig(nb), idig(1,nb)
cbug      write (iout, 9902) ndig(nb), idig(1,nb)
cbugc***DEBUG ends.
        if (ndig(nb) .eq. 0) then
          ndig(nb)   = 1
          idig(1,nb) = 0
        endif
  
        if ((ndig(nb) .gt. 1) .or.
     &     ((ndig(nb) .eq. 1) .and. (idig(1,nb) .gt. 0))) then

          nbigs = nbigs + 1

          if (abigtag(nb) .eq. ' ') then
            abigtag(nb) = 'is nonzero.'
          endif

cold 8110     format ('' / 'Big integer "',a1,'" ',a32,
cold     &      '    Length is',i6,' digits.  ',a)
 8110     format (/ 'big ',a1,' = ',a32,
     &      '  ! Length is',i6,' digits.  ',a)

          write (   6, 8110) abig(nb), abigtag(nb), ndig(nb), asign(nb)
          write (iout, 8110) abig(nb), abigtag(nb), ndig(nb), asign(nb)

 8120     format ('big ',a1,i5,12x,' = ',50a1)

          do nin = 1, ndig(nb), 50
            nlast = min (nin + 49, ndig(nb))
            write (   6, 8120) abig(nb), nin,
     &                         (adigit(idig(n,nb)), n = nin, nlast)
            write (iout, 8120) abig(nb), nin,
     &                         (adigit(idig(n,nb)), n = nin, nlast)
          enddo

 8130     format (14x,'          ^         ^         ^         ^',
     &      '         ^         ^')
  
          if (ndig(nb) .ge. 10) then
            write (   6, 8130)
            write (iout, 8130)
          endif

          if ((amount .eq. 'max')      .or.
     &        (amount .eq. 'maximum')) then

          call aptbtod (10, idig(1,nb), ndig(nb), idmax, iemax,
     &                  ideca, fdeca, floga, nerra)

 8150     format (2x,a1,' has integer  value =',i22,'  (dec)')
 8160     format (2x,a1,' has floating value =',1pe22.15,'  (dec)')
 8162     format (2x,a1,' has floating value =',f18.15,
     &            'E+',i3,'  (dec)')
 8163     format (2x,a1,' has floating value =',f18.15,
     &            'E+',i4,'  (dec)')
 8170     format (2x,a1,' has log_10   value =',1pe22.15,'  (dec)')
 8172     format (2x,a1,' has log_e    value =',1pe22.15,'  (dec)')

          if (ideca .ne. -999999) then
            write (   6, 8150) abig(nb), ideca
            write (iout, 8150) abig(nb), ideca
          endif

          if (fdeca .ne. -1.e99) then
            write (   6, 8160) abig(nb), fdeca
            write (iout, 8160) abig(nb), fdeca
          endif


          if (floga .ne. -99.0) then

            iloga  = floga
            if ((iloga + 0.5) .ge. iemax) then
              flogai = iloga
              flmant  = floga - flogai
              fmant   = 10.0**flmant
              if (iloga .lt. 1000) then
                write (   6, 8162) abig(nb), fmant, iloga
                write (iout, 8162) abig(nb), fmant, iloga
              else
                write (   6, 8163) abig(nb), fmant, iloga
                write (iout, 8163) abig(nb), fmant, iloga
              endif
            endif

            write (   6, 8170) abig(nb), floga
            write (iout, 8170) abig(nb), floga

            floge = floga / log10 (ebase)

            write (   6, 8172) abig(nb), floge
            write (iout, 8172) abig(nb), floge
          endif

          endif                       ! Asked for maximum output.

        elseif ((nb1 .eq. nb2) .and.
     &         ((ndig(nb) .eq. 1) .and. (idig(1,nb) .eq. 0))) then

          write (   6, 8110) abig(nb), abigtag(nb), ndig(nb), asign(nb)
          write (iout, 8110) abig(nb), abigtag(nb), ndig(nb), asign(nb)

          nin = 1
          write (   6, 8120) abig(nb), nin, adigit(idig(1,nb))
          write (iout, 8120) abig(nb), nin, adigit(idig(1,nb))
  
cold          write (   6, 8130)
cold          write (iout, 8130)

        endif

cbugc***DEBUG begins.
cbug        if ((amount .eq. 'max')      .or.
cbug     &      (amount .eq. 'maximum')) then
cbug
cbug   8180   format (/ 2x,a1,' digit =',10i5)
cbug   8190   format (  2x,a1,' count =',10i5)
cbug  
cbug          if (ndig(nb) .gt. 1) then
cbug            call aptbsum (10, idig(1,nb), ndig(nb), icount, nerra)
cbug  
cbug            write (   6, 8180) abig(nb), (nd,         nd = 0, 9)
cbug            write (iout, 8180) abig(nb), (nd,         nd = 0, 9)
cbug            write (   6, 8190) abig(nb), (icount(nd), nd = 0, 9)
cbug            write (iout, 8190) abig(nb), (icount(nd), nd = 0, 9)
cbug  
cbug          endif                       ! More than one digit.
cbug
cbug        endif                         ! Asked for maximum output.
cbugc***DEBUG ends.
  
      enddo                           ! End of loop over big integers.

 8200 format ('  No nonzero big integers exist.')
 8210 format (/ 'Number of nonzero big integers =',i4,'.')

      if ((nb1 .eq. 1) .and. (nb2 .eq. nbigm)) then
        if (nbigs .eq. 0) then
          write (iout, 8200)
          write (   6, 8200)
        else
          write (iout, 8210) nbigs
          write (   6, 8210) nbigs
        endif
      endif

  999 return

c.... End of subroutine bigout.       (+1 line)
      end
