      subroutine copy

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  copy     Updated     2006 January 9 14:00.
c               copy     Originated  1993 November 9 15:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To copy aliases, markers, variables, vectors, operators, points,
c               clusters, lines, triangles, annular disks, regular polygons,
c               planes, spheres, circular cylinders, circular cones, hyperbolic
c               paraboloids, ellipsoids, axisymmetric quadric surfaces, general
c               quadric surfaces, regular polyhedrons, tetrahedrons, bricks,
c               zones, bins, probability distribution functions (pdfs),
c               symbols and big integers, to create a new object.
c
c     Logic:    Entry points:  copy
c copy          Called by:  duplicate*, helpall, main.
c
c copy          Calls:  alout, aptchfp, bigout, binout, brout, clout, dkout,
c               incrsubs, lout, markout, namechk, opout, pdfout, pgout, phout,
c               plout, pout, ptfind, quout, qutype, symbdisp, tetout, trout,
c               varout, vectfind, vout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*80 asrce              ! Acceptable first char of variable name.
      character*24 awordx
      character*24 awordy
      character*1  afirst             ! First character of point name.

c.... Input line:  copy [help]

 8110 format ('  Command "copy":  acceptable input lines include:' /
     &  '  "copy [help]"  Display this message.' //
     &  '  "copy alias    ANAME   ANAME2" ',
     &  '   Copy alias    ANAME   to ANAME2.'/
     &  '  "copy marker   SYNNAME SYNNAME2" ',
     &  ' Copy marker   SYNNAME to SYNNAME2.'/
     &  '  "copy variable VARNAME VARNAME2" ',
     &  ' Copy variable VARNAME to VARNAME2.'/
     &  '  "copy big      BIGNAME to BIGNAM2" ',
     &  ' Copy big integer BIGNAME to BIGNAME2.' /
     &  '  "copy point    PNAME   PNAME2" ',
     &  '   Copy point    PNAME   to PNAME2.'/
     &  '  "copy cluster  CLNAME  CLNAME2"',
     &  '   Copy cluster  CLNAME  to CLNAME2.'/
     &  '  "copy line     ALNAME  ALNAME2"',
     &  '   Copy line     ALNAME  to ALNAME2.'/
     &  '  "copy vector   VNAME   VNAME2" ',
     &  '   Copy vector   VNAME   to VNAME2.'/
     &  '  "copy operator OPNAME  OPNAME2"',
     &  '   Copy operator OPNAME  to OPNAME2.' /
     &  '  "copy triangle TRNAME  TRNAME2"',
     &  '   Copy triangle TRNAME  to TRNAME2.')

 8112 format (
     &  '  "copy polygon  PGNAME  PGNAME2"',
     &  '   Copy regular polygon PGNAME to PGNAME2.'/
     &  '  "copy disk     DKNAME  DKNAME2"',
     &  '   Copy annular disk DKNAME to DKNAME2.'/
     &  '  "copy plane    PLNAME  PLNAME2"',
     &  '   Copy plane    PLNAME  to PLNAME2.' /
     &  '  "copy sphere   SPHNAME SPHNAME2" ',
     &  ' Copy sphere   SPHNAME to SPHNAME2.' /
     &  '  "copy cylinder CYLNAME CYLNAME2" ',
     &  ' Copy cylinder CYLNAME to CYLNAME2.' /
     &  '  "copy cone     CNNAME  CNNAME2"',
     &  '   Copy cone     CNNAME  to CNNAME2.' /
     &  '  "copy ellipsoid ELNAME ELNAME2"',
     &  '   Copy ellipsoid ELNAME to ELNAME2.' /
     &  '  "copy hyperb   HPNAME  HPNAME2"',
     &  '   Copy hyperbolic paraboloid  HPNAME' /
     &  '                                      to HPNAME2.' /
     &  '  "copy axisym   AXNAME  AXNAME2"',
     &  '   Copy axisymmetric quadric AXNAME' /
     &  '                                      to AXNAME2.' /
     &  '  "copy quadric  QNAME   QNAME2"',
     &  '    Copy quadric  QNAME   to QNAME2.')

 8114 format (
     &  '  "copy polyhedron PHNAME PHNAME2"',
     &  '  Copy regular polyhedron PHNAME' /
     &  '                                      to PHNAME2.' /
     &  '  "copy tetrahedron TETNAME TETNAME2"',
     &  ' Copy tetrahedron TETNAME to TETNAME2.'/
     &  '  "copy brick    BRNAME  BRNAME2"',
     &  '   Copy brick    BRNAME  to BRNAME2.'/
     &  '  "copy zone     ZNAME   ZNAME2"',
     &  '    Copy zone     ZNAME   to ZNAME2.' /
     &  '  "copy bin      BINAME  BINAME2"',
     &  '   Copy probability bin BINAME to BINAME2.' /
     &  '  "copy pdf      PDFNAME PDFNAME2"',
     &  '  Copy pdf      PDFNAME to PDFNAME2.' /
     &  '  "copy symbol   SYMBNAME SYMBNAME2"',
     &  '  Copy symbol SYMBNAME to SYMBNAME2.' /
     &  '  "copy OBJTYPE  OBJNAME OBJNAME2"',
     &  '  Copy OBJTYPE  OBJNAME to OBJNAME2,' /
     &  '    where OBJTYPE is any of the object types used above.' )

 8116 format (/
     &  '  Note:  use +/- for OBJNAME2 to incr/decr the last OBJTYPE',
     &  ' name.' //
     &  '  Synonyms:  [alias, al, a], [brick, br],',
     &  ' [copy, cp, duplicate, dup],' /
     &  '    [cluster, cl], [cylinder, cyl], [disk, disc, dk],' /
     &  '    [ellipsoid, ellipse, ell, el], [help, h],',
     &  ' [hyperb, saddle],' /
     &  '    [line, ln, l], [marker, mark, m],' /
     &  '    [operator, oper, op, o], [plane, pl],' /
     &  '    [point, pnt, pt, p], [polygon, polyg, pg],' /
     &  '    [polyhedron, polyh, ph], [quadric, quad, q],',
     &  ' [sphere, sph],' /
     &  '    [symbol, symb, sym], [tetrahedron, tetra, tet],',
     &  ' [triangle, tri],' /
     &  '    [variable, var, set, s], [vector, vect, v],',
     &  ' [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8116)
        write (iout, 8116)
        go to 999
      endif

c.... Test for the wrong number of words on "copy" line.

 8120 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "copy" line:  ', i3 /
     &  '  Usage:  copy OBJTYPE OBJNAME OBJNAME2' )
      if (nword .ne. 4) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c.... Test for the lengths of the geometric object names.

 8125 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of object not from 1 to ',i3,' characters:  ',a)

      lenmax = 24

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'big')    .or.
     &    (aword(2) .eq. 'm'))     then
        lenmax = 1
      endif

      do 125 n = 3, 4
        if ((lword(n) .lt. 1) .or. (lword(n) .gt. lenmax)) then
          write (   6, 8125) ainput, ninput, lenmax, aword(n)
          write (iout, 8125) ainput, ninput, lenmax, aword(n)
          go to 999
        endif
  125 continue

c.... Test to see if geometric object names are different.

 8126 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Source and sink objects have same name:  ',a)

      if (aword(3) .eq. aword(4)) then
        write (   6, 8126) ainput, ninput, aword(3)
        write (iout, 8126) ainput, ninput, aword(3)
        go to 999
      endif

c===============================================================================

c.... Input line:  copy variable VARNAME VARNAME2

      if ((aword(2) .eq. 'variable') .or.
     &    (aword(2) .eq. 'var')      .or.
     &    (aword(2) .eq. 'set')      .or.
     &    (aword(2) .eq. 's'))       then

        nerr = 0

c....   See if original variable exists.

 8140   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named variable does not exist:  ',a)

        do 130 n = 1, nvars
          if (aword(3) .eq. avarbl(n)) then
            n1 = n
            go to 140
          endif
  130   continue
        write (   6, 8140) ainput, ninput, aword(3)
        write (iout, 8140) ainput, ninput, aword(3)
        nerr = 1

c....   See if last variable name should be incremented and used.

 8128   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name of last ',a24,' can not be incremented:  ',a)
 8129   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name of last ',a24,' can not be decremented:  ',a)

  140   if (aword(4) .eq. '+') then
          aword(4) = avarbls
          lword(4) = lavarbls
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), avarbls
            write (iout, 8128) ainput, ninput, aword(2), avarbls
            nerr = 1
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = avarbls
          lword(4) = lavarbls
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), avarbls
            write (iout, 8129) ainput, ninput, aword(2), avarbls
            nerr = 1
          endif
        endif

 8905   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name too dangerous to use:  ',a)

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c.... Test for acceptable variable name.

 8122 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Unacceptable name.  Too dangerous:  ',a)
 8127 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Unacceptable name.  Use command "tol".')
 8132 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name must begin with a-z, A-Z, $, % or @:  ',a)
 8135   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Single-character name must be lower case a to z:  ',a)

      if (mtype(2) .ne. 1) then
        write (   6, 8122) ainput, ninput, aword(4)
        write (iout, 8122) ainput, ninput, aword(4)
        go to 999
      endif

      if ((aword(4) .eq. 'random') .or. (aword(4) .eq. '?')) then
        write (   6, 8122) ainput, ninput, aword(4)
        write (iout, 8122) ainput, ninput, aword(4)
        go to 999
      endif

      if (aword(4) .eq. 'tol') then
        write (   6, 8127) ainput, ninput
        write (iout, 8127) ainput, ninput
        go to 999
      endif

      asrce = '$%@ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

      call aptchfp (asrce, 1, 55, aword(4), 1, ipat, nerra)

      if (ipat .eq. 0) then
        write (   6, 8132) ainput, ninput, aword(4)
        write (iout, 8132) ainput, ninput, aword(4)
        go to 999
      endif
  
      if (lword(4) .eq. 1) then       ! Single-character name must be a-z.

        asrce = 'abcdefghijklmnopqrstuvwxyz'

        call aptchfp (asrce, 1, 26, aword(4), 1, ipat, nerra)

        if (ipat .eq. 0) then
          write (   6, 8135) ainput, ninput, aword(4)
          write (iout, 8135) ainput, ninput, aword(4)
          go to 999
        endif

      endif

 8133 format ('  copy WARNING. Reading ',a40,' line ',i5,'.' /
     &  '  A symbol or its replacement has the same name:  ',a)

      do 133 nsymb = 1, nsymbs
        if ((aword(4) .eq. asymbl(nsymb))  .or.
     &      (aword(4) .eq. asymbr(nsymb))) then
          write (   6, 8133) ainput, ninput, aword(4)
          write (iout, 8133) ainput, ninput, aword(4)
          call symbdisp (nsymb)
        endif
  133 continue

c....   See if new variable exists.

 8150   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not copy over existing variable:  ',a)

        do 150 n = 1, nvars
          if (aword(4) .eq. avarbl(n)) then
            write (   6, 8150) ainput, ninput, aword(4)
            write (iout, 8150) ainput, ninput, aword(4)
            nerr = 1
          endif
  150   continue

 8130   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Cannot exceed maximum number of variables:  ',i3)

        if (nvars .eq. nvarm) then
          write (   6, 8130) nvarm
          write (iout, 8130) nvarm
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

 8131   format ('  Copied variable ',a,' to ',a)

        write (   6, 8131) avarbl(n1), aword(4)
        write (iout, 8131) avarbl(n1), aword(4)

        nvars     = nvars + 1
        n         = nvars
        avarbl(n) = aword(4)
        avarbls   = aword(4)
        lavarbls  = lword(4)
        avar(n)   = avar(n1)
        fvar(n)   = fvar(n1)
        ivar(n)   = ivar(n1)
        lvar(n)   = lvar(n1)
        mvar(n)   = mvar(n1)
        avartag(n) = avartag(n1)

        call varout (n, n)

        go to 999

      endif                           ! Copied one variable.

c===============================================================================

c.... Input line:  copy big BIGNAME BIGNAME2

      if (aword(2) .eq. 'big') then

        nerr = 0

c....   See if original big integer exists.

 8142   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named big integer does not exist:  ',a)

        n1 = 0
        do 132 n = 1, nbigm
          if (aword(3) .eq. abig(n)) then
            n1 = n
            go to 142
          endif
  132   continue
        write (   6, 8142) ainput, ninput, aword(3)
        write (iout, 8142) ainput, ninput, aword(3)
        nerr = 1
  142   continue

c....   See if last big integer name should be incremented and used.

        if (aword(4) .eq. '+') then
          aword(4) = abigs
          call incrsubs (1, aword(4), 1, 1, nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), abigs
            write (iout, 8128) ainput, ninput, aword(2), abigs
            nerr = 1
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = abigs
          call incrsubs (-1, aword(4), 1, 1, nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), abigs
            write (iout, 8129) ainput, ninput, aword(2), abigs
            nerr = 1
          endif
        endif

c....   See if new big integer already exists.

 8153   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  New big integer name already exists:  ',a)

        n2 = 0
        do 153 n = 1, nbigm
          if (aword(4) .eq. abig(n)) then
            n2 = n
            if ((ndig(n2) .gt. 1) .or.
     &         ((ndig(n2) .eq. 1) .and. (idig(1,n2) .gt. 0))) then
              write (   6, 8153) ainput, ninput, aword(4)
              write (iout, 8153) ainput, ninput, aword(4)
              nerr = 1
            endif
            go to 162
          endif
  153   continue
        write (   6, 8142) ainput, ninput, aword(4)
        write (iout, 8142) ainput, ninput, aword(4)
        nerr = 1
  162   continue

        if (nerr .ne. 0) go to 999

 8134   format ('  Copied big integer ',a,' to ',a)

        write (   6, 8134) abig(n1), aword(4)
        write (iout, 8134) abig(n1), aword(4)

        do i = 1, ndig(n1)
          idig(i,n2) = idig(i,n1)
        enddo

        ndig(n2) = ndig(n1)

        abigtag(n2) = abigtag(n1)
        abigs = abig(n2)

        call bigout (n2, n2, 'min')

        go to 999

      endif                           ! Copied big integer.

c===============================================================================

c.... Input line:  copy point PNAME PNAME2

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))     then

c....   See if original point exists.

        call ptfind (3, n1)

        if (n1 .eq. 0) go to 999

c....   See if last point name should be incremented and used.

  170   if (aword(4) .eq. '+') then
          aword(4) = apoints
          lword(4) = lapoints
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apoints
            write (iout, 8128) ainput, ninput, aword(2), apoints
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apoints
          lword(4) = lapoints
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apoints
            write (iout, 8129) ainput, ninput, aword(2), apoints
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

        afirst = aword(4)

        if (afirst .eq. '-') then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new point exists.

 8180 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing point:  ',a)

        do 180 n = 1, npoints
          if (aword(4) .eq. apoint(n)) then
            write (   6, 8180) ainput, ninput, aword(4)
            write (iout, 8180) ainput, ninput, aword(4)
            go to 999
          endif
  180   continue

 8159 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of points:  ',i3)

        if (npoints .eq. npointm) then
          write (   6, 8159) npointm
          write (iout, 8159) npointm
          go to 999
        endif

 8160 format ('  Copied point ',a,' to ',a)

        write (   6, 8160) apoint(n1), aword(4)
        write (iout, 8160) apoint(n1), aword(4)

        npoints   = npoints + 1
        n         = npoints
        apoint(n) = aword(4)
        apoints   = aword(4)
        lapoints  = lword(4)
        x(n)      = x(n1)
        y(n)      = y(n1)
        z(n)      = z(n1)

        call pout (n, n, 'min')

        go to 999

      endif                           ! Copied one point.

c===============================================================================

c.... Input line:  copy cluster CLNAME CLNAME2

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

 8610 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

c....   See if original cluster exists.

        do 610 n = 1, nclusts
          if (aword(3) .eq. aclust(n)) then
            n1 = n
            go to 620
          endif
  610   continue
        write (   6, 8610) ainput, ninput, aword(3)
        write (iout, 8610) ainput, ninput, aword(3)
        go to 999

c....   See if last cluster name should be incremented and used.

  620   if (aword(4) .eq. '+') then
          aword(4) = aclusts
          lword(4) = laclusts
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), aclusts
            write (iout, 8128) ainput, ninput, aword(2), aclusts
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = aclusts
          lword(4) = laclusts
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), aclusts
            write (iout, 8129) ainput, ninput, aword(2), aclusts
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new cluster exists.

 8620 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing cluster:  ',a)

        do 630 n = 1, nclusts
          if (aword(4) .eq. aclust(n)) then
            write (   6, 8620) ainput, ninput, aword(4)
            write (iout, 8620) ainput, ninput, aword(4)
            go to 999
          endif
  630   continue

 8629 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of clusters:  ',i3)

        if (nclusts .eq. nclustm) then
          write (   6, 8629) nclustm
          write (iout, 8629) nclustm
          go to 999
        endif

 8630 format ('  Copied cluster ',a,' to ',a)

        write (   6, 8630) aclust(n1), aword(4)
        write (iout, 8630) aclust(n1), aword(4)

        nclusts    = nclusts + 1
        n          = nclusts
        aclust(n)  = aword(4)
        laclust(n) = lword(4)
        aclusts    = aword(4)
        laclusts   = lword(4)

c....   Copy pairs from old cluster to pairs for new cluster.

 8640 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum cluster pair table length:  ',i4)

        npairx = npairs

        do 640 npr = 1, npairx
          if (apaircl(npr) .eq. aword(3)) then
            npairs = npairs + 1
            if (npairs .gt. npairm) then
              write (   6, 8640) ainput, ninput, npairm
              write (iout, 8640) ainput, ninput, npairm
              aclust(n) = ' '
              nclusts = nclusts - 1
              go to 999
            endif
            apaircl(npairs) = aword(4)
            apairpt(npairs) = apairpt(npr)
          endif
  640   continue

        call clout (n, n, 'min')

        go to 999

      endif                           ! Copied one cluster.

c===============================================================================

c.... Input line:  copy line ALNAME ALNAME2

      if ((aword(2) .eq. 'line') .or.
     &    (aword(2) .eq. 'ln')   .or.
     &    (aword(2) .eq. 'l'))   then

 8250 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)

c....   See if original line exists.

        do 240 n = 1, nlines
          if (aword(3) .eq. aline(n)) then
            n1 = n
            go to 250
          endif
  240   continue
        write (   6, 8250) ainput, ninput, aword(3)
        write (iout, 8250) ainput, ninput, aword(3)
        go to 999

c....   See if last line name should be incremented and used.

  250   if (aword(4) .eq. '+') then
          aword(4) = alines
          lword(4) = lalines
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), alines
            write (iout, 8128) ainput, ninput, aword(2), alines
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = alines
          lword(4) = lalines
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), alines
            write (iout, 8129) ainput, ninput, aword(2), alines
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new line exists.

 8290 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing line:  ',a)

        do 260 n = 1, nlines
          if (aword(4) .eq. aline(n)) then
            write (   6, 8290) ainput, ninput, aword(4)
            write (iout, 8290) ainput, ninput, aword(4)
            go to 999
          endif
  260   continue

 8269 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of lines:  ',i3)

        if (nlines .eq. nlinem) then
          write (   6, 8269) nlinem
          write (iout, 8269) nlinem
          go to 999
        endif

 8270 format ('  Copied line ',a,' to ',a)

        write (   6, 8270) aline(n1), aword(4)
        write (iout, 8270) aline(n1), aword(4)

        nlines     = nlines + 1
        n          = nlines
        aline(n)   = aword(4)
        alipt1(n)  = alipt1(n1)
        alipt2(n)  = alipt2(n1)
        lalipt1(n) = lalipt1(n1)
        lalipt2(n) = lalipt2(n1)

        alines     = aword(4)
        lalines    = lword(4)

        call lout (n, n)

        go to 999

      endif                           ! Copied one line.

c===============================================================================

c.... Input line:  copy triangle TRNAME TRNAME2

      if ((aword(2) .eq. 'triangle') .or.
     &    (aword(2) .eq. 'tri'))   then

 8380 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)

c....   See if original triangle exists.

        do 340 n = 1, ntrigs
          if (aword(3) .eq. atrig(n)) then
            n1 = n
            go to 350
          endif
  340   continue
        write (   6, 8380) ainput, ninput, aword(3)
        write (iout, 8380) ainput, ninput, aword(3)
        go to 999

c....   See if last triangle name should be incremented and used.

  350   if (aword(4) .eq. '+') then
          aword(4) = atrigs
          lword(4) = latrigs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), atrigs
            write (iout, 8128) ainput, ninput, aword(2), atrigs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = atrigs
          lword(4) = latrigs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), atrigs
            write (iout, 8129) ainput, ninput, aword(2), atrigs
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new triangle exists.

 8390 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing triangle:  ',a)

        do 360 n = 1, ntrigs
          if (aword(4) .eq. atrig(n)) then
            write (   6, 8390) ainput, ninput, aword(4)
            write (iout, 8390) ainput, ninput, aword(4)
            go to 999
          endif
  360   continue

 8369 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of triangles:  ',i3)

        if (ntrigs .eq. ntrigm) then
          write (   6, 8369) ntrigm
          write (iout, 8369) ntrigm
          go to 999
        endif

 8370 format ('  Copied triangle ',a,' to ',a)

        write (   6, 8370) atrig(n1), aword(4)
        write (iout, 8370) atrig(n1), aword(4)

        ntrigs      = ntrigs + 1
        n           = ntrigs
        atrig(n)    = aword(4)
        atrigs      = aword(4)
        latrigs     = lword(4)
        atript1(n)  = atript1(n1)
        atript2(n)  = atript2(n1)
        atript3(n)  = atript3(n1)
        latript1(n) = latript1(n1)
        latript2(n) = latript2(n1)
        latript3(n) = latript3(n1)

        call trout (n, n, 'min')

        go to 999

      endif                           ! Copied one triangle.

c===============================================================================

c.... Input line:  copy polygon PGNAME PGNAME2

      if ((aword(2) .eq. 'polygon') .or.
     &    (aword(2) .eq. 'polyg')   .or.
     &    (aword(2) .eq. 'pg'))     then

 8550 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polygon does not exist:  ',a)

c....   See if original polygon exists.

        do 550 n = 1, npolygs
          if (aword(3) .eq. apolyg(n)) then
            n1 = n
            go to 560
          endif
  550   continue
        write (   6, 8550) ainput, ninput, aword(3)
        write (iout, 8550) ainput, ninput, aword(3)
        go to 999

c....   See if last polygon name should be incremented and used.

  560   if (aword(4) .eq. '+') then
          aword(4) = apolygs
          lword(4) = lapolygs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apolygs
            write (iout, 8128) ainput, ninput, aword(2), apolygs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apolygs
          lword(4) = lapolygs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apolygs
            write (iout, 8129) ainput, ninput, aword(2), apolygs
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new polygon exists.

 8560 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing polygon:  ',a)

        do 570 n = 1, npolygs
          if (aword(4) .eq. apolyg(n)) then
            write (   6, 8560) ainput, ninput, aword(4)
            write (iout, 8560) ainput, ninput, aword(4)
            go to 999
          endif
  570   continue

 8569 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of polygons:  ',i3)

        if (npolygs .eq. npolygm) then
          write (   6, 8569) npolygm
          write (iout, 8569) npolygm
          go to 999
        endif

 8570 format ('  Copied polygon ',a,' to ',a)

        write (   6, 8570) apolyg(n1), aword(4)
        write (iout, 8570) apolyg(n1), aword(4)

        npolygs    = npolygs + 1
        n          = npolygs
        apolyg(n)  = aword(4)
        apolygs    = aword(4)
        lapolygs   = lword(4)
        nverpg(n)  = nverpg(n1)
        acenpg(n)  = acenpg(n1)
        averpg(n)  = averpg(n1)
        acoppg(n)  = acoppg(n1)

        call pgout (n, n, 'min')

        go to 999

      endif                           ! Copied one polygon.

c===============================================================================

c.... Input line:  copy disk DKNAME DKNAME2

      if ((aword(2) .eq. 'disk') .or.
     &    (aword(2) .eq. 'disc') .or.
     &    (aword(2) .eq. 'dk'))  then

 8650 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named annular disk does not exist:  ',a)

c....   See if original annular disk exists.

        do 650 n = 1, ndisks
          if (aword(3) .eq. adisk(n)) then
            n1 = n
            go to 660
          endif
  650   continue
        write (   6, 8650) ainput, ninput, aword(3)
        write (iout, 8650) ainput, ninput, aword(3)
        go to 999

c....   See if last annular disk name should be incremented and used.

  660   if (aword(4) .eq. '+') then
          aword(4) = adisks
          lword(4) = ladisks
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), adisks
            write (iout, 8128) ainput, ninput, aword(2), adisks
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = adisks
          lword(4) = ladisks
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), adisks
            write (iout, 8129) ainput, ninput, aword(2), adisks
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new annular disk exists.

 8660 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing annular disk:  ',a)

        do 670 n = 1, ndisks
          if (aword(4) .eq. adisk(n)) then
            write (   6, 8660) ainput, ninput, aword(4)
            write (iout, 8660) ainput, ninput, aword(4)
            go to 999
          endif
  670   continue

 8669 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of disks:  ',i3)

        if (ndisks .eq. ndiskm) then
          write (   6, 8669) ndiskm
          write (iout, 8669) ndiskm
          go to 999
        endif

 8670 format ('  Copied annular disk ',a,' to ',a)

        write (   6, 8670) adisk(n1), aword(4)
        write (iout, 8670) adisk(n1), aword(4)

        ndisks     = ndisks + 1
        n          = ndisks
        adisk(n)   = aword(4)
        adisks     = aword(4)
        ladisks    = lword(4)
        radkin(n)  = radkin(n1)
        radkout(n) = radkout(n1)
        acendk(n)  = acendk(n1)
        xnormdk(n) = xnormdk(n1)
        ynormdk(n) = ynormdk(n1)
        znormdk(n) = znormdk(n1)

        call dkout (n, n)

        go to 999

      endif                           ! Copied one disk.

c===============================================================================

c.... Input line:  copy polyhedron PHNAME PHNAME2

      if ((aword(2) .eq. 'polyhedron') .or.
     &    (aword(2) .eq. 'polyh')   .or.
     &    (aword(2) .eq. 'ph'))     then

 8580 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polyhedron does not exist:  ',a)

c....   See if original polyhedron exists.

        do 580 n = 1, npolyhs
          if (aword(3) .eq. apolyh(n)) then
            n1 = n
            go to 590
          endif
  580   continue
        write (   6, 8580) ainput, ninput, aword(3)
        write (iout, 8580) ainput, ninput, aword(3)
        go to 999

c....   See if last polyhedron name should be incremented and used.

  590   if (aword(4) .eq. '+') then
          aword(4) = apolyhs
          lword(4) = lapolyhs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apolyhs
            write (iout, 8128) ainput, ninput, aword(2), apolyhs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apolyhs
          lword(4) = lapolyhs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apolyhs
            write (iout, 8129) ainput, ninput, aword(2), apolyhs
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new polyhedron exists.

 8590 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing polyhedron:  ',a)

        do 600 n = 1, npolyhs
          if (aword(4) .eq. apolyh(n)) then
            write (   6, 8590) ainput, ninput, aword(4)
            write (iout, 8590) ainput, ninput, aword(4)
            go to 999
          endif
  600   continue

 8599 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of polyhedrons:  ',i3)

        if (npolyhs .eq. npolyhm) then
          write (   6, 8599) npolyhm
          write (iout, 8599) npolyhm
          go to 999
        endif

 8600 format ('  Copied polyhedron ',a,' to ',a)

        write (   6, 8600) apolyh(n1), aword(4)
        write (iout, 8600) apolyh(n1), aword(4)

        npolyhs    = npolyhs + 1
        n          = npolyhs
        apolyh(n)  = aword(4)
        apolyhs    = aword(4)
        lapolyhs   = lword(4)
        atypeh(n)  = atypeh(n1)
        nverph(n)  = nverph(n1)
        nedgph(n)  = nedgph(n1)
        acenph(n)  = acenph(n1)
        averph(n)  = averph(n1)
        acopph(n)  = acopph(n1)

        call phout (n, n, 'min')

        go to 999

      endif                           ! Copied one polyhedron.

c===============================================================================

c.... Input line:  copy tetrahedron TETNAME TETNAME2

      if ((aword(2) .eq. 'tetrahedron') .or.
     &    (aword(2) .eq. 'tetra')       .or.
     &    (aword(2) .eq. 'tet'))        then

 8410 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named tetrahedron does not exist:  ',a)

c....   See if original tetrahedron exists.

        do 410 n = 1, ntets
          if (aword(3) .eq. atetra(n)) then
            n1 = n
            go to 420
          endif
  410   continue
        write (   6, 8410) ainput, ninput, aword(3)
        write (iout, 8410) ainput, ninput, aword(3)
        go to 999

c....   See if last tetrahedron name should be incremented and used.

  420   if (aword(4) .eq. '+') then
          aword(4) = atetras
          lword(4) = latetras
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), atetras
            write (iout, 8128) ainput, ninput, aword(2), atetras
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = atetras
          lword(4) = latetras
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), atetras
            write (iout, 8129) ainput, ninput, aword(2), atetras
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new tetrahedron exists.

 8420 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing tetrahedron:  ',a)

        do 430 n = 1, ntets
          if (aword(4) .eq. atetra(n)) then
            write (   6, 8420) ainput, ninput, aword(4)
            write (iout, 8420) ainput, ninput, aword(4)
            go to 999
          endif
  430   continue

 8429 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of tetrahedrons:  ',i3)

        if (ntets .eq. ntetm) then
          write (   6, 8429) ntetm
          write (iout, 8429) ntetm
          go to 999
        endif

 8430 format ('  Copied tetrahedron ',a,' to ',a)

        write (   6, 8430) atetra(n1), aword(4)
        write (iout, 8430) atetra(n1), aword(4)

        ntets      = ntets + 1
        n          = ntets
        atetra(n)  = aword(4)
        atetras    = aword(4)
        latetras   = lword(4)
        do 435 nv = 1, 4
          atetpt(nv,n)  = atetpt(nv,n1)
          latetpt(nv,n) = latetpt(nv,n1)
  435   continue
        voltet(n)  = voltet(n1)

        call tetout (n, n, 'min')

        go to 999

      endif                           ! Copied one tetrahedron.

c===============================================================================

c.... Input line:  copy brick BRNAME BRNAME2

      if ((aword(2) .eq. 'brick') .or.
     &    (aword(2) .eq. 'br'))   then

 8440 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named brick does not exist:  ',a)

c....   See if original brick exists.

        do 440 n = 1, nbricks
          if (aword(3) .eq. abrick(n)) then
            n1 = n
            go to 460
          endif
  440   continue
        write (   6, 8440) ainput, ninput, aword(3)
        write (iout, 8440) ainput, ninput, aword(3)
        go to 999

c....   See if last brick name should be incremented and used.

  460   if (aword(4) .eq. '+') then
          aword(4) = abricks
          lword(4) = labricks
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), abricks
            write (iout, 8128) ainput, ninput, aword(2), abricks
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = abricks
          lword(4) = labricks
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), abricks
            write (iout, 8129) ainput, ninput, aword(2), abricks
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new brick exists.

 8460 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing brick:  ',a)

        do 470 n = 1, nbricks
          if (aword(4) .eq. abrick(n)) then
            write (   6, 8460) ainput, ninput, aword(4)
            write (iout, 8460) ainput, ninput, aword(4)
            go to 999
          endif
  470   continue

 8469 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of bricks:  ',i3)

        if (nbricks .eq. nbrickm) then
          write (   6, 8469) nbrickm
          write (iout, 8469) nbrickm
          go to 999
        endif

 8470 format ('  Copied brick ',a,' to ',a)

        write (   6, 8470) abrick(n1), aword(4)
        write (iout, 8470) abrick(n1), aword(4)

        nbricks    = nbricks + 1
        n          = nbricks
        abrick(n)  = aword(4)
        abricks    = aword(4)
        labricks   = lword(4)
        asysbr(n)  = asysbr(n1)
        angbr(n)   = angbr(n1)
        ubr(n)     = ubr(n1)
        dubr(n)    = dubr(n1)
        vbr(n)     = vbr(n1)
        dvbr(n)    = dvbr(n1)
        wbr(n)     = wbr(n1)
        dwbr(n)    = dwbr(n1)
        volbr(n)   = volbr(n1)

        call brout (n, n, 'min')

        go to 999

      endif                           ! Copied one brick.

c===============================================================================

c.... Input line:  copy vector VNAME VNAME2

      if ((aword(2) .eq. 'vector') .or.
     &    (aword(2) .eq. 'vect')   .or.
     &    (aword(2) .eq. 'v'))     then

c....   See if original vector exists.

        call vectfind (3, n1)

        if (n1 .eq. 0) go to 999

c....   See if last vector name should be incremented and used.

  220   if (aword(4) .eq. '+') then
          aword(4) = avectors
          lword(4) = lavectors
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), avectors
            write (iout, 8128) ainput, ninput, aword(2), avectors
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = avectors
          lword(4) = lavectors
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), avectors
            write (iout, 8129) ainput, ninput, aword(2), avectors
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new vector exists.

 8230 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing vector:  ',a)

        do 230 n = 1, nvectors
          if (aword(4) .eq. avector(n)) then
            write (   6, 8230) ainput, ninput, aword(4)
            write (iout, 8230) ainput, ninput, aword(4)
            go to 999
          endif
  230   continue

 8209 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of vectors:  ',i3)

        if (nvectors .eq. nvectm) then
          write (   6, 8209) nvectm
          write (iout, 8209) nvectm
          go to 999
        endif

 8210 format ('  Copied vector ',a,' to ',a)

        write (   6, 8210) avector(n1), aword(4)
        write (iout, 8210) avector(n1), aword(4)

        nvectors    = nvectors + 1
        n           = nvectors
        avector(n)  = aword(4)
        lavector(n) = lword(4)
        avectors    = aword(4)
        lavectors   = lword(4)
        xcom(n)     = xcom(n1)
        ycom(n)     = ycom(n1)
        zcom(n)     = zcom(n1)
        xbnd(n)     = xbnd(n1)
        ybnd(n)     = ybnd(n1)
        zbnd(n)     = zbnd(n1)
        vlen(n)     = vlen(n1)

        call vout (n, n)

        go to 999

      endif                           ! Copied one vector.

c===============================================================================

c.... Input line:  copy operator OPNAME OPNAME2

      if ((aword(2) .eq. 'operator') .or.
     &    (aword(2) .eq. 'oper')     .or.
     &    (aword(2) .eq. 'op'))      then  ! Copy an operator.

 8280 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)

c....   See if original operator exists.

        do 270 n = 1, nopers
          if (aword(3) .eq. aoper(n)) then
            n1 = n
            go to 280
          endif
  270   continue
        write (   6, 8280) ainput, ninput, aword(3)
        write (iout, 8280) ainput, ninput, aword(3)
        go to 999

c....   See if last operator name should be incremented and used.

  280   if (aword(4) .eq. '+') then
          aword(4) = aopers
          lword(4) = laopers
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), aopers
            write (iout, 8128) ainput, ninput, aword(2), aopers
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = aopers
          lword(4) = laopers
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), aopers
            write (iout, 8129) ainput, ninput, aword(2), aopers
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new operator exists.

 8320 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing operator:  ',a)

        do 290 n = 1, nopers
          if (aword(4) .eq. aoper(n)) then
            write (   6, 8320) ainput, ninput, aword(4)
            write (iout, 8320) ainput, ninput, aword(4)
            go to 999
          endif
  290   continue

 8299 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of operators:  ',i3)

        if (nopers .eq. noperm) then
          write (   6, 8299) noperm
          write (iout, 8299) noperm
          go to 999
        endif

 8300 format ('  Copied operator ',a,' to ',a)

        write (   6, 8300) aoper(n1), aword(4)
        write (iout, 8300) aoper(n1), aword(4)

        nopers     = nopers + 1
        n          = nopers
        aoper(n)   = aword(4)
        aopers     = aword(4)
        laopers    = lword(4)
        aoptype(n) = aoptype(n1)
        aoptyps(n) = aoptyps(n1)

        do 310 i = 1, 3
          do 300 j = 1, 3
            toper(i,j,n) = toper(i,j,n1)
            topin(i,j,n) = topin(i,j,n1)
  300     continue
  310   continue

        call opout (n, n, 'min')

        go to 999

      endif                           ! Copied one operator.

c===============================================================================

c.... Input line:  copy plane     PLNAME PLNAME2
c.... Input line:  copy sphere    SPHNAME SPHNAME2
c.... Input line:  copy cylinder  CYLNAME CYLNAME2
c.... Input line:  copy cone      CNNAME CNNAME2
c.... Input line:  copy hyperb    HPNAME HPNAME2
c.... Input line:  copy ellipsoid ELNAME ELNAME2
c.... Input line:  copy axisym    AXNAME AXNAME2
c.... Input line:  copy quadric   QNAME QNAME2

      if ((aword(2) .eq. 'quadric')   .or.
     &    (aword(2) .eq. 'quad')      .or.
     &    (aword(2) .eq. 'q')         .or.
     &    (aword(2) .eq. 'plane')     .or.
     &    (aword(2) .eq. 'pl')        .or.
     &    (aword(2) .eq. 'sphere')    .or.
     &    (aword(2) .eq. 'sph')       .or.
     &    (aword(2) .eq. 'cylinder')  .or.
     &    (aword(2) .eq. 'cyl')       .or.
     &    (aword(2) .eq. 'cone')      .or.
     &    (aword(2) .eq. 'hyperb')    .or.
     &    (aword(2) .eq. 'saddle')    .or.
     &    (aword(2) .eq. 'ellipsoid') .or.
     &    (aword(2) .eq. 'ellipse')   .or.
     &    (aword(2) .eq. 'ell')       .or.
     &    (aword(2) .eq. 'el')        .or.
     &    (aword(2) .eq. 'axisym'))   then

 8472 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

c....   See if original quadric exists, and is the right type.

        do 472 n = 1, nquads
          if (aword(3) .eq. aquad(n)) then

            call qutype (aword(2), n, 1, nerrq)

            if (nerrq .ne. 0) go to 999

            n1 = n

            go to 475
          endif
  472   continue
        write (   6, 8472) ainput, ninput, aword(3)
        write (iout, 8472) ainput, ninput, aword(3)
        go to 999

c....   See if last quadric surface name should be incremented and used.

  475   if ((aword(4) .eq. '+')  .or.
     &      (aword(4) .eq. '-')) then

          awordx = aword(4)

          if ((aword(2) .eq. 'quadric')   .or.
     &        (aword(2) .eq. 'quad')      .or.
     &        (aword(2) .eq. 'q'))        then
            aword(4) = aquads
            lword(4) = laquads
          endif

          if ((aword(2) .eq. 'plane')     .or.
     &        (aword(2) .eq. 'pl'))       then
            aword(4) = aplanes
            lword(4) = laplanes
          endif

          if ((aword(2) .eq. 'sphere')    .or.
     &        (aword(2) .eq. 'sph'))      then
            aword(4) = aspheres
            lword(4) = laspheres
          endif

          if ((aword(2) .eq. 'cylinder')  .or.
     &        (aword(2) .eq. 'cyl'))      then
            aword(4) = acyls
            lword(4) = lacyls
          endif

          if ((aword(2) .eq. 'hyperb')    .or.
     &        (aword(2) .eq. 'saddle'))   then
            aword(4) = ahyperbs
            lword(4) = lahyperbs
          endif

          if  (aword(2) .eq. 'cone')      then
            aword(4) = acones
            lword(4) = lacones
          endif

          if ((aword(2) .eq. 'ellipsoid') .or.
     &        (aword(2) .eq. 'ellipse')   .or.
     &        (aword(2) .eq. 'ell')       .or.
     &        (aword(2) .eq. 'el'))       then
            aword(4) = aellips
            lword(4) = laellips
          endif

          if  (aword(2) .eq. 'axisym')    then
            aword(4) = axisyms
            lword(4) = laxisyms
          endif

          if (awordx .eq. '+') then
            awordy = aword(4)
            lwordy = lword(4)
            call incrsubs (1, aword(4), 1, lword(4), nerra)
            if (nerra .ne. 0) then
              write (   6, 8128) ainput, ninput, aword(2), awordy
              write (iout, 8128) ainput, ninput, aword(2), awordy
              go to 999
            endif
          endif

          if (awordx .eq. '-') then
            awordy = aword(4)
            lwordy = lword(4)
            call incrsubs (-1, aword(4), 1, lword(4), nerra)
            if (nerra .ne. 0) then
              write (   6, 8129) ainput, ninput, aword(2), awordy
              write (iout, 8129) ainput, ninput, aword(2), awordy
            endif
          endif

          call namechk (aword(4), lword(4), nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, aword(4)
            write (iout, 8905) ainput, ninput, aword(4)
            go to 999
          endif

        endif

c....   See if new quadric exists.

 8475 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing quadric:  ',a)

        do 480 n = 1, nquads
          if (aword(4) .eq. aquad(n)) then
            write (   6, 8475) ainput, ninput, aword(4)
            write (iout, 8475) ainput, ninput, aword(4)
            go to 999
          endif
  480   continue

 8477 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of quadrics:  ',i3)

        if (nquads .eq. nquadm) then
          write (   6, 8477) nquadm
          write (iout, 8477) nquadm
          go to 999
        endif

 8478 format ('  Copied ',a9,' ',a,' to ',a)

        write (   6, 8478) aword(2), aquad(n1), aword(4)
        write (iout, 8478) aword(2), aquad(n1), aword(4)

        nquads    = nquads + 1
        n         = nquads
        aquads    = aword(4)
        laquads   = lword(4)

        aquad(n)  = aword(4)
        laquad(n) = lword(4)
        aqtype(n) = aqtype(n1)
        aqform(n) = aqform(n1)
        asym(n)   = asym(n1)
        xcen(n)   = xcen(n1)
        ycen(n)   = ycen(n1)
        zcen(n)   = zcen(n1)
        qc(n)     = qc(n1)
        qx(n)     = qx(n1)
        qy(n)     = qy(n1)
        qz(n)     = qz(n1)
        qxy(n)    = qxy(n1)
        qyz(n)    = qyz(n1)
        qzx(n)    = qzx(n1)
        qxx(n)    = qxx(n1)
        qyy(n)    = qyy(n1)
        qzz(n)    = qzz(n1)

        if (asym(n) .eq. 'axisymmetric') then
          axisyms  = aword(4)
          laxisyms = lword(4)
        endif

        if ((aword(2) .eq. 'quadric')   .or.
     &      (aword(2) .eq. 'quad')      .or.
     &      (aword(2) .eq. 'q'))        then
          aquads    = aword(4)
          laquads   = lword(4)
        endif

        if ((aword(2) .eq. 'plane')     .or.
     &      (aword(2) .eq. 'pl'))       then
          aplanes   = aword(4)
          laplanes  = lword(4)
        endif

        if ((aword(2) .eq. 'sphere')    .or.
     &      (aword(2) .eq. 'sph'))      then
          aspheres  = aword(4)
          laspheres = lword(4)
        endif

        if ((aword(2) .eq. 'cylinder')  .or.
     &      (aword(2) .eq. 'cyl'))      then
          acyls     = aword(4)
          lacyls    = lword(4)
        endif

        if  (aword(2) .eq. 'cone')      then
          acones    = aword(4)
          lacones   = lword(4)
        endif

        if ((aword(2) .eq. 'hyperb')    .or.
     &      (aword(2) .eq. 'saddle'))   then
          ahyperbs  = aword(4)
          lahyperbs = lword(4)
        endif

        if ((aword(2) .eq. 'ellipsoid') .or.
     &      (aword(2) .eq. 'ellipse')   .or.
     &      (aword(2) .eq. 'ell')       .or.
     &      (aword(2) .eq. 'el'))       then
          aellips   = aword(4)
          laellips  = lword(4)
        endif

        if  (aword(2) .eq. 'axisym')    then
          axisyms   = aword(4)
          laxisyms  = lword(4)
        endif

        if ((aword(2) .eq. 'plane') .or.
     &      (aword(2) .eq. 'pl'))  then
          call plout (n, n)
        else
          call quout (n, n, 'min', nerrq)
        endif

        go to 999

      endif                           ! Copied one quadric.

c===============================================================================

c.... Input line:  copy zone ZNAME ZNAME2

      if ((aword(2) .eq. 'zone') .or.
     &    (aword(2) .eq. 'zn')   .or.
     &    (aword(2) .eq. 'z'))   then

 8505 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

c....   See if original zone exists.

        do 505 n = 1, nzones
          if (aword(3) .eq. azone(n)) then

            n1 = n

            go to 515
          endif
  505   continue
        write (   6, 8505) ainput, ninput, aword(3)
        write (iout, 8505) ainput, ninput, aword(3)
        go to 999

c....   See if last zone name should be incremented and used.

  515   if (aword(4) .eq. '+') then
          aword(4) = azones
          lword(4) = lazones
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), azones
            write (iout, 8128) ainput, ninput, aword(2), azones
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = azones
          lword(4) = lazones
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), azones
            write (iout, 8129) ainput, ninput, aword(2), azones
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new zone exists.

 8515 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing zone:  ',a)

        do 520 n = 1, nzones
          if (aword(4) .eq. azone(n)) then
            write (   6, 8515) ainput, ninput, aword(4)
            write (iout, 8515) ainput, ninput, aword(4)
            go to 999
          endif
  520   continue

 8529 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of zones:  ',i3)

        if (nzones .eq. nzonem) then
          write (   6, 8529) nzonem
          write (iout, 8529) nzonem
          go to 999
        endif

 8530 format ('  Copied ',a9,' ',a,' to ',a)

        write (   6, 8530) aword(2), azone(n1), aword(4)
        write (iout, 8530) aword(2), azone(n1), aword(4)

        nzones     = nzones + 1
        azones     = aword(4)
        lazones    = lword(4)

        n          = nzones
        azone(n)   = aword(4)
        nzquads(n) = nzquads(n1)
        do 530 nn = 1, nzquads(n)
          azquad(nn,n)  = azquad(nn,n1)
          lazquad(nn,n) = lazquad(nn,n1)
          azsign(nn,n)  = azsign(nn,n1)
  530   continue

        volz(n)    = volz(n1)
        volzerr(n) = volzerr(n1)

        call zout (n, n, 'min')

        go to 999

      endif                           ! Copied one zone.

c===============================================================================

c.... Input line:  copy bin BINAME BINAME2

      if (aword(2) .eq. 'bin') then

 8720 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named probability bin does not exist:  ',a)

c....   See if original bin exists.

        do 710 n = 1, nbins
          if (aword(3) .eq. abin(n)) then
            n1 = n
            go to 720
          endif
  710   continue
        write (   6, 8720) ainput, ninput, aword(3)
        write (iout, 8720) ainput, ninput, aword(3)
        go to 999

c....   See if last probability bin name should be incremented and used.

  720   if (aword(4) .eq. '+') then
          aword(4) = abins
          lword(4) = labins
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), abins
            write (iout, 8128) ainput, ninput, aword(2), abins
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = abins
          lword(4) = labins
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), abins
            write (iout, 8129) ainput, ninput, aword(2), abins
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new bin exists.

 8730 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing bin:  ',a)

        do 730 n = 1, nbins
          if (aword(4) .eq. abin(n)) then
            write (   6, 8730) ainput, ninput, aword(4)
            write (iout, 8730) ainput, ninput, aword(4)
            go to 999
          endif
  730   continue

 8709 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of bins:  ',i3)

        if (nbins .eq. nbinm) then
          write (   6, 8709) nbinm
          write (iout, 8709) nbinm
          go to 999
        endif

 8710 format ('  Copied bin ',a,' to ',a)

        write (   6, 8710) abin(n1), aword(4)
        write (iout, 8710) abin(n1), aword(4)

        nbins     = nbins + 1
        n         = nbins
        abin(n)   = aword(4)
        abins     = aword(4)
        labins    = lword(4)
        atypeb(n) = atypeb(n1)
        probl(n)  = probl(n1)
        probr(n)  = probr(n1)
        vranl(n)  = vranl(n1)
        vranr(n)  = vranr(n1)
        prbin(n)  = prbin(n1)
        prbint(n) = prbint(n1)
        conbin(n) = conbin(n1)
        cosbin(n) = cosbin(n1)
        vexbin(n) = vexbin(n1)

        call binout (n, n, 'min')

        go to 999

      endif                           ! Copied one bin.

c===============================================================================

c.... Input line:  copy pdf PDFNAME PDFNAME2

      if (aword(2) .eq. 'pdf') then

 8740 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named pdf does not exist:  ',a)

c....   See if original pdf exists.

        do 740 n = 1, npdfs
          if (aword(3) .eq. apdf(n)) then
            n1 = n
            go to 750
          endif
  740   continue
        write (   6, 8740) ainput, ninput, aword(3)
        write (iout, 8740) ainput, ninput, aword(3)
        go to 999

c....   See if last pdf name should be incremented and used.

  750   if (aword(4) .eq. '+') then
          aword(4) = apdfs
          lword(4) = lapdfs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apdfs
            write (iout, 8128) ainput, ninput, aword(2), apdfs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apdfs
          lword(4) = lapdfs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apdfs
            write (iout, 8129) ainput, ninput, aword(2), apdfs
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new pdf exists.

 8750 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing pdf:  ',a)

        do 760 n = 1, npdfs
          if (aword(4) .eq. apdf(n)) then
            write (   6, 8750) ainput, ninput, aword(4)
            write (iout, 8750) ainput, ninput, aword(4)
            go to 999
          endif
  760   continue

 8760 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of pdfs:  ',i3)

        if (npdfs .eq. npdfm) then
          write (   6, 8760) npdfm
          write (iout, 8760) npdfm
          go to 999
        endif

 8765 format ('  Copied pdf ',a,' to ',a)

        write (   6, 8765) apdf(n1), aword(4)
        write (iout, 8765) apdf(n1), aword(4)

        npdfs     = npdfs + 1
        n         = npdfs
        apdf(n)   = aword(4)
        apdfs     = aword(4)
        lapdfs    = lword(4)
        prpdf(n)  = prpdf(n1)
        prpdft(n) = prpdft(n1)
        vexpdf(n) = vexpdf(n1)

c....   Copy pdf/bin pairs from old pdf to pdf/bin pairs for new pdf.

 8770 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum pdf/bin pair table length:  ',i4)

        npdfxx = npdfxs

        do 770 npx = 1, npdfxx
          if (apdfx(npx) .eq. aword(3)) then
            npdfxs = npdfxs + 1
            if (npdfxs .gt. npdfxm) then
              write (   6, 8770) ainput, ninput, npdfxm
              write (iout, 8770) ainput, ninput, npdfxm
              apdf(n) = ' '
              npdfxs = npdfxs - 1
              go to 999
            endif
            apdfx(npdfxs) = aword(4)
            abinx(npdfxs) = abinx(npx)
          endif
  770   continue

        call pdfout (n, n, 'min')

        go to 999

      endif                           ! Copied one pdf.

c===============================================================================

c.... Input line:  copy alias ANAME ANAME2

      if ((aword(2) .eq. 'alias') .or.
     &    (aword(2) .eq. 'al')    .or.
     &    (aword(2) .eq. 'a'))    then

 8810 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named alias does not exist:  ',a)

c....   See if original alias exists.

        do 810 n = 1, nalias
          if (aword(3) .eq. alias1(n)) then
            n1 = n
            go to 820
          endif
  810   continue
        write (   6, 8810) ainput, ninput, aword(3)
        write (iout, 8810) ainput, ninput, aword(3)
        go to 999

c....   See if last alias name should be incremented and used.

  820   if (aword(4) .eq. '+') then
          aword(4) = alias1s
          lword(4) = lalias1s
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), alias1s
            write (iout, 8128) ainput, ninput, aword(2), alias1s
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = alias1s
          lword(4) = lalias1s
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), alias1s
            write (iout, 8129) ainput, ninput, aword(2), alias1s
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new alias exists.

 8820 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing alias:  ',a)

        do 830 n = 1, nalias
          if (aword(4) .eq. alias1(n)) then
            write (   6, 8820) ainput, ninput, aword(4)
            write (iout, 8820) ainput, ninput, aword(4)
            go to 999
          endif
  830   continue

 8830 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of aliases:  ',i3)

        if (nalias .eq. naliasm) then
          write (   6, 8830) naliasm
          write (iout, 8830) naliasm
          go to 999
        endif

 8835 format ('  Copied alias ',a,' to ',a)

        write (   6, 8835) alias1(n1), aword(4)
        write (iout, 8835) alias1(n1), aword(4)

        nalias     = nalias + 1
        n          = nalias
        alias1(n)  = aword(4)
        alias2(n)  = alias2(n1)
        lalias2(n) = lalias2(n1)

        alias1s    = aword(4)
        lalias1s   = lword(4)

        call alout (n, n)

        go to 999

      endif                           ! Copied one alias.

c===============================================================================

c.... Input line:  copy marker AMNAME AMNAME2

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then

 8840 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named marker does not exist:  ',a1)

c....   See if original marker exists.

        do 840 n = 1, nmarks
          if (aword(3) .eq. amark(n)) then
            n1 = n
            go to 850
          endif
  840   continue
        write (   6, 8840) ainput, ninput, aword(3)
        write (iout, 8840) ainput, ninput, aword(3)
        go to 999

c....   See if last marker name should be incremented and used.

  850   if (aword(4) .eq. '+') then
          aword(4) = amarks
          lword(4) = lamarks
cbugc***DEBUG begins.
cbug 8701 format ('aword=',a24,'  lword = ',i5)
cbug      write (   6, 8701) aword(4), lword(4)
cbug      write (iout, 8701) aword(4), lword(4)
cbugc***DEBUG ends.
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), amarks
            write (iout, 8128) ainput, ninput, aword(2), amarks
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = amarks
          lword(4) = lamarks
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), amarks
            write (iout, 8129) ainput, ninput, aword(2), amarks
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new marker exists.

 8850 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not copy over existing marker:  ',a1)

        do 860 n = 1, nmarks
          if (aword(4) .eq. amark(n)) then
            write (   6, 8850) ainput, ninput, aword(4)
            write (iout, 8850) ainput, ninput, aword(4)
            go to 999
          endif
  860   continue

 8860 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot exceed maximum number of markers:  ',i3)

        if (nmarks .eq. nmarkm) then
          write (   6, 8860) nmarkm
          write (iout, 8860) nmarkm
          go to 999
        endif

 8870 format ('  Copied marker ',a1,' to ',a1)

        write (   6, 8870) amark(n1), aword(4)
        write (iout, 8870) amark(n1), aword(4)

        nmarks    = nmarks + 1
        n         = nmarks
        amark(n)  = aword(4)
        amarks    = aword(4)
        lamarks   = lword(4)
        amarkt(n) = amarkt(n1)

        call markout (n, n)

        go to 999

      endif                           ! Copied one marker.

c===============================================================================

c.... Input line:  copy symbol SYMBNAME SYMBNAME2

      if ((aword(2) .eq. 'symbol') .or.
     &    (aword(2) .eq. 'symb')   .or.
     &    (aword(2) .eq. 'sym'))   then

        nerr = 0

c....   See if original symbol exists.

 8785   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named symbol does not exist:  ',a)

        do 780 n = 1, nsymbs
          if (aword(3) .eq. asymbl(n)) then
            n1 = n
            go to 785
          endif
  780   continue
        write (   6, 8785) ainput, ninput, aword(3)
        write (iout, 8785) ainput, ninput, aword(3)
        nerr = 1

c....   See if last symbol name should be incremented and used.

  785   if (aword(4) .eq. '+') then
          aword(4) = asymbls
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), asymbls
            write (iout, 8128) ainput, ninput, aword(2), asymbls
            nerr = 1
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = asymbls
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), asymbls
            write (iout, 8129) ainput, ninput, aword(2), asymbls
            nerr = 1
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new symbol exists.

 8790   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not copy over existing symbol:  ',a)

        do 790 n = 1, nsymbs
          if (aword(4) .eq. asymbl(n)) then
            write (   6, 8790) ainput, ninput, aword(4)
            write (iout, 8790) ainput, ninput, aword(4)
            nerr = 1
          endif
  790   continue

 8792   format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Cannot exceed maximum number of symbols:  ',i3)

        if (nsymbs .eq. nsymbm) then
          write (   6, 8792) nsymbm
          write (iout, 8792) nsymbm
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

 8780   format ('  Copied symbol ',a,' to ',a)

        write (   6, 8780) asymbl(n1), aword(4)
        write (iout, 8780) asymbl(n1), aword(4)

        nsymbs    = nsymbs + 1
        n         = nsymbs
        asymbl(n) = aword(4)
        asymbls   = aword(4)
        asymbr(n) = asymbr(n1)
        fsymb(n)  = fsymb(n1)
        isymb(n)  = isymb(n1)
        lsymb(n)  = lsymb(n1)
        msymb(n)  = msymb(n1)

        call symbdisp (n)

        go to 999

      endif                           ! Copied one symbol.

c===============================================================================

c.... Input line:  (not recognized)

 8880 format ('  copy ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Specified object type does not exist:  ',a24)

      write (   6, 8880) ainput, ninput, aword(2)
      write (iout, 8880) ainput, ninput, aword(2)

  999 return

c.... End of subroutine copy.         (+1 line)
      end
