      subroutine cross

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  cross    Updated     1994 August 30 17:00.
c               cross    Originated  1993 November 10 17:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To interpret data on "cross" input lines, to find the cross
c               (outer) product of two vectors in 3-D space.
c
c     Logic:    Entry points:  cross.
c cross         Called by:  helpall, main.
c
c cross         Calls:  aptvaxb, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input line:  cross [help]

 8110 format ('  Command "cross":  acceptable input lines include:' /
     &  '  "cross [help]"               Display this message.' //
     &  '  "cross VNAME1 VNAME2"        Find the cross product' /
     &  '     of vectors VNAME1 and VNAME2.' //
     &  '  Synonyms:  [cross, outer], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Test for wrong number of words on "cross" line.

 8120 format ('  cross ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 3 words on "cross" line:  ', i3)
      if (nword .ne. 3) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

c.... Find the components of the first vector.

      call vectfind (2, n1)

      if (n1 .eq. 0) go to 999

c.... Find the components of the second vector.

      call vectfind (3, n2)

      if (n2 .eq. 0) go to 999

c=======================================================================********

c.... Input line:  cross VNAME1 VNAME2

c.... Find the cross product of the two vectors.

      call aptvaxb (xcom(n1), ycom(n1), zcom(n1),
     &              xcom(n2), ycom(n2), zcom(n2), 1, tol,
     &              dx, dy, dz, vlength, nerr)

c.... There is no bound point, so can not define the vector components in
c....   other than rectangular coordinates.

 8160 format ('cross dx, dy, dz  ',1p3e20.12 /
     &        'length (2*area)   ',1pe20.12)
      write (   6, 8160) dx, dy, dz, vlength
      write (iout, 8160) dx, dy, dz, vlength

c=======================================================================********

  999 return

c.... End of subroutine cross.        (+1 line)
      end
