      subroutine crt

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  crt      Updated     2004 April 9 15:20.
c               crt      Originated  2003 August 3 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To solve the Chinese Remainder Theorem Problem:  given the
c               remainders of divisions of an unknown integer by several
c               relatively prime divisors, find the smallest possible value
c               of the unknown integer, and the next three higher solutions.
c               Conversely, given an integer, find the remainders for
c               divisions by sufficient prime numbers to make the integer
c               the smallest solution.
c
c     Logic:    Entry points:  crt.
c crt           Called by:  helpall, main.
c
c crt           Calls:  aptcrtr, aptcrts, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /icrt/ idiv(50)          ! Divisors.  
      common /icrt/ irem(50)          ! Remainders.
      common /icrt/ iceof(50)         ! Coefficient iprod / idiv.
      common /icrt/ imult(50)         ! Multiplier.
      common /icrt/ iprod             ! Product of all divisors.
      common /icrt/ ival(4)           ! First four solutions.

c=======================================================================********

c.... Input line:  crt [help]

 8110 format ('  Command "crt":  acceptable input lines include:' /
     &  '  "crt [help]"           Display this message.' //
     &  '  "crt IDIV1 IREM1 IDIV2 IREM2 IDIV3 IREM3 ...',
     &  ' IDIVN IRMIN"' /
     &  '     Find the least integer for which divisions by IDIVn' /
     &  '     have the remainders IDIVn, n = 1, N.' //
     &  '  "crt = IVALUE"' /
     &  '     For the integer IVALUE, find the remainders of divisions'/
     &  '     by the first few prime numbers.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  crt IDIV1 IREM1 IDIV2 IREM2 IDIV3 IREM3 ... IDIVN IREMN

 8120 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer:  ',a)

      if ((aword(2) .ne. '=' ) .and. (mod (nword, 2) .eq. 1)) then

c....   Find the values of all arguments.

        call varfind (2, nword)

        nerr = 0

c....   Verify that all arguments are integer.

        do 130 nw = 2, nword
          if (mtype(nw) .ne. 2) then
            nerr = 1
            write (   6, 8120) ainput, ninput, aword(nw)
            write (iout, 8120) ainput, ninput, aword(nw)
          endif
  130  continue

        if (nerr .ne. 0) go to 999

c....   Store the divisors and remainders.

 8125 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A divisor is less than 2:  ',i15)
 8130 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A remainder is less than 0:  ',i15)
 8140 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A divisor does not exceed its remainder:  ',2i15)

        nd = 0
        do 140 nw = 2, nword, 2

          if (iword(nw) .lt. 2) then
            nerr = 1
            write (   6, 8125) ainput, ninput, iword(nw)
            write (iout, 8125) ainput, ninput, iword(nw)
          endif

          if (iword(nw+1) .lt. 0) then
            nerr = 1
            write (   6, 8130) ainput, ninput, iword(nw+1)
            write (iout, 8130) ainput, ninput, iword(nw+1)
          endif

          if (iword(nw+1) .ge. iword(nw)) then
            nerr = 1
            write (   6, 8140) ainput, ninput, iword(nw), iword(nw+1)
            write (iout, 8140) ainput, ninput, iword(nw), iword(nw+1)
          endif

          nd       = nd + 1
          idiv(nd) = iword(nw)
          irem(nd) = iword(nw+1)
  140   continue

 8150 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Fewer than 2 divisors:  ',i3)

        if (nd .lt. 2) then
          nerr = 1
          write (   6, 8150) ainput, ninput, nd
          write (iout, 8150) ainput, ninput, nd
        endif

 8180 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Two divisors have common factors:  ',2i18)

        imax = idiv(1)
        do 150 n = 2, nd
          if (idiv(n) .gt. imax) imax = idiv(n)
  150   continue

        do 180 n = 2, nd
          do 170 nn = 1, n - 1
            do 160 nnn  = 2, imax
              if ((mod (idiv(n),  nnn) .eq. 0)  .and.
     &            (mod (idiv(nn), nnn) .eq. 0)) then
                nerr = 1
                write (   6, 8180) ainput, ninput, idiv(nn), idiv(n)
                write (iout, 8180) ainput, ninput, idiv(nn), idiv(n)
              endif
  160       continue
  170     continue
  180   continue

        if (nerr .ne. 0) go to 999

c....   Find the integer satisfying the conditions.

        call aptcrts (idiv, irem, nd, iprod, icoef, imult, ival, nerra)

c....   Test for errors.

 8190 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  ERROR in subroutine aptcrts:  ',i3)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8190) ainput, ninput, nerra
          write (iout, 8190) ainput, ninput, nerra
          go to 999
        endif

c....   Generate more solutions.

        nstot = 1
        do 210 ns = 2, 4
          fval     = ival(ns-1)
          fprod    = iprod
          fval     = fval + fprod
          if (fval .lt. 1.e18) then
            nstot    = nstot + 1
            ival(ns) = ival(ns-1) + iprod
          endif
  210   continue

 8210 format ('var    ival(1)          = ',i20,2x,'! (integer)')
 8215 format ('var    ival(',i1,')          = ',i20,2x,'! (integer)')

        write (   6, 8210) ival(1)
        write (iout, 8210) ival(1)
        write (   6, 8215) (ns, ival(ns), ns = 2, nstot)
        write (iout, 8215) (ns, ival(ns), ns = 2, nstot)

        go to 999

      endif                           ! Find an unknown integer.

c=======================================================================********

c.... Input line:  crt = IVAL

      if ((aword(2) .eq. '=') .and. (nword .eq. 3)) then

c....   Find the integer.

        call varfind (3, 3)

        if (mtype(3) .ne. 2) then
          nerr = 1
          write (   6, 8120) ainput, ninput, aword(3)
          write (iout, 8120) ainput, ninput, aword(3)
          go to 999
        endif

 8220 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  The integer is less than 2:  ',i15)

        if (iword(3) .lt. 2) then
          nerr = 1
          write (   6, 8220) ainput, ninput, iword(3)
          write (iout, 8220) ainput, ninput, iword(3)
          go to 999
        endif

c....   Find the remainders from division be the first few primes.

        call aptcrtr (iword(3), idiv, irem, nd, nerra)

 8240 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  ERROR in subroutine aptcrtr:  ',i3)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8240) ainput, ninput, nerra
          write (iout, 8240) ainput, ninput, nerra
          go to 999
        endif

 8245 format ('   ','  I  R  I  R  I  R  I  R  I  R ...')
 8250 format ('crt',26i3)

        write (   6, 8245)
        write (iout, 8245)
        write (   6, 8250) (idiv(n), irem(n), n = 1, nd)
        write (iout, 8250) (idiv(n), irem(n), n = 1, nd)

        go to 999

      endif                           ! Find remainders.

c=======================================================================********

c.... Input line:  crt = IVAL IDIV1 IDIV2 ... IDIVn

      if ((aword(2) .eq. '=') .and. (nword .gt. 3)) then

c....   Find the values of all arguments.

        call varfind (3, nword)

        nerr = 0

c....   Verify that all arguments are integer.

        do 330 nw = 3, nword
          if (mtype(nw) .ne. 2) then
            nerr = 1
            write (   6, 8120) ainput, ninput, aword(nw)
            write (iout, 8120) ainput, ninput, aword(nw)
          endif
  330  continue

        if (nerr .ne. 0) go to 999

c....   Store the divisors.

        nd = 0
        do 340 nw = 4, nword

          if (iword(nw) .lt. 2) then
            nerr = 1
            write (   6, 8125) ainput, ninput, iword(nw)
            write (iout, 8125) ainput, ninput, iword(nw)
          endif

          nd       = nd + 1
          idiv(nd) = iword(nw)
  340   continue

        if (nd .lt. 2) then
          nerr = 1
          write (   6, 8150) ainput, ninput, nd
          write (iout, 8150) ainput, ninput, nd
        endif

        imax = idiv(1)
        do 350 n = 2, nd
          if (idiv(n) .gt. imax) imax = idiv(n)
  350   continue

        do 380 n = 2, nd
          do 370 nn = 1, n - 1
            do 360 nnn  = 2, imax
              if ((mod (idiv(n),  nnn) .eq. 0)  .and.
     &            (mod (idiv(nn), nnn) .eq. 0)) then
                nerr = 1
                write (   6, 8180) ainput, ninput, idiv(nn), idiv(n)
                write (iout, 8180) ainput, ninput, idiv(nn), idiv(n)
              endif
  360       continue
  370     continue
  380   continue

        if (nerr .ne. 0) go to 999

c....   Find remainders.

        do 390 n = 1, nd
          irem(n) = mod (iword(3), idiv(n))
  390   continue

        write (   6, 8250) (idiv(n), irem(n), n = 1, nd)
        write (iout, 8250) (idiv(n), irem(n), n = 1, nd)

        go to 999

      endif                           ! Find remainders of specified divisors.

c=======================================================================********

 8810 format ('  crt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine crt.        (+1 line)
      end
