      subroutine cute

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  cute     Updated     2006 April 10 18:30.
c               cute     Originated  1999 August 30 12:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c
c     Purpose:  For a triangle with edge lengths a, b and c, to extend edges
c               a and b to add a new section with the same perimeter and area
c               as the original triangle.  There may be 0, 1 or 2 ways to do so.
c
c     Logic:    Entry points:  cute.
c cute          Called by:  helpall, main.
c
c cute          Calls:  aptrext, aptrich, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      dimension    fa(2)              ! Ratio a / aa.
      dimension    fb(2)              ! Ratio b / bb.
      dimension    fc(2)              ! Ratio c / cc.
      dimension    aa(2)              ! Length of new side aa = a / fa.
      dimension    bb(2)              ! Length of new side bb = b / fb.
      dimension    cc(2)              ! Length of new side cc = c / fc.

c=======================================================================********

c.... Input line:  cute [help]

 8110 format ('  Command "cute":  acceptable input lines include:' /
     &  '  "cute [help]"           Display this message.' //
     &  '  "cute VA VB VC"' /
     &  '   For the triangle with edges VA, VB and VC, extend sides',
     &  ' VA and VB' /
     &  '     to make the perimeter 2*(VA + VB), and double the area.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  cute VA VB VC

 8010 format ('  cute ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "cute" line.' )

      nerr = 0

      if (nword .ne. 4) then
        write (   6, 8010) ainput, ninput
        write (iout, 8010) ainput, ninput
        nerr = 1
      endif

c.... Find lengths of small triangle edges.

      call varfind (2, nword)

c.... Verify that all are integer or floating point.

 8130 format ('  cute ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      do 110 nw = 2, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  110 continue

      a = fword(2)
      b = fword(3)
      c = fword(4)

 8128 format ('  Edge lengths abc',1p3e20.12 )

      write (   6, 8128) a, b, c
      write (iout, 8128) a, b, c

c.... Test for a good triangle.

      call aptrich (a, b, c, tol, nerra)

 8115 format ('Not a triangle.  Edge(s) too short or negative.')
 8116 format ('Not a triangle.  Linear or edge(s) too long.')

      if (nerra .eq. 1) then
        write (   6, 8115)
        write (iout, 8115)
        nerr = 1
      endif

      if (nerra .eq. 2) then
        write (   6, 8116)
        write (iout, 8116)
        nerr = 2
      endif

      if (nerr .ne. 0) go to 999

c.... Find each extended triangle.

      nextot = 0

      call aptrext (a, b, c, tol, next, fa, aa, fb, bb, fc, cc, nerr)

 8205 format ('Extend sides a and b.')
 8210 format ('  fa, fb, fc   =  ',1p3e20.12 /
     &        '  aa, bb, cc   =  ',1p3e20.12 )

      if (next .gt. 0) then
        do 440 n = 1, next
          write (   6, 8205)
          write (iout, 8205)
          write (   6, 8210) fa(n), fb(n), fc(n), aa(n), bb(n), cc(n)
          write (iout, 8210) fa(n), fb(n), fc(n), aa(n), bb(n), cc(n)
  440   continue
      endif

      nextot = nextot + next
 
      call aptrext (b, c, a, tol, next, fb, bb, fc, cc, fa, aa, nerr)

 8215 format ('Extend sides b and c.')
 8220 format ('  fb, fc, fa   =  ',1p3e20.12 /
     &        '  bb, cc, aa   =  ',1p3e20.12 )

      if (next .gt. 0) then
        do 450 n = 1, next
          write (   6, 8215)
          write (iout, 8215)
          write (   6, 8220) fb(n), fc(n), fa(n), bb(n), cc(n), aa(n)
          write (iout, 8220) fb(n), fc(n), fa(n), bb(n), cc(n), aa(n)
  450   continue
      endif

      nextot = nextot + next
 
      call aptrext (c, a, b, tol, next, fc, cc, fa, aa, fb, bb, nerr)

 8225 format ('Extend sides c and a.')
 8230 format ('  fc, fa, fb   =  ',1p3e20.12 /
     &        '  cc, aa, bb   =  ',1p3e20.12 )

      if (next .gt. 0) then
        do 460 n = 1, next
          write (   6, 8225)
          write (iout, 8225)
          write (   6, 8230) fc(n), fa(n), fb(n), cc(n), aa(n), bb(n)
          write (iout, 8230) fc(n), fa(n), fb(n), cc(n), aa(n), bb(n)
  460   continue
      endif

      nextot = nextot + next

 8240 format ('No sides can be extended.')

      if (nextot .eq. 0) then
        write (   6, 8240)
        write (iout, 8240)
      endif

c=======================================================================********

  999 return

c.... End of subroutine cute.       (+1 line)
      end
