      subroutine disk

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  disk     Updated     2005 April 5 17:45.
c               disk     Originated  1993 November 17 18:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create annular disks by specifying a center
c               point, a normal vector, an inner radius, and an outer radius.
c
c     Logic:    Entry points:  disk.
c disk          Called by:  helpall, main.
c
c disk          Calls:  aptsubr, dkout, incrsubs, namechk, ptfind, varfind,
c               vectfind,
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input line:  disk [help]

 8110 format (
     &  '  Command "disk":  acceptable input lines include:' /
     &  '  "disk [help]"  Display this message.' /
     &  '  "disk all"     Display all annular disks.' /
     &  '  "disk list [DKNAME1 DKNAME2 DKNAME3 ...]"' /
     &  '     Display all listed annular disks (default is all',
     &  ' disks).' /
     &  '  "disk DKNAME"  Display annular disk DKNAME.' //
     &  '  "disk DKNAME fit PCEN VNORM RADKIN RADKOUT"' /
     &  '     Create annular disk DKNAME, centered at point PCEN,' /
     &  '     with normal vector VNORM, inner radius RADKIN, outer',
     &  ' radius RADKOUT.' )
 8112 format (/
     &  '  Note:  use +/- for DKNAME to incr/decr the last',
     &  ' disk name:  ',a //
     &  '  Synonyms:  [disk, disc, dk], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112) adisks
        write (iout, 8112) adisks
        go to 999
      endif

c===============================================================================

c.... Input line:  disk all
c.... Input line:  disk list

 8120 format (/ 'Number of disks = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        ndiskx = 0
        do 120 n = 1, ndisks
          if (adisk(n) .eq. ' ') go to 120
          call dkout (n, n)
          ndiskx = ndiskx + 1
  120   continue
        write (   6, 8120) ndiskx
        write (iout, 8120) ndiskx
        go to 999
      endif

c===============================================================================

c.... Input line:  disk list DKNAME1 DKNAME2 DKNAME3 ...

 8130 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named disk does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, ndisks
            call aptsubr (adisk(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. adisk(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call dkout (n, n)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  disk DKNAME

      if (nword .eq. 2) then
        do 130 n = 1, ndisks
          if (aword(2) .eq. adisk(n)) then
            call dkout (n, n)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8135 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Wrong number of words on "disk" line:  ',i3)

 8136 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of disk not from 1 to 24 characters:  ',a)

      if (nword .ne. 7) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        go to 999
      endif

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

 8137 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      if (aword(3) .ne. 'fit') then
        write (   6, 8137) ainput, ninput
        write (iout, 8137) ainput, ninput
        go to 999
      endif

c===============================================================================

c.... See if last annular disk name should be incremented and used.

 8128 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last annular disk can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = adisks
        lword(2) = ladisks
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, adisks
          write (iout, 8128) ainput, ninput, adisks
          go to 999
        endif
      endif

 8129 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last annular disk can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = adisks
        lword(2) = ladisks
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, adisks
          write (iout, 8129) ainput, ninput, adisks
          go to 999
        endif
      endif

 8905 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of annular disk too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if a disk with the same name exists.

 8160 format ('  WARNING:  replacing existing disk:  ',a)

      nrep = 0
      do 136 n = 1, ndisks
        if (aword(2) .eq. adisk(n)) then
          nrep = 1
          write (   6, 8160) adisk(n)
          write (iout, 8160) adisk(n)
          call dkout (n, n)
          go to 138
        endif
  136 continue

      do 137 n = 1, ndisks
        if (adisk(n) .eq. ' ') go to 138
  137 continue

      ndisks = ndisks + 1

 8165 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of disks exceeds limit ',i5)

      if (ndisks .gt. ndiskm) then
        ndisks = ndiskm
        write (   6, 8165) ainput, ninput, ndiskm
        write (iout, 8165) ainput, ninput, ndiskm
        go to 999
      endif

      n = ndisks

  138 nerr = 0

c===============================================================================

c.... Input line:  disk DKNAME fit PCEN VNORM RADKIN RADKOUT

c.... Find the inner and outer radii of the annular disk.

 8140 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 6 not integer or floating point value or variable:  ',a)
 8145 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 7 not integer or floating point value or variable:  ',a)

      call varfind (6, 7)

      if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
        nerr = 1
        write (   6, 8140) ainput, ninput, aword(6)
        write (iout, 8140) ainput, ninput, aword(6)
      endif

      if ((mtype(7) .ne. 2) .and. (mtype(7) .ne. 3)) then
        nerr = 1
        write (   6, 8145) ainput, ninput, aword(7)
        write (iout, 8145) ainput, ninput, aword(7)
      endif

c.... Find the central point of the annular disk.

      call ptfind (4, np)

      if (np .eq. 0) nerr = 1

c.... Find the vector normal to the annular disk.

      call vectfind (5, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 250
      endif

 8240 format ('  disk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Normal vector has zero length:  ',a)

      if (vlen(nv) .eq. 0.0) then
        nerr = 1
        write (   6, 8240) ainput, ninput, aword(5)
        write (iout, 8240) ainput, ninput, aword(5)
      endif

  250 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          adisk(n) = ' '
          if (n .eq. ndisks) ndisks = ndisks - 1
        endif
        go to 999
      endif

c.... Find the data for the annular disk.

      adisk(n)   = aword(2)
      adisks     = aword(2)
      ladisks    = lword(2)
      radkin(n)  = fword(6)
      radkout(n) = fword(7)
      acendk(n)  = apoint(np)

      xnormdk(n) = xcom(nv)
      ynormdk(n) = ycom(nv)
      znormdk(n) = zcom(nv)

      call dkout (n, n)

      go to 999

c===============================================================================

  999 return

c.... End of subroutine disk.         (+1 line)
      end
