      subroutine intcirc

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intcirc  Updated     1996 March 12 17:100.
c               intcirc  Originated  1994 February 10 17:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find any intersections between two circles, axis and distance
c               between centers, and the surface separation.
c
c     Logic:    Entry points:  intcirc.
c intcirc       Called by:  helpall, main.
c
c intcirc       Calls:  aptcinc, aptvdis, plotpt, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input line:  intcirc [help]

 8110 format ('  Command "intcirc":  acceptable input lines include:' /
     &  '  "intcirc [help]"  Display this message.' //
     &  '  "intcirc U1 V1 RADIUS1 U2 V2 RADIUS2"  Find any',
     &  ' intersections' /
     &  '     of  the circle at point (U1, V1) with radius RADIUS1' /
     &  '     and the circle at point (U2, V2) with radius RADIUS2,' /
     &  '     where U1, V1, U2 and V2 are rectangular coordinates.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Test for wrong number of words on "intcirc" line.

 8120 format ('  intcirc ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "intcirc" line:  ', i3)

      if (nword .ne. 7) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

c.... Find the radii and coordinates.

 8130 format ('  intcirc ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      nerr = 0

      call varfind (2, 7)

      do 140 nw = 2, 7
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  140 continue

c=======================================================================********

c.... See if the radii are non-negative.

c.... Test for wrong number of words on "intcirc" line.

 8140 format ('  intcirc ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A radius is negative:  ',1p2e20.12)

      if ((fword(4) .lt. 0.0)  .or.
     &    (fword(7) .lt. 0.0)) then
        write (   6, 8140) ainput, ninput, fword(4), fword(7)
        write (iout, 8140) ainput, ninput, fword(4), fword(7)
        go to 999
      endif

c=======================================================================********

c.... Input line:  intcirc U1 V1 RADIUS1 U2 V2 RADIUS2

c.... Find the data for the two circles.

      u1 = fword(2)
      v1 = fword(3)
      r1 = fword(4)
      circ1 = 2.0 * pi * r1
      area1 = pi * r1**2

      call plotpt (u1,      v1,      0.0, nerrg)
      call plotpt (u1 - r1, v1,      0.0, nerrg)
      call plotpt (u1 + r1, v1,      0.0, nerrg)
      call plotpt (u1,      v1 - r1, 0.0, nerrg)
      call plotpt (u1,      v1 + r1, 0.0, nerrg)

      u2 = fword(5)
      v2 = fword(6)
      r2 = fword(7)
      circ2 = 2.0 * pi * r2
      area2 = pi * r2**2

      call plotpt (u2,      v2,      0.0, nerrg)
      call plotpt (u2 - r2, v2,      0.0, nerrg)
      call plotpt (u2 + r2, v2,      0.0, nerrg)
      call plotpt (u2,      v2 - r2, 0.0, nerrg)
      call plotpt (u2,      v2 + r2, 0.0, nerrg)

 8150 format ('  r1, perim, area ',1p3e20.12 /
     &        '  Center u1, v1   ',1p2e20.12 /
     &        '  r2, perim, area ',1p3e20.12 /
     &        '  Center u2, v2   ',1p2e20.12 )

      write (   6, 8150) r1, circ1, area1, u1, v1,
     &                   r2, circ2, area2, u2, v2
      write (iout, 8150) r1, circ1, area1, u1, v1,
     &                   r2, circ2, area2, u2, v2

      call aptvdis (u1, v1, 0., u2, v2, 0., 1, tol,
     &              du, dv, dw, d12, nerr)

 8160 format ('  du12, dv12      ',1p2e20.12 /
     &        '  Center distance ',1pe20.12 )

      write (   6, 8160) du, dv, d12
      write (iout, 8160) du, dv, d12

      distex1 = d12 - r1 - r2
      distex2 = d12 + r1 + r2
      distin1 = d12 - r1 + r2
      distin2 = d12 + r1 - r2
      disterr = tol * (d12 + r1 + r2)
      if (abs (distex1) .lt. disterr) distex1 = 0.0
      if (abs (distex2) .lt. disterr) distex2 = 0.0
      if (abs (distin1) .lt. disterr) distin1 = 0.0
      if (abs (distin2) .lt. disterr) distin2 = 0.0

 8170 format ('  Separation (ext)',1pe20.12 /
     &        '  Separation (ext ',1pe20.12 /
     &        '  Separation (int)',1pe20.12 /
     &        '  Separation (int)',1pe20.12 )

      write (   6, 8170) distex1, distex2, distin1, distin2
      write (iout, 8170) distex1, distex2, distin1, distin2

c.... Find any intersections of the two circles.

      call aptcinc (r1, u1, v1, r2, u2, v2, 1, tol,
     &              u3, v3, u4, v4, nint, nerr)

 8190 format ('  No intersection.')
 8200 format ('  1 tangent point ',1p2e20.12 )
 8210 format ('  2 intersections ',1p2e20.12 /
     &        '                  ',1p2e20.12 )
 8220 format ('  Coincident.     ')

      if (nint .eq. 0) then
        write (   6, 8190)
        write (iout, 8190)
      elseif (nint .eq. 1) then
        write (   6, 8200) u3, v3
        write (iout, 8200) u3, v3
        call plotpt (u3, v3, 0.0, nerrg)
      elseif (nint .eq. 2) then
        write (   6, 8210) u3, v3, u4, v4
        write (iout, 8210) u3, v3, u4, v4
        call plotpt (u3, v3, 0.0, nerrg)
        call plotpt (u4, v4, 0.0, nerrg)
      else
        write (   6, 8220)
        write (iout, 8220)
      endif

  999 return

c.... End of subroutine intcirc.      (+1 line)
      end
