      subroutine intplcy (npl, ncyl)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intplcy  Updated     1996 March 13 16:40.
c               intplcy  Originated  1995 January 11 14:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the distance of separation or overlap between a plane
c               and a circular cylinder, any proximal lines on the plane, and
c               points on any ellipse of intersection.
c
c     Logic:    Entry points:  intplcy
c intplcy       Called by:  distance.
c
c intplcy       Calls:  aptplcy, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /lintplcy/ pinx(4)       ! Coordinate x of an intersection point.
      common /lintplcy/ piny(4)       ! Coordinate y of an intersection point.
      common /lintplcy/ pinz(4)       ! Coordinate z of an intersection point.

c-----------------------------------------------------------------------********

c.... If the plane is parallel to the axis of the circular cylinder,
c....   find the distance between the plane and the axis of the circular
c....   cylinder, the distance of separation or overlap of the plane and
c....   the surface of the cylinder, and if they overlap, points on the two
c....   lines of intersection, and if they are tangent, a point on the single
c....   line of intersection, and the vector direction of such lines.
c....   If the plane is not parallel to the axis of the circular cylinder,
c....   find the intercept of the axis on the plane, and the four points at
c....   the ends of the minor and major axes of the ellipse of intersection
c....   of the plane with the surface of the circular cylinder.

      call aptplcy (xcen(npl), ycen(npl), zcen(npl),
     &              qx(npl), qy(npl), qz(npl),
     &              qc(ncyl), qx(ncyl), qy(ncyl), qz(ncyl),
     &              qxy(ncyl), qyz(ncyl), qzx(ncyl),
     &              qxx(ncyl), qyy(ncyl), qzz(ncyl), tol,
     &              rcyl, cx, cy, cz, dx, dy, dz,
     &              ipar, daxis, dpax, dpay, dpaz, dsurf1, dsurf2,
     &              dpcx, dpcy, dpcz, pcx, pcy, pcz,
     &              ppx, ppy, ppz, pinx, piny, pinz, nerra)

      pdist = dsurf1

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx = -1.e99
      proxvy = -1.e99
      proxvz = -1.e99

c-----------------------------------------------------------------------********

      if (ipar .eq. 1) then           ! Plane is parallel to cylinder axis.

 8010   format ('Plane contains axis of cylinder.' )
 8020   format ('Plane is parallel to axis of cylinder.' /
     &  '  Plane-axis dist ',1pe20.12)
 8025   format ('  Vector     (xyz)',1p3e20.12)      

        if (daxis .eq. 0.0) then
          write (   6, 8010)
          write (iout, 8010)
        else
          write (   6, 8020) daxis
          write (iout, 8020) daxis
          write (   6, 8025) dpax, dpay, dpaz
          write (iout, 8025) dpax, dpay, dpaz
        endif

 8040   format ('Distances, vectors from plane to surface',
     &  ' of cylinder:' /
     &  '  Min distance    ',1pe20.12,'  daxis - rcyl,        1  |2' /
     &  '  Min vector (xyz)',1p3e20.12 /
     &  '  Max distance    ',1pe20.12,'  daxis + rcyl,        1  2|' /
     &  '  Max vector (xyz)',1p3e20.12 )

        err2  = tol * dsurf2
        dpc2x = 2.0 * dpax - dpcx
        dpc2y = 2.0 * dpay - dpcy
        dpc2z = 2.0 * dpaz - dpcz
        if (abs (dpc2x) .le. err2) dpc2x = 0.0
        if (abs (dpc2y) .le. err2) dpc2y = 0.0
        if (abs (dpc2z) .le. err2) dpc2z = 0.0

        write (   6, 8040) dsurf1, dpcx, dpcy, dpcz, dsurf2,
     &    dpc2x, dpc2y, dpc2z
        write (iout, 8040) dsurf1, dpcx, dpcy, dpcz, dsurf2,
     &    dpc2x, dpc2y, dpc2z

        pdist   = dsurf1

        proxvx = dpcx
        proxvy = dpcy
        proxvz = dpcz

 8050   format ('Tangent line in plane and cylinder:' /
     &  '  Point      (xyz)',1p3e20.12 /
     &  '  Direction  (xyz)',1p3e20.12 )

 8052   format ('Proximal lines in plane and cylinder:' /
     &  '  Point pl   (xyz)',1p3e20.12 /
     &  '  Point cyl  (xyz)',1p3e20.12 /
     &  '  Direction  (xyz)',1p3e20.12 )
 8060   format ('Intersection lines of plane and cylinder:' /
     &  '  Point 1    (xyz)',1p3e20.12 /
     &  '  Point 2    (xyz)',1p3e20.12 /
     &  '  Direction  (xyz)',1p3e20.12 )

        if (dsurf1 .eq. 0.0) then     ! Plane and cylinder are tangent.

          write (   6, 8050) pinx(1), piny(1), pinz(1), dx, dy, dz
          write (iout, 8050) pinx(1), piny(1), pinz(1), dx, dy, dz

          call plotpt (pinx(1), piny(1), pinz(1), nerrg)

          proxptx = pinx(1)
          proxpty = piny(1)
          proxptz = pinz(1)

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        elseif (dsurf1 .lt. 0.0) then ! Plane and cylinder intersect.

          write (   6, 8052) ppx, ppy, ppz, pcx, pcy, pcz,
     &      dx, dy, dz
          write (iout, 8052) ppx, ppy, ppz, pcx, pcy, pcz,
     &      dx, dy, dz

          call plotpt (ppx, ppy, ppz, nerrg)
          call plotpt (pcx, pcy, pcz, nerrg)

          proxptx = pcx
          proxpty = pcy
          proxptz = pcz

          write (   6, 8060) pinx(1), piny(1), pinz(1),
     &      pinx(2), piny(2), pinz(2), dx, dy, dz
          write (iout, 8060) pinx(1), piny(1), pinz(1),
     &      pinx(2), piny(2), pinz(2), dx, dy, dz

          call plotpt (pinx(1), piny(1), pinz(1), nerrg)
          call plotpt (pinx(2), piny(2), pinz(2), nerrg)

        else                          ! Plane and cylinder do not intersect.

          write (   6, 8052) ppx, ppy, ppz, pcx, pcy, pcz,
     &      dx, dy, dz
          write (iout, 8052) ppx, ppy, ppz, pcx, pcy, pcz,
     &      dx, dy, dz

          call plotpt (ppx, ppy, ppz, nerrg)
          call plotpt (pcx, pcy, pcz, nerrg)

          proxptx = pcx
          proxpty = pcy
          proxptz = pcz

        endif

        err2 = 2.0 * tol * rcyl
        pc2x = 2.0 * cx - pcx
        pc2y = 2.0 * cy - pcy
        pc2z = 2.0 * cz - pcz
        if (abs (pc2x) .le. err2) pc2x = 0.0
        if (abs (pc2y) .le. err2) pc2y = 0.0
        if (abs (pc2z) .le. err2) pc2z = 0.0

 8065   format ('Distal line in cylinder:' /
     &  '  Point cyl  (xyz)',1p3e20.12 /
     &  '  Direction  (xyz)',1p3e20.12 )

        write (   6, 8065) pc2x, pc2y, pc2z, dx, dy, dz
        write (iout, 8065) pc2x, pc2y, pc2z, dx, dy, dz

        call plotpt (pc2x, pc2y, pc2z, nerrg)

        go to 999

      else                            ! Axis of cylinder intersects plane.

 8070   format ('Center and semi-axes points on ellipse of',
     &  ' intersection:' /
     &  '  Center     (xyz)',1p3e20.12 /
     &  '  Min axis 1 (xyz)',1p3e20.12 /
     &  '  Min axis 2 (xyz)',1p3e20.12 /
     &  '  Maj axis 1 (xyz)',1p3e20.12 /
     &  '  Maj axis 2 (xyz)',1p3e20.12 )

        write (   6, 8070) ppx, ppy, ppz,
     &    (pinx(n), piny(n), pinz(n), n = 1, 4)
        write (iout, 8070) ppx, ppy, ppz,
     &    (pinx(n), piny(n), pinz(n), n = 1, 4)

        call plotpt (ppx, ppy, ppz, nerrg)

        do 100 n = 1, 4
          call plotpt (pinx(n), piny(n), pinz(n), nerrg)
  100   continue

      endif                           ! Tested whether axes intersect or not.

c-----------------------------------------------------------------------********

  999 return

c.... End of subroutine intplcy.      (+1 line)
      end
