      subroutine matrix

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  matrix   Updated     2004 November 23 13:30.
c               matrix   Originated  1993 November 22 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To operate on one or all points, clusters, vectors, planes,
c               spheres, circular cylinders, circular cones, hyperbolic
c               paraboloids, ellipsoids, axisymmetric quadric surfaces or
c               general quadric surfaces, with a 3 by 3 matrix operator, of type
c               "invert", "reflect", "rotate" or "scale",  with a specified
c               invariant point.
c
c     Logic:    Entry points:  matrix.
c matrix        Called by:  helpall, main, rotate* .
c
c matrix        Calls:  aptmopv, aptrefs, aptrois, clout, pout, ptfind, quout,
c               qutype, vectfind, vout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lmatrix/ refm(3,3)      ! Reflection tensor.
      character*8      aopx           ! A valid operator type.
      common /lmatrix/ amount         ! Amount of output data.
      character*8      amount         ! Amount of output data.

c.... Input line:  invert  [help]
c.... Input line:  reflect [help]
c.... Input line:  rotate  [help]
c.... Input line:  scale   [help]

 8110 format ('  Command "',a8,'":  acceptable input lines include:' /
     &  '  "',a8,' [help]"   Display this message.' /
     &  '  "',a8,' point   PNAME  [-] OPNAME [PINV]"' /
     &  '     to ',a8,' point PNAME with operator OPNAME,' /
     &  '     and invariant point PINV (defaults to origin).' /
     &  '  "',a8,' cluster CLNAME [-] OPNAME [PINV]"' /
     &  '     to ',a8,' cluster CLNAME with operator OPNAME,' /
     &  '     and invariant point PINV (defaults to origin).' /
     &  '  "',a8,' point   all    [-] OPNAME [PINV]"' /
     &  '     to ',a8,' all points  with operator OPNAME,' /
     &  '     and invariant point PINV (defaults to origin).' /
     &  '  "',a8,' vector  VNAME  [-] OPNAME [PINV]"' /
     &  '     to ',a8,' vector VNAME with operator OPNAME,' /
     &  '     and invariant point PINV (defaults to origin).' /
     &  '  "',a8,' vector  all    [-] OPNAME [PINV]"' /
     &  '     to ',a8,' all vectors with operator OPNAME,' /
     &  '     and invariant point PINV (defaults to origin).' )

 8111 format (
     &  '  "',a8,' QTYPE  QNAME [-] OPNAME [PINV]"' /
     &  '     to ',a8,' quadric type QTYPE with name QNAME,' /
     &  '     with operator OPNAME, and invariant point PINV.' /
     &  '  "',a8,' QTYPE  all   [-] OPNAME [PINV]"' /
     &  '     to ',a8,' all quadrics of type QTYPE,' /
     &  '     with operator OPNAME, and invariant point PINV.' //
     &  '  Note:  OPNAME may not be "-".' //
     &  '  Options:  QTYPE must be plane, sphere, cylinder, cone,',
     &  ' hyperb, ellipsoid,' /
     &  '    axisym or quadric.' /
     &  '    [blank] indicates the operator, and [-] its inverse' /
     &  '    (use only for operators rotate and scale).' /
     &  '    Invariant point PINV defaults to the origin.' )

 8114 format (/
     &  '  "reflect  vector  VNAME normal VNORM"' /
     &  '    to reflect vector VNAME in the surface with normal',
     &  ' vector VNORM.' )

 8116 format (/
     &  '  Synonyms:  [cluster, cl], [cylinder, cyl],',
     &  ' [ellipsoid, ellipse, ell, el],' /
     &  '    [help, h], [hyperb, saddle], [invert, inv],'
     &  ' [plane, pl],' /
     &  '    [normal, norm], [point, pnt, pt, p],' /
     &  '    [quadric, quad, q], [reflect, refl], [rotate, rot],' /
     &  '    [scale, sc], [sphere, sph], [vector, vect, v].' )

      if (aword(1) .eq. 'inv' ) aword(1) = 'invert'
      if (aword(1) .eq. 'refl') aword(1) = 'reflect'
      if (aword(1) .eq. 'rot' ) aword(1) = 'rotate'
      if (aword(1) .eq. 'sc' )  aword(1) = 'scale'

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then

        write (   6, 8110) aword(1), aword(1), aword(1), aword(1),
     &    aword(1), aword(1), aword(1), aword(1), aword(1), aword(1),
     &    aword(1), aword(1)
        write (iout, 8110) aword(1), aword(1), aword(1), aword(1),
     &    aword(1), aword(1), aword(1), aword(1), aword(1), aword(1),
     &    aword(1)
        write (   6, 8111) aword(1), aword(1), aword(1), aword(1)
        write (iout, 8111) aword(1), aword(1), aword(1), aword(1)

        if (aword(1) .eq. 'reflect') then
          write (   6, 8114)
          write (iout, 8114)
        endif

        write (   6, 8116)
        write (iout, 8116)

        go to 999

      endif

c===============================================================================

c.... Input line:  reflect vector VNAME normal VNORM

      if (((aword(1) .eq. 'reflect') .or.
     &     (aword(1) .eq. 'refl'))   .and.
     &    ((aword(2) .eq. 'vector')  .or.
     &     (aword(2) .eq. 'vect')    .or.
     &     (aword(2) .eq. 'v'))      .and.
     &    ((aword(4) .eq. 'normal')  .or.
     &     (aword(4) .eq. 'norm'))   .and.
     &     (nword .eq. 5))           then

c.... Find the vector to operate on.

        call vectfind (3, nv)

        if (nv .eq. 0) go to 999

c.... Find the normal vector of the reflecting surface.

        call vectfind (5, nr)

        if (nr .eq. 0) go to 999

 8117   format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Normal vector has zero length:  ',a)

c....   See if the normal vector length has zero length.

        if (vlen(nr) .eq. 0.0) then
          write (   6, 8117) ainput, ninput, aword(5)
          write (iout, 8117) ainput, ninput, aword(5)
          go to 999
        endif

c....   Reflect the vector in the surface.

        call aptrefs (0, 0., 0., 0., xcom(nr), ycom(nr), zcom(nr),
     &                0., 0., 0., xcom(nv), ycom(nv), zcom(nv), 1, tol,
     &                refm, nerra) 

        call vout (nv, nv)

        go to 999

      endif

c===============================================================================

c.... Test for correct number of words in command.

 8118 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4-6 words in command:  ',i3)

      if ((nword .lt. 4) .or. (nword .gt. 6)) then
        write (   6, 8118) ainput, ninput, nword
        write (iout, 8118) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... See whether to use the operator or its inverse.

      if (aword(4) .eq. '+') then     ! Use the operator.
        nwop = 5                      ! Word specifying the operator.
        inv  = 0
      elseif (aword(4) .eq. '-') then ! Use the inverse of the operator.
        nwop = 5
        inv  = 1
      else                            ! Use the operator.
        nwop = 4
        inv  = 0
      endif                           ! Tested for the inverse option.

c.... See if the operator exists.

 8120 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)

      do 120 n = 1, nopers
        if (aword(nwop) .eq. aoper(n)) then
          nop = n
          go to 130
        endif
  120 continue
      write (   6, 8120) ainput, ninput, aword(nwop)
      write (iout, 8120) ainput, ninput, aword(nwop)
      go to 999

  130 continue

c===============================================================================

c.... Find the invariant point.

 8150 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Extra words at end of command:  ',i3)

      if (nword .eq. nwop) then       ! Use default, invariant at origin.

        xinv = 0.0
        yinv = 0.0
        zinv = 0.0

      elseif (nword .eq. (nwop + 1)) then  ! Use specified invariant point.

        nw = nwop + 1

        call ptfind (nw, n)

        if (n .eq. 0) go to 999

        xinv = x(n)
        yinv = y(n)
        zinv = z(n)

      else                            ! Wrong number of words.

        nextra = nword - nw
        write (   6, 8150) ainput, ninput, nextra
        write (iout, 8150) ainput, ninput, nextra
        go to 999

      endif                           ! Tested number of words.

  150 continue

c===============================================================================

c.... See if the operator is the right type.

 8160 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Operator type is not matrix:  ',a)
 8170 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Operator type is not recognized:  ',a)
 8180 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Operator and its inverse are the same:  ',a)

      if ((aword(1) .eq. 'invert') .or.
     &    (aword(1) .eq. 'inv'))   then

        aopx = 'invert'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8160) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8160) ainput, ninput, aopx, aoptype(nop)
          go to 999
        endif

        if (inv .eq. 1) then
          write (   6, 8180) ainput, ninput, aword(1)
          write (iout, 8180) ainput, ninput, aword(1)
          go to 999
        endif

      elseif ((aword(1) .eq. 'reflect') .or.
     &        (aword(1) .eq. 'refl'))    then

        aopx = 'reflect'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8160) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8160) ainput, ninput, aopx, aoptype(nop)
          go to 999
        endif

        if (inv .eq. 1) then
          write (   6, 8180) ainput, ninput, aword(1)
          write (iout, 8180) ainput, ninput, aword(1)
          go to 999
        endif

      elseif ((aword(1) .eq. 'rotate') .or.
     &        (aword(1) .eq. 'rot'))   then

        aopx = 'rotate'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8160) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8160) ainput, ninput, aopx, aoptype(nop)
          go to 999
        endif

      elseif (aword(1) .eq. 'scale') then

        aopx = 'scale'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8160) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8160) ainput, ninput, aopx, aoptype(nop)
          go to 999
        endif

      else
        write (   6, 8170) ainput, ninput, aword(1)
        write (iout, 8170) ainput, ninput, aword(1)
        go to 999
      endif

c===============================================================================

c.... Input line:  OPTYPE point PNAME [-] OPNAME [PINV]
c.... Input line:  OPTYPE point all   [-] OPNAME [PINV]

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

 8215 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no points.')

        if (npoints .eq. 0) then
          write (   6, 8215) ainput, ninput
          write (iout, 8215) ainput, ninput
          go to 999
        endif

        if (aword(3) .eq. 'all') then

c....     Operate on all points.

          if ((aword(1) .eq. 'scale') .and.
     &        (inv .eq. 1)) then

            call aptmopv (topin(1,1,nop), inv, xinv, yinv, zinv,
     &                    x, y, z, npoints, tol, nerr)

          else

            call aptmopv (toper(1,1,nop), inv, xinv, yinv, zinv,
     &                    x, y, z, npoints, tol, nerr)

          endif

          amount = 'maximum'
          call pout (1, npoints, amount)

        else

c....     Operate on one point.

          call ptfind (3, n)

          if (n .eq. 0) go to 999

          if ((aword(1) .eq. 'scale') .and.
     &        (inv .eq. 1)) then

            call aptmopv (topin(1,1,nop), inv, xinv, yinv, zinv,
     &                    x(n), y(n), z(n), 1, tol, nerr)

          else

            call aptmopv (toper(1,1,nop), inv, xinv, yinv, zinv,
     &                    x(n), y(n), z(n), 1, tol, nerr)

          endif

          amount = 'maximum'
          call pout (n, n, amount)

        endif                         ! Operated on one or all points.

c===============================================================================

c.... Input line:  OPTYPE cluster CLNAME [-] OPNAME [PINV]

      elseif ((aword(2) .eq. 'cluster') .or.
     &        (aword(2) .eq. 'cl'))    then

 8410 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no clusters.')

      if (nclusts .eq. 0) then
        write (   6, 8410) ainput, ninput
        write (iout, 8410) ainput, ninput
        go to 999
      endif

      do 440 n = 1, nclusts           ! Loop over clusters.
        if (aword(3) .ne. aclust(n)) go to 440

        call clout (n, n, 'min')

        if (aclust(n) .eq. ' ') go to 450

        do 430 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .ne. aclust(n)) go to 430

          do 420 np = 1, npoints      ! Loop over points.
            if (apoint(np) .ne. apairpt(npr)) go to 420

            amount = 'maximum'
            call pout (np, np, amount)

            if ((aword(1) .eq. 'scale') .and.
     &          (inv .eq. 1)) then

              call aptmopv (topin(1,1,nop), inv, xinv, yinv, zinv,
     &                      x(np), y(np), z(np), 1, tol, nerr)

            else

              call aptmopv (toper(1,1,nop), inv, xinv, yinv, zinv,
     &                      x(np), y(np), z(np), 1, tol, nerr)

            endif

            call pout (np, np, 'min')

            go to 430                 ! Find next point in cluster.
  420     continue                    ! End of loop over points.

 8420     format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named cluster point does not exist:  ',a)

          write (   6, 8420) ainput, ninput, apairpt(npr)
          write (iout, 8420) ainput, ninput, apairpt(npr)
          go to 999

  430   continue                      ! End of loop over cluster pairs.

        go to 999
  440 continue                        ! End of loop over clusters.

 8450 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

  450 write (   6, 8450) ainput, ninput, aword(3)
      write (iout, 8450) ainput, ninput, aword(3)
      go to 999

c===============================================================================

c.... Input line:  OPTYPE vector VNAME [-] OPNAME [PINV]
c.... Input line:  OPTYPE vector all   [-] OPNAME [PINV]

      elseif ((aword(2) .eq. 'vector') .or.
     &        (aword(2) .eq. 'vect')   .or.
     &        (aword(2) .eq. 'v'))     then

 8250 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no vectors.')

        if (nvectors .eq. 0) then
          write (   6, 8250) ainput, ninput
          write (iout, 8250) ainput, ninput
          go to 999
        endif

        if (aword(3) .eq. 'all') then

c....     Operate on all vectors.

          if ((aword(1) .eq. 'scale') .and.
     &        (inv .eq. 1)) then

            call aptmopv (topin(1,1,nop), inv, 0., 0., 0.,
     &                    xcom, ycom, zcom, nvectors, tol, nerr)

            call aptmopv (topin(1,1,nop), inv, xinv, yinv, zinv,
     &                    xbnd, ybnd, zbnd, nvectors, tol, nerr)

          else

            call aptmopv (toper(1,1,nop), inv, 0., 0., 0.,
     &                    xcom, ycom, zcom, nvectors, tol, nerr)

            call aptmopv (toper(1,1,nop), inv, xinv, yinv, zinv,
     &                    xbnd, ybnd, zbnd, nvectors, tol, nerr)

          endif

          call vout (1, nvectors)

        else

c....     Find the vector to operate on.

          call vectfind (3, n)

          if (n .eq. 0) go to 999

          if ((aword(1) .eq. 'scale') .and.
     &      (inv .eq. 1)) then

            call aptmopv (topin(1,1,nop), inv, 0., 0., 0.,
     &                    xcom(n), ycom(n), zcom(n), 1, tol, nerr)

            call aptmopv (topin(1,1,nop), inv, xinv, yinv, zinv,
     &                    xbnd(n), ybnd(n), zbnd(n), 1, tol, nerr)

          else
  
            call aptmopv (toper(1,1,nop), inv, 0., 0., 0.,
     &                    xcom(n), ycom(n), zcom(n), 1, tol, nerr)

            call aptmopv (toper(1,1,nop), inv, xinv, yinv, zinv,
     &                    xbnd(n), ybnd(n), zbnd(n), 1, tol, nerr)

          endif

          call vout (n, n)

        endif                         ! Operated on one or all vectors.

c===============================================================================

c.... Input line:  OPTYPE plane     PLNAME  [-] OPNAME [PINV]
c.... Input line:  OPTYPE plane     all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE sphere    SPHNAME [-] OPNAME [PINV]
c.... Input line:  OPTYPE sphere    all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE cylinder  CYLNAME [-] OPNAME [PINV]
c.... Input line:  OPTYPE cylinder  all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE cone      CNNAME  [-] OPNAME [PINV]
c.... Input line:  OPTYPE cone      all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE hyperb    HPNAME  [-] OPNAME [PINV]
c.... Input line:  OPTYPE hyperb    all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE ellipsoid ELNAME  [-] OPNAME [PINV]
c.... Input line:  OPTYPE ellipsoid all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE axisym    AXNAME  [-] OPNAME [PINV]
c.... Input line:  OPTYPE axisym    all     [-] OPNAME [PINV]
c.... Input line:  OPTYPE quadric   QNAME   [-] OPNAME [PINV]
c.... Input line:  OPTYPE quadric   all     [-] OPNAME [PINV]

      elseif ((aword(2) .eq. 'quadric')   .or.
     &        (aword(2) .eq. 'quad')      .or.
     &        (aword(2) .eq. 'q')         .or.
     &        (aword(2) .eq. 'plane')     .or.
     &        (aword(2) .eq. 'pl')        .or.
     &        (aword(2) .eq. 'sphere')    .or.
     &        (aword(2) .eq. 'sph')       .or.
     &        (aword(2) .eq. 'cylinder')  .or.
     &        (aword(2) .eq. 'cyl')       .or.
     &        (aword(2) .eq. 'cone')      .or.
     &        (aword(2) .eq. 'hyperb')    .or.
     &        (aword(2) .eq. 'saddle')    .or.
     &        (aword(2) .eq. 'ellipsoid') .or.
     &        (aword(2) .eq. 'ellipse')   .or.
     &        (aword(2) .eq. 'ell')       .or.
     &        (aword(2) .eq. 'el')        .or.
     &        (aword(2) .eq. 'axisym'))   then

 8310 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no such quadric surfaces:  ',a)
 8320 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

        if (nquads .eq. 0) then
          write (   6, 8310) ainput, ninput
          write (iout, 8310) ainput, ninput
          go to 999
        endif

        nquadx = 0

        if (aword(3) .eq. 'all') then

c....     Operate on all quadric surfaces of the specified type.

          nfind = 0

          if ((aword(1) .eq. 'scale') .and.
     &        (inv .eq. 0)) then

            do 300 n = 1, nquads

              call qutype (aword(2), n, 0, nerrq)

              if (nerrq .eq. 1) go to 300

              nquadx = nquadx + 1

              nfind = 1

              call quout (n, n, 'min', nerrq)

              call aptrois (topin(1,1,nop), inv, xinv, yinv, zinv,
     &             qc(n), qx(n), qy(n), qz(n),
     &             qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &             tol, nerr)

              call quout (n, n, 'min', nerrq)

  300       continue

          else

            do 305 n = 1, nquads

              call qutype (aword(2), n, 0, nerrq)

              if (nerrq .eq. 1) go to 305

              nfind = 1

              nquadx = nquadx + 1

              call quout (n, n, 'min', nerrq)

              call aptrois (toper(1,1,nop), inv, xinv, yinv, zinv,
     &             qc(n), qx(n), qy(n), qz(n),
     &             qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &             tol, nerr)

              call quout (n, n, 'min', nerrq)

  305       continue

          endif

          if (nfind .eq. 0) then
            write (   6, 8310) ainput, ninput, aword(2)
            write (iout, 8310) ainput, ninput, aword(2)
            go to 999
          endif

        else                          ! Operate on a single quadric surface.

c....     Find the quadric surface to operate on.

          do 310 n = 1, nquads
            if (aword(3) .eq. aquad(n)) then

              call qutype (aword(2), n, 1, nerrq)

              if (nerrq .eq. 1) go to 999

              nquadx = nquadx + 1

              call quout (n, n, 'min', nerrq)

              if ((aword(1) .eq. 'scale') .and.
     &          (inv .eq. 0)) then

              call aptrois (topin(1,1,nop), inv, xinv, yinv, zinv,
     &             qc(n), qx(n), qy(n), qz(n),
     &             qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &             tol, nerr)

              else

              call aptrois (toper(1,1,nop), inv, xinv, yinv, zinv,
     &             qc(n), qx(n), qy(n), qz(n),
     &             qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &             tol, nerr)
 
              endif

              call quout (n, n, 'min', nerrq)

              go to 320
            endif
  310     continue                    ! End of loop over quadrics.

          write (   6, 8320) ainput, ninput, aword(3)
          write (iout, 8320) ainput, ninput, aword(3)
          go to 999

        endif                         ! Operated on one or all QTYPE.

 8330   format (/ 'Number of ',a24,' operated on = ',i5)

  320   write (   6, 8330) aword(2), nquadx
        write (iout, 8330) aword(2), nquadx

c===============================================================================

c.... No such object.

      else

 8610 format ('  matrix ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Object type is not recognized:  ',a)

        write (   6, 8610) ainput, ninput, aword(2)
        write (iout, 8610) ainput, ninput, aword(2)

      endif                           ! Found object to operate on.

  999 return

c.... End of subroutine matrix.       (+1 line)
      end
