      subroutine meshexch (alayer1, alayer2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshexch Updated     2005 March 18 18:20.
c               meshexch Originated  1995 September 1 15:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To exchange the relative indices of type alayer1 with the
c               relative indices of type alayer2, for all mesh points in
c               the mesh block.
c
c     Logic:    Entry points:  meshexch.
c meshexch      Called by:  mesh.
c
c meshexch      Calls:  meshsub, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     alayer1   Input    Type of first  layer to exchange, "k", "l" or "m".
c
c     alayer2   Input    Type of second layer to exchange, "l", "m" or "k".
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input variables.

      character*8 alayer1             ! Type of layer exchanged, "k", "l", "m".
      character*8 alayer2             ! Type of layer exchanged, "l", "m", "k".

c.... Initialize count of mesh points with indices exchanged.

      npexch = 0

c.... Exchange the "alayer1" indices with the "alayer2" indices.

 8150 format ('  mesh point ',a,2x,3i6,7x,'! EXCHANGED')

      if     (((alayer1 .eq. 'k') .and. (alayer2 .eq. 'l'))  .or.
     &        ((alayer1 .eq. 'l') .and. (alayer2 .eq. 'k'))) then

 8155   format ('  meshexch ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Sizes of mesh block in k and l directions',
     &    ' different:  ',2i7)

        nb1 = kbl2 - kbl1 + 1         ! Number of k values.
        nb2 = lbl2 - lbl1 + 1         ! Number of l values.

        if (nb2 .ne. nb1) then
          write (   6, 8155) ainput, ninput, nb1, nb2
          write (iout, 8155) ainput, ninput, nb1, nb2
          go to 999
        endif

        do 160 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 160
          if ((kpoint(np) .lt. kbl1)  .or.
     &        (kpoint(np) .gt. kbl2)  .or.
     &        (lpoint(np) .lt. lbl1)  .or.
     &        (lpoint(np) .gt. lbl2)  .or.
     &        (mpoint(np) .lt. mbl1)  .or.
     &        (mpoint(np) .gt. mbl2)) go to 160  ! Not in the mesh block.

          knew = kbl1 + lpoint(np) - lbl1  ! New k index.
          lnew = lbl1 + kpoint(np) - kbl1  ! New l index.

          if ((kpoint(np) .eq. knew)  .and.
     &        (lpoint(np) .eq. lnew)) go to 160

          call pout (np, np, 'min')

          npexch     = npexch + 1
          kpoint(np) = knew
          lpoint(np) = lnew

          call meshsub (np, nrep, nerrsub)

 8160     format ('  meshexch ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Can not make point subscripts match mesh indices.')

          if (nerrsub .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)

  160   continue                      ! End of loop over points.

      elseif (((alayer1 .eq. 'l') .and. (alayer2 .eq. 'm'))  .or.
     &        ((alayer1 .eq. 'm') .and. (alayer2 .eq. 'l'))) then

 8165   format ('  meshexch ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Sizes of mesh block in l and m directions',
     &    ' different:  ',2i7)

        nb1 = lbl2 - lbl1 + 1         ! Number of l values.
        nb2 = mbl2 - mbl1 + 1         ! Number of m values.

        if (nb2 .ne. nb1) then
          write (   6, 8165) ainput, ninput, nb1, nb2
          write (iout, 8165) ainput, ninput, nb1, nb2
          go to 999
        endif

        do 170 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 170
          if ((kpoint(np) .lt. kbl1)  .or.
     &        (kpoint(np) .gt. kbl2)  .or.
     &        (lpoint(np) .lt. lbl1)  .or.
     &        (lpoint(np) .gt. lbl2)  .or.
     &        (mpoint(np) .lt. mbl1)  .or.
     &        (mpoint(np) .gt. mbl2)) go to 170  ! Not in the mesh block.

          lnew = lbl1 + mpoint(np) - mbl1  ! New l index.
          mnew = mbl1 + lpoint(np) - lbl1  ! New m index.

          if ((lpoint(np) .eq. lnew)  .and.
     &        (mpoint(np) .eq. mnew)) go to 170

          call pout (np, np, 'min')

          npexch     = npexch + 1
          lpoint(np) = lnew
          mpoint(np) = mnew

          call meshsub (np, nrep, nerrsub)

          if (nerrsub .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)

  170   continue                      ! End of loop over points.

      elseif (((alayer1 .eq. 'm') .and. (alayer2 .eq. 'k'))  .or.
     &        ((alayer1 .eq. 'k') .and. (alayer2 .eq. 'm'))) then

 8175   format ('  meshexch ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Sizes of mesh block in m and k directions',
     &    ' different:  ',2i7)

        nb1 = mbl2 - mbl1 + 1         ! Number of m values.
        nb2 = kbl2 - kbl1 + 1         ! Number of k values.

        if (nb2 .ne. nb1) then
          write (   6, 8175) ainput, ninput, nb1, nb2
          write (iout, 8175) ainput, ninput, nb1, nb2
          go to 999
        endif

        do 180 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 180
          if ((kpoint(np) .lt. kbl1)  .or.
     &        (kpoint(np) .gt. kbl2)  .or.
     &        (lpoint(np) .lt. lbl1)  .or.
     &        (lpoint(np) .gt. lbl2)  .or.
     &        (mpoint(np) .lt. mbl1)  .or.
     &        (mpoint(np) .gt. mbl2)) go to 180  ! Not in the mesh block.

          mnew = mbl1 + kpoint(np) - kbl1  ! New m index.
          knew = kbl1 + mpoint(np) - mbl1  ! New k index.

          if ((mpoint(np) .eq. mnew)  .and.
     &        (kpoint(np) .eq. knew)) go to 180

          call pout (np, np, 'min')

          npexch     = npexch + 1
          mpoint(np) = mnew
          kpoint(np) = knew

          call meshsub (np, nrep, nerrsub)

          if (nerrsub .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)

  180   continue                      ! End of loop over points.

      else                            ! Error in input data.

 8110   format ('  meshexch ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Mesh layer types are not k, l or m, or are same:  ',2a)

        write (   6, 8110) ainput, ninput, alayer1, alayer2
        write (iout, 8110) ainput, ninput, alayer1, alayer2
        go to 999

      endif                           ! Tested for kl, lm or mk layer exch.

c.... Display the number of points exchanged.

 8210 format ('  Number of mesh points exchanged =',i7 )

      if (npexch .gt. 0) then
        write (   6, 8210) npexch
        write (iout, 8210) npexch
      endif

  999 return

c.... End of subroutine meshexch.     (+1 line)
      end
