      subroutine meshinv (alayer)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshinv  Updated     2005 March 21 16:20.
c               meshinv  Originated  1995 August 15 14:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To invert mesh layers of index type "alayer" ("k", "l" or "m")
c               in the mesh block.
c
c     Logic:    Entry points:  meshinv.
c meshinv       Called by:  mesh.
c
c meshinv       Calls:  meshsub, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     alayer    Input    Type of layer to invert, "k", "l" or "m".
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input variables.

      character*8 alayer              ! Type of layer inverted, "k", "l" or "m".

c.... Initialize count of mesh points with index "alayer" inverted.

      npinv = 0

c.... Invert the "alayer" indices from n1 to n2.

 8150 format ('  mesh point ',a,2x,3i6,7x,'! INVERTED')

      if (alayer .eq. 'k') then       ! Inverting k layers.

 8155   format ('  meshinv ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Cannot invert a single k layer:  ',i7)

        if (kbl1 .eq. kbl2) then
          write (   6, 8155) ainput, ninput, kbl1
          write (iout, 8155) ainput, ninput, kbl1
          go to 999
        endif

        do 160 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 160
          if ((kpoint(np) .lt. kbl1)  .or.
     &        (kpoint(np) .gt. kbl2)  .or.
     &        (lpoint(np) .lt. lbl1)  .or.
     &        (lpoint(np) .gt. lbl2)  .or.
     &        (mpoint(np) .lt. mbl1)  .or.
     &        (mpoint(np) .gt. mbl2)) go to 160  ! Not in the mesh block.

          kpointx = kbl1 + kbl2 - kpoint(np)

          if (kpointx .eq. kpoint(np)) go to 160

          call pout (np, np, 'min')

          npinv      = npinv + 1
          kpoint(np) = kbl1 + kbl2 - kpoint(np)

          call meshsub (np, nrep, nerrsub)

 8160     format ('  meshinv ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Can not make point subscripts match mesh indices.')

          if (nerrsub .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)

  160   continue                      ! End of loop over points.

      elseif (alayer .eq. 'l') then   ! Inverting l layers.

 8165   format ('  meshinv ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Cannot invert a single l layer:  ',i7)

        if (lbl1 .eq. lbl2) then
          write (   6, 8165) ainput, ninput, lbl1
          write (iout, 8165) ainput, ninput, lbl1
          go to 999
        endif

        do 170 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 170
          if ((kpoint(np) .lt. kbl1)  .or.
     &        (kpoint(np) .gt. kbl2)  .or.
     &        (lpoint(np) .lt. lbl1)  .or.
     &        (lpoint(np) .gt. lbl2)  .or.
     &        (mpoint(np) .lt. mbl1)  .or.
     &        (mpoint(np) .gt. mbl2)) go to 170  ! Not in the mesh block.

          lpointx = lbl1 + lbl2 - lpoint(np)

          if (lpointx .eq. lpoint(np)) go to 170

          call pout (np, np, 'min')

          npinv      = npinv + 1
          lpoint(np) = lbl1 + lbl2 - lpoint(np)

          call meshsub (np, nrep, nerrsub)

          if (nerrsub .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)

  170   continue                      ! End of loop over points.

      elseif (alayer .eq. 'm') then   ! Inverting m layers.

 8175   format ('  meshinv ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Cannot invert a single m layer:  ',i7)

        if (mbl1 .eq. mbl2) then
          write (   6, 8175) ainput, ninput, mbl1
          write (iout, 8175) ainput, ninput, mbl1
          go to 999
        endif

        do 180 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 180
          if ((kpoint(np) .lt. kbl1)  .or.
     &        (kpoint(np) .gt. kbl2)  .or.
     &        (lpoint(np) .lt. lbl1)  .or.
     &        (lpoint(np) .gt. lbl2)  .or.
     &        (mpoint(np) .lt. mbl1)  .or.
     &        (mpoint(np) .gt. mbl2)) go to 180  ! Not in the mesh block.

          mpointx = mbl1 + mbl2 - mpoint(np)

          if (mpointx .eq. mpoint(np)) go to 180

          call pout (np, np, 'min')

          npinv      = npinv + 1
          mpoint(np) = mbl1 + mbl2 - mpoint(np)

          call meshsub (np, nrep, nerrsub)

          if (nerrsub .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np)

  180   continue                      ! End of loop over points.

      else                            ! Layer type not k, l or m.

 8180   format ('  meshinv ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Mesh layer type is not k, l or m:  ',a)

        write (   6, 8180) ainput, ninput, alayer
        write (iout, 8180) ainput, ninput, alayer
        go to 999

      endif                           ! Tested for k, l or m layer inversion.

c.... Display the number of points inverted.

 8210 format ('  Number of mesh points inverted =',i7 )

      if (npinv .gt. 0) then
        write (   6, 8210) npinv
        write (iout, 8210) npinv
      endif

  999 return

c.... End of subroutine meshinv.      (+1 line)
      end
