      subroutine meshout (k1, k2, l1, l2, m1, m2, npmesh)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshout  Updated     2005 March 9 14:40.
c               meshout  Originated  1995 July 21 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display all points with mesh indices k from k1 to k2,
c               l from l1 to l2, and m from m1 to m2, and count the number
c               of such points.
c
c     Logic:    Entry points:  meshout.
c meshout       Called by:  mesh.
c
c meshout       Calls:  meshchk, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     k1, k2    Input    Minimum and maximum k indices.
c                          1 <= k1 <= k2 <= kmax.
c
c     l1, l2    Input    Minimum and maximum l indices.
c                          1 <= l1 <= l2 <= lmax.
c
c     m1, m2    Input    Minimum and maximum m indices.
c                          1 <= m1 <= m2 <= mmax.
c
c     npmesh    Output   Number of mesh points displayed.
c                          0 if input indices are bad.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

c.... Initialize.

      npmesh  = 0

      call meshchk (k1, k2, l1, l2, m1, m2, nerrchk)

      if (nerrchk .ne. 0) then
        go to 999
      endif

c.... Display all points with the specified indices.

      do 240 k = k1, k2               ! Loop over k.
        do 230 l = l1, l2             ! Loop over l.
          do 220 m = m1, m2           ! Loop over m.
            do 210 n = 1, npoints     ! Loop over points.

              if (apoint(n) .eq. ' ') go to 210
              if (kpoint(n) .ne. k)   go to 210
              if (lpoint(n) .ne. l)   go to 210
              if (mpoint(n) .ne. m)   go to 210

              npmesh = npmesh + 1

              call pout (n, n, 'min')

  210       continue                  ! End of loop over points.
  220     continue                    ! End of loop over m.
  230   continue                      ! End of loop over l.
  240 continue                        ! End of loop over k.

  999 return

c.... End of subroutine meshout.      (+1 line)
      end
