      subroutine meshrlxm

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshrlxm Updated     2005 March 9 14:40.
c               meshrlxm Originated  1995 October 8 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the x, y and z coordinates of all mesh points, except
c               for exterior mesh points or mesh block points, by a relaxation
c               method, which moves each mesh point to the average coordinates
c               of its nearest neighbors.  Any mesh point with exactly one
c               neighboring mesh point in the k, l or m direction is an exterior
c               mesh point.
c
c     Logic:    Entry points:  meshrlxm.
c meshrlxm      Called by:  mesh.
c
c meshrlxm      Calls:  meshfind, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Initialize.

      nerr  = 0

c.... Test for presence of interior mesh points in the mesh,
c....   outside the mesh block.

 8105 format ('  meshrlxm ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh has no interior mesh points outside the mesh',
     &  ' block to relax.')

      if (kmax .eq. 1) then
        kbeg = 1
        kend = kmax
      elseif (kmax .eq. 2) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      else
        kbeg = 2
        kend = kmax - 1
      endif

      if (lmax .eq. 1) then
        lbeg = 1
        lend = lmax
      elseif (lmax .eq. 2) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      else
        lbeg = 2
        lend = lmax - 1
      endif

      if (mmax .eq. 1) then
        mbeg = 1
        mend = mmax
      elseif (mmax .eq. 2) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      else
        mbeg = 2
        mend = mmax - 1
      endif

      if ((kbeg .ge. kbl1)  .and.
     &    (kend .le. kbl2)  .and.
     &    (lbeg .ge. lbl1)  .and.
     &    (lend .le. lbl2)  .and.
     &    (mbeg .ge. mbl1)  .and.
     &    (mend .le. mbl2)) then
        nerr = 1
        write (   6, 8105) ainput, ninput
        write (iout, 8105) ainput, ninput
      endif

c.... Test for mesh indices not in the interior of the mesh block,
c....   not assigned to mesh points.

 8110 format ('  meshrlxm ERROR.  Reading ',a40,' line ',i5,'.' /
     &  'Mesh indices           ',3i6,7x,'! UNASSIGNED')

      do 330 k = 1, kmax              ! Loop over mesh.
        do 320 l = 1, lmax
          do 310 m = 1, mmax

c....       Test all mesh points not in the interior of the mesh block.

            if ((k .gt. kbl1) .and. (k .lt. kbl2) .and.
     &          (l .gt. lbl1) .and. (l .lt. lbl2) .and.
     &          (m .gt. mbl1) .and. (m .lt. mbl2)) go to 310

            call meshfind (k, l, m, np)

            if (np .le. 0) then
              nerr = 1
              klm = k + (l - 1) * kmax + (m - 1) * kmax * lmax
              write (   6, 8110) ainput, ninput, k, l, m
              write (iout, 8110) ainput, ninput, k, l, m
            endif

  310     continue
  320   continue
  330 continue                        ! End of loop over outside points.

      if (nerr .ne. 0) go to 999

c.... Relax interior mesh points not in the mesh block.

      kdone = 0
      niter = 0

c.... Return here for another iteration to relax mesh point coordinates.

  340 nch   = 0

      do 370 m = mbeg, mend           ! Loop over interior points.
        do 360 l = lbeg, lend
          do 350 k = kbeg, kend

c....       Exclude points in the mesh block.

            if ((k .ge. kbl1) .and.
     &          (k .le. kbl2) .and.
     &          (l .ge. lbl1) .and.
     &          (l .le. lbl2) .and.
     &          (m .ge. mbl1) .and.
     &          (m .le. mbl2)) go to 350

c....       Sum the non-vacuum coordinates of any pairs of nearest neighbors.

            ntot = 0
            xtot = 0.0
            ytot = 0.0
            ztot = 0.0

            k1 = k - 1
            k2 = k + 1

            call meshfind (k1, l, m, npk1)
            call meshfind (k2, l, m, npk2)

            if ((npk1 .gt. 0)          .and.
     &          (npk2 .gt. 0)) then
              if( x(npk1) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npk1)
                ytot = ytot + y(npk1)
                ztot = ztot + z(npk1)
              endif
              if( x(npk2) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npk2)
                ytot = ytot + y(npk2)
                ztot = ztot + z(npk2)
              endif
            endif                     ! Mesh point has two neighbors.

            l1 = l - 1
            l2 = l + 1

            call meshfind (k, l1, m, npl1)
            call meshfind (k, l2, m, npl2)

            if ((npl1 .gt. 0)          .and.
     &          (npl2 .gt. 0)) then
              if( x(npl1) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npl1)
                ytot = ytot + y(npl1)
                ztot = ztot + z(npl1)
              endif
              if( x(npl2) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npl2)
                ytot = ytot + y(npl2)
                ztot = ztot + z(npl2)
              endif
            endif                     ! Mesh point has two neighbors.

            m1 = m - 1
            m2 = m + 1

            call meshfind (k, l, m1, npm1)
            call meshfind (k, l, m2, npm2)

            if ((npm1 .gt. 0)          .and.
     &          (npm2 .gt. 0)) then
              if( x(npm1) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npm1)
                ytot = ytot + y(npm1)
                ztot = ztot + z(npm1)
              endif
              if( x(npm2) .gt. -1.e99) then
                ntot = ntot + 1
                xtot = xtot + x(npm2)
                ytot = ytot + y(npm2)
                ztot = ztot + z(npm2)
              endif
            endif                     ! Mesh point has two neighbors.

c....       Change the coordinates to the average of the pairs of nearest
c....         neighbors and test for convergence.

            if (ntot .gt. 0) then     ! Recalculate coordinates.

              call meshfind (k, l, m, np)

              if (kdone .eq. 0) then  ! Not converged.

                xold  = x(np)
                yold  = y(np)
                zold  = z(np)

                x(np) = xtot / ntot
                y(np) = ytot / ntot
                z(np) = ztot / ntot

                if ((abs (x(np) - xold) .gt. tol * abs (xold))  .or.
     &              (abs (y(np) - yold) .gt. tol * abs (yold))  .or.
     &              (abs (z(np) - zold) .gt. tol * abs (zold))) then
                  nch = nch + 1
                endif

              else                    ! Relaxation method converged.

                call pout (np, np, 'minimum')

              endif                   ! Tested for convergence.

            endif                     ! Recalculated coordinates.

  350     continue
  360   continue
  370 continue                        ! End of loop over interior points.

      if (kdone .eq. 0) then
        if (nch .eq. 0) kdone = 1
        niter = niter + 1
        go to 340
      endif

c.... Relaxation method converged.

 8410 format ('  Number of mesh points relaxed =',i7)
 8420 format ('  Number of iterations          =',i7)

      nrlax = (kend - kbeg + 1) * (lend - lbeg + 1) * (mend - mbeg + 1)
     &      - (kbl2 - kbl1 + 1) * (lbl2 - lbl1 + 1) * (mbl2 - mbl1 + 1)

      write (   6, 8410) nrlax
      write (iout, 8410) nrlax
      write (   6, 8420) niter
      write (iout, 8420) niter

  999 return

c.... End of subroutine meshrlxm.     (+1 line)
      end
