      subroutine meshvout (k1, k2, l1, l2, m1, m2, nvoidt)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshvout Updated     2005 March 9 14:40.
c               meshvout Originated  1995 September 12 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display all unassigned mesh indices with
c               k from k1 to k2, l from l1 to l2, and m from m1 to m2,
c               and count the number of such unassigned mesh indices.
c
c     Logic:    Entry points:  meshvout.
c meshvout      Called by:  mesh.
c
c meshvout      Calls:  meshchk.
c
c     Biblio:   ?
c
c     Glossary:
c
c     k1, k2    Input    Minimum and maximum k indices.
c                          1 <= k1 <= k2 <= kmax.
c
c     l1, l2    Input    Minimum and maximum l indices.
c                          1 <= l1 <= l2 <= lmax.
c
c     m1, m2    Input    Minimum and maximum m indices.
c                          1 <= m1 <= m2 <= mmax.
c
c     nvoidt   Output    Number of unassigned mesh indices displayed.
c                          -1 if input indices are bad.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

c.... Initialize.

      nvoid  = 0
      nvoidt = 0

      call meshchk (k1, k2, l1, l2, m1, m2, nerrchk)

      if (nerrchk .ne. 0) then
        nvoidt = -1
        go to 999
      endif

c.... Display all unassigned mesh indices in the specified ranges.

 8210 format ('Mesh index             ',3i6,7x,'! UNASSIGNED')
 8220 format ('Mesh indices from      ',3i6,7x,'! UNASSIGNED' /
     &        '             to        ',3i6,7x,'! UNASSIGNED (',i7,')' )

      do 240 m = m1, m2               ! Loop over m.
        do 230 l = l1, l2             ! Loop over l.
          do 220 k = k1, k2           ! Loop over k.

            do 210 n = 1, npoints     ! Loop over points.

              if (apoint(n) .eq. ' ') go to 210
              if (kpoint(n) .ne. k)   go to 210
              if (lpoint(n) .ne. l)   go to 210
              if (mpoint(n) .ne. m)   go to 210

c....         A mesh point is assigned.

              if (nvoid .gt. 0) then  ! Display unassigned mesh indices.

                klmbeg = kbeg + (lbeg - 1) * kmax +
     &                   (mbeg - 1) * kmax * lmax
                klmend = kend + (lend - 1) * kmax +
     &                   (mend - 1) * kmax * lmax

                if (nvoid .eq. 1) then  ! Just one unassigned index.

                  write (   6, 8210) kbeg, lbeg, mbeg
                  write (iout, 8210) kbeg, lbeg, mbeg

                else                  ! More than one unassigned index.

                  write (   6, 8220) kbeg, lbeg, mbeg,
     &                               kend, lend, mend, nvoid
                  write (iout, 8220) kbeg, lbeg, mbeg, 
     &                               kend, lend, mend, nvoid

                endif                 ! Displayed unassigned indices.

                nvoid = 0

              endif

              go to 220               ! A mesh point is assigned.

  210       continue                  ! End of loop over points.

c....       A mesh point is not assigned.  Save indices.

            nvoid  = nvoid  + 1
            nvoidt = nvoidt + 1

            if (nvoid .eq. 1) then
              kbeg = k
              lbeg = l
              mbeg = m
            endif

            kend = k
            lend = l
            mend = m

  220     continue                    ! End of loop over k.
  230   continue                      ! End of loop over l.
  240 continue                        ! End of loop over m.

      if (nvoid .gt. 0) then          ! Display unassigned mesh indices.

         klmbeg = kbeg + (lbeg - 1) * kmax +
     &            (mbeg - 1) * kmax * lmax
         klmend = kend + (lend - 1) * kmax +
     &            (mend - 1) * kmax * lmax

        if (nvoid .eq. 1) then        ! Just one unassigned index.

          write (   6, 8210) kbeg, lbeg, mbeg
          write (iout, 8210) kbeg, lbeg, mbeg

        else                          ! More than one unassigned index.

          write (   6, 8220) kbeg, lbeg, mbeg, 
     &                       kend, lend, mend, nvoid
          write (iout, 8220) kbeg, lbeg, mbeg, 
     &                       kend, lend, mend, nvoid

        endif                         ! Displayed unassigned index.

      endif                           ! Tested for unassigned index.

  999 return

c.... End of subroutine meshvout.     (+1 line)
      end
