      subroutine openx (ius, aname, atype, iopt, isize)
 
c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  openx    Updated     1993 November 3 13:30.
c               openx    Originated  1991 March 12 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To connect I/O unit ius to the file aname, of type atype,
c               according to option iopt:
c                 -1:  Open if the file exists.  Otherwise, set isize = -1
c                      and return.
c                  0:  Open if the file exists.  Otherwise, create the file.
c                      If that fails, terminate with an error message.
c                  1:  Open if the file exists.  Otherwise, terminate with an
c                      error message.
c
c     Logic:    Entry points:  openx.
c openx         Called by:  define, geomend, getio, input, main, output.
c
c openx         Calls:  geomend.
c               System calls:  open.
c
c     Biblio:   FORTLIB-THEORY and FORTLIB-SUBS (Category 9).
c
c     Glossary:
c
c     aname     Input    Name of file to be opened.  A character expression,
c                          which may have meaningless trailing blanks.
c
c     atype     Input    File type:  'text', 'hardcopy' or 'absolute'.
c                          'text':      ASCII input file.
c                          'hardcopy':  ASCII output file
c                            (same as 'text' on UNICOS).
c                          'absolute':  word-addressable, random-access file.
c                            (not currently allowed).
c
c     iopt      Output   Option flag:
c                          -1:  If file exists, open.  If not, set isize = -1
c                               and return.
c                           0:  If file exists, open.  If not, create.
c                           1:  If file exists, open.  If not, terminate.
c
c     isize     In/Out   Size of file (absolute files only).
c                          Returned as -1 if iopt = -1 and the file does not
c                          exist.
c
c     ius       Input    Number of I/O unit to be connected to file aname.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c
 
c.... Input argument specifications.
 
      character*80 aname              ! Name of file to open.
      character*8 atype               ! File type specification.
 
c.... Local variables.
 
      common /aopenx/ acrop           ! Word "Opened" or "Cr/Opend".
      character*8 acrop               ! Word "Opened" or "Cr/Opend".
      common /aopenx/ astat           ! File status, 'old' or 'unknown'.
      character*8 astat               ! File status, 'old' or 'unknown'.
      common /aopenx/ atermess        ! Termination message.
      character*80    atermess        ! Termination message.
 
      common /aopenx/ ierr            ! Error flag from system.

c.... Initialize expected file status.
 
      if (iopt .eq. 0) then           ! Old or new file ok.
        astat = 'unknown'
        acrop = 'Cr/Opend'
      else                            ! File must exist.
        astat = 'old'
        acrop = 'Opened'
      endif
 
c.... Try to open an existing file, or create a file (iopt = 0).
 
      if ((atype .eq. 'text    ') .or. (atype .eq. 'hardcopy')) then
 
        open (unit=ius, iostat=ierr, file=aname, status=astat)
 
      elseif (atype .eq. 'absolute') then
        ierr = 1
      else                            ! Error in file type specification.
        ierr = 1
      endif                           ! Tested atype.
 
 8120 format (/ '  openx ERROR.  Opening file ',a,' on ',
     &  ' I/O unit=',i5 / '  iopt=',i2,'  ierr=',i5)
 
      if (ierr .ne. 0) then           ! File does not exist.
        if (iopt .eq. -1) then        ! Return a warning flag.
          isize = -1
cbug      write (   *, 8130) acrop, atype, aname, ius, isize
          go to 210
        elseif (iopt .eq. 0) then     ! Create a new file.
          go to 110
        else                          ! Terminate run.
          write (   *, 8120) aname, ius, iopt, ierr
          atermess = 'openx ERROR.  Opening a file.'
          call geomend (atermess)
        endif
      endif
 
      go to 130
 
c.... Try to create a new file.
 
  110 if ((atype .eq. 'text    ') .or. (atype .eq. 'hardcopy')) then
 
        open (unit=ius, iostat=ierr, file=aname, status='new')
 
      elseif (atype .eq. 'absolute') then
 
cuni        ierr = izcreate (ius, aname, isize)
        ierr = 1
 
      else                            ! Error in file type specification.
 
        ierr = 1
 
      endif                           ! Tested atype.
 
 8110 format (/ '  openx ERROR.  Creating file ',a,' on',
     &  ' I/O unit=',i5 / '  iopt=',i2,'  ierr=',i5)
 
      if (ierr .ne. 0) then           ! Error opening or creating file.
        write (   *, 8110) aname, ius, iopt, ierr
        atermess = 'openx ERROR.  Creating a file.'
        call geomend (atermess)
      endif
 
c.... File has been created or opened.
 
 8130 format (2x,a,1x,a,' file ',a,' on I/O unit ',i5,'.  Size=',i9)
 
  130 continue
cbug      write (   *, 8130) acrop, atype, aname, ius, isize
 
  210 return
 
c.... End of subroutine openx.        (+1 line).
      end
