      subroutine opout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  opout    Updated     2004 August 26 14:30.
c               opout    Originated  1993 November 11 17:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the value of operators n1 to n2, which may be
c               inversion, reflection, rotation or scaling operators.
c
c     Logic:    Entry points:  opout.
c opout         Called by:  copy, delete, operator, ptoper, quoper, renamea,
c               renamex, search, voper.
c
c opout         Calls:  aptdet3, aptrefq, aptrotq, aptvunz.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 amount
      real        s(3,3)              ! Elements of 3 x 3 operator.

c.... Initialize.

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. nopers) nn2 = nopers

 8100 format ('operator  ',a,2x,a,2x,a)
 8105 format ('inverse   ',a,2x,a,2x,a)
 8110 format ('  op11,op12,op13  ',1p3e20.12 /
     &        '  op21,op22,op23  ',1p3e20.12 /
     &        '  op31,op32,op33  ',1p3e20.12)
 8115 format ('  in11,in12,in13  ',1p3e20.12 /
     &        '  in21,in22,in23  ',1p3e20.12 /
     &        '  in31,in32,in33  ',1p3e20.12)

      do 110 n = nn1, nn2             ! Loop over operators.

        if (aoper(n) .eq. ' ') go to 110
        write (   6, 8100) aoper(n), aoptype(n), aoptyps(n)
        write (iout, 8100) aoper(n), aoptype(n), aoptyps(n)
        write (   6, 8110) ((toper(i,j,n), j = 1, 3), i = 1, 3)
        write (iout, 8110) ((toper(i,j,n), j = 1, 3), i = 1, 3)
        write (   6, 8105) aoper(n), aoptype(n), aoptyps(n)
        write (iout, 8105) aoper(n), aoptype(n), aoptyps(n)
        write (   6, 8110) ((topin(i,j,n), j = 1, 3), i = 1, 3)
        write (iout, 8110) ((topin(i,j,n), j = 1, 3), i = 1, 3)

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

c....   Analyse operator.

c....   Find determinant.

        s(1,1) = toper(1,1,n)
        s(1,2) = toper(1,2,n)
        s(1,3) = toper(1,3,n)
        s(2,1) = toper(2,1,n)
        s(2,2) = toper(2,2,n)
        s(2,3) = toper(2,3,n)
        s(3,1) = toper(3,1,n)
        s(3,2) = toper(3,2,n)
        s(3,3) = toper(3,3,n)

        call aptdet3 (s, tol, det)

 8118 format (/ '  Determinant     ',1pe20.12)

        write (   6, 8118) det
        write (iout, 8118) det



c=======================================================================********

c....   Uniform scaling operator.

 8120   format ('  Scale factor,inv',1p2e20.12 )

        if ((aoptype(n) .eq. 'scale')    .and.
     &      (aoptyps(n) .eq. 'uniform')) then

          ratio = toper(1,1,n)
          ratin = 1.e99
          if (ratio .ne. 0.0) ratin = 1.0 / ratio

          write (   6, 8120) ratio, ratin
          write (iout, 8120) ratio, ratin

          go to 110

        endif

c=======================================================================********

c....   Linear scaling operator.

 8130   format ('  Scale factor,inv',1p2e20.12 /
     &          '  Direction (+,-) ',1p3e20.12 )
 8132 format ('  opout ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Bad operator.')

        if ((aoptype(n) .eq. 'scale')  .and.
     &      (aoptyps(n) .eq. 'linear')) then

          ratio = toper(1,1,n) + toper(2,2,n) + toper(3,3,n) - 2.0
          ratin = 1.e99
          if (ratio .ne. 0.0) ratin = 1.0 / ratio

          if ((ratio .ge. (1.0 - tol)) .and.
     &        (ratio .le. (1.0 + tol))) ratio = 1.0
          if ((ratio .ge. (-1.0 - tol)) .and.
     &        (ratio .le. (-1.0 + tol))) ratio = -1.0

          if (abs (ratio) .eq. 1.0) then
            ux = 1.0
            uy = 0.0
            uz = 0.0
          else
            ux = sqrt ((toper(1,1,n) - 1.0) / (ratio - 1.0))
            if (ux .ne. 0.0) then
              uy = (toper(1,2,n) / (ratio - 1.0)) / ux
              uz = (toper(1,3,n) / (ratio - 1.0)) / ux
            else
              uy = sqrt ((toper(2,2,n) - 1.0) / (ratio - 1.0))
              if (uy .ne. 0.0) then
                ux = (toper(2,1,n) / (ratio - 1.0)) / uy
                uz = (toper(2,3,n) / (ratio - 1.0)) / uy
              else
                uz = sqrt ((toper(3,3,n) - 1.0) / (ratio - 1.0))
                if (uz .ne. 0.0) then
                  ux = (toper(3,1,n) / (ratio - 1.0)) / uz
                  uy = (toper(3,2,n) / (ratio - 1.0)) / uz
                else
                  ux = -1.e99
                  uy = -1.e99
                  uz = -1.e99
                  write (   6, 8132) ainput, ninput
                  write (iout, 8132) ainput, ninput
                endif
              endif
            endif
          endif

          write (   6, 8130) ratio, ratin, ux, uy, uz
          write (iout, 8130) ratio, ratin, ux, uy, uz

          go to 110

        endif

c=======================================================================********

c....   Radial scaling operator.

 8140   format ('  Scale factor,inv',1p2e20.12 /
     &          '  Axis (+,-)      ',1p3e20.12 )
 8142 format ('  opout ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Bad operator.')

        if ((aoptype(n) .eq. 'scale')  .and.
     &      (aoptyps(n) .eq. 'radial')) then

          ratio = 0.5 * (toper(1,1,n) + toper(2,2,n) +
     &                   toper(3,3,n) - 1.0)
          ratin = 1.e99
          if (ratio .ne. 0.0) ratin = 1.0 / ratio

          if ((ratio .ge. (1.0 - tol)) .and.
     &        (ratio .le. (1.0 + tol))) ratio = 1.0
          if ((ratio .ge. (-1.0 - tol)) .and.
     &        (ratio .le. (-1.0 + tol))) ratio = -1.0
          stol = sqrt (tol)

          if (abs (ratio) .eq. 1.0) then
            ux = 1.0
            uy = 0.0
            uz = 0.0
          else
            ux = sqrt ((toper(1,1,n) - ratio) / (1.0 - ratio))
            if (ux .gt. stol) then
              uy = (toper(1,2,n) / (1.0 - ratio)) / ux
              uz = (toper(1,3,n) / (1.0 - ratio)) / ux
            else
              uy = sqrt ((toper(2,2,n) - ratio) / (1.0 - ratio))
              if (uy .gt. stol) then
                ux = (toper(2,1,n) / (1.0 - ratio)) / uy
                uz = (toper(2,3,n) / (1.0 - ratio)) / uy
              else
                uz = sqrt ((toper(3,3,n) - ratio) / (1.0 - ratio))
                if (uz .gt. stol) then
                  ux = (toper(3,1,n) / (1.0 - ratio)) / uz
                  uy = (toper(3,2,n) / (1.0 - ratio)) / uz
                else
                  ux = -1.e99
                  uy = -1.e99
                  uz = -1.e99
                  write (   6, 8142) ainput, ninput
                  write (iout, 8142) ainput, ninput
                endif
              endif
            endif
          endif

          write (   6, 8140) ratio, ratin, ux, uy, uz
          write (iout, 8140) ratio, ratin, ux, uy, uz

          go to 110

        endif

c=======================================================================********

c....   Reflection operator.

 8160   format ('  Scale factor,inv',1p2e20.12 /
     &          '  Direction (+,-) ',1p3e20.12 )

        if (aoptype(n) .eq. 'reflect') then

          call aptrefq (toper(1,1,n), tol, ux, uy, uz, nerra)

          if (nerra .eq. 0) then
            ratio = -1.0
            ratin = -1.0
            write (   6, 8160) ratio, ratin, ux, uy, uz
            write (iout, 8160) ratio, ratin, ux, uy, uz
            go to 110
          endif

        endif

c=======================================================================********

c....   Rotation operator.

 8170   format ('  Rotation angle  ',1pe20.12 /
     &          '    around axis   ',1p3e20.12 /
     &          '  or x,y,z angles ',1p3e20.12 )

        if (aoptype(n) .eq. 'rotate') then

          call aptrotq (toper(1,1,n), iunit, tol, theta, ux, uy, uz,
     &                  thx, thy, thz, nerra)
  
          if (nerra .eq. 0) then
  
            call aptvunz (ux, uy, uz, tol, vlength)
  
            if (vlength .ne. 0.0) then
  
              write (   6, 8170) theta, ux, uy, uz,
     &          thx, thy, thz
              write (iout, 8170) theta, ux, uy, uz,
     &          thx, thy, thz
  
            endif
  
          endif

        endif

c=======================================================================********

c....   Inversion operator.

 8180   format ('  Scale factor    ',1pe20.12 )

        if (aoptype(n) .eq. 'invert') then

          ratio = -1.0
  
          write (   6, 8180) ratio
          write (iout, 8180) ratio

        endif

c=======================================================================********

        endif                         ! Display maximum output.

  110 continue                        ! End of loop over operators.

  999 return

c.... End of subroutine opout.        (+1 line)
      end
