      subroutine polygon

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  polygon  Updated     2005 April 5 17:45.
c               polygon  Originated  1994 February 1 17:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create regular polygons by specifying a center,
c               a vertex point, and a coplanar point.
c
c     Logic:    Entry points:  polygon.
c polygon       Called by:  helpall, main.
c
c polygon       Calls:  aptpoly, aptsubr, incrsubs, namechk, pgout, ptfind,
c               varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount             ! Amount of output to write in pgout.

c.... Input line:  polygon [help]

 8110 format (
     &  '  Command "polygon":  acceptable input lines include:' /
     &  '  "polygon [help]"  Display this message.' /
     &  '  "polygon all"     Display all polygons.' /
     &  '  "polygon list [PGNAME1 PGNAME2 PGNAME3] ..."' /
     &  '       Display all listed regular polygons (default is',
     &  ' all regular polygons).' /
     &  '  "polygon PGNAME"  Display regular polygon PGNAME.' //
     &  '  "polygon PGNAME fit NVER PCEN PVER PCOP"' /
     &  '       Create regular polygon PGNAME with NVER',
     &  ' vertices,' /
     &  '       centered at point PCEN, with the first vertex at',
     &  ' point PVER,' /
     &  '       and coplanar with point PCOP.' //
     &  '  Note:  use +/- for PGNAME to incr/decr the last polygon',
     &  ' name:  ',a //
     &  '  Synonyms:  [help, h], [polygon, polyg, pg].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110) apolygs
        write (iout, 8110) apolygs
        go to 999
      endif

c===============================================================================

c.... Input line:  polygon all
c.... Input line:  polygon list

 8120 format (/ 'Number of polygons = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        npolygx = 0
        do 120 n = 1, npolygs
          if (apolyg(n) .eq. ' ') go to 120
          npolygx = npolygx + 1
  120   continue
        if (npolygx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call pgout (1, npolygs, amount)
        write (   6, 8120) npolygx
        write (iout, 8120) npolygx
        go to 999

      endif

c===============================================================================

c.... Input line:  polygon list PGNAME1 PGNAME2 PGNAME3 ...

 8130 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polygon does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, npolygs
            call aptsubr (apolyg(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. apolyg(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              call pgout (n, n, 'min')
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  polygon PGNAME

c.... See if user wants one polygon displayed.

      if (nword .eq. 2) then
        do 130 n = 1, npolygs
          if (aword(2) .eq. apolyg(n)) then
            amount = 'maximum'
            call pgout (n, n, amount)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8135 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "polygon" line not 7:  ',i3)
      if (nword .ne. 7) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        go to 999
      endif

 8136 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of polygon not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

 8138 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of vertices bad, or not 3 -',i5,':  ',a)

      call varfind (4, 4)

      if (mtype(4) .ne. 2) then
        write (   6, 8138) ainput, ninput, nverpgm, aword(4)
        write (iout, 8138) ainput, ninput, nverpgm, aword(4)
        go to 999
      endif
 
      nverpgx = iword(4)

      if ((nverpgx .lt. 3) .or. (nverpgx .gt. nverpgm)) then
        write (   6, 8138) ainput, ninput, aword(4)
        write (iout, 8138) ainput, ninput, aword(4)
        go to 999
      endif

 8137 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      if (aword(3) .ne. 'fit') then
        write (   6, 8137) ainput, ninput
        write (iout, 8137) ainput, ninput
        go to 999
      endif

c===============================================================================

c.... See if last polygon name should be incremented and used.

 8128 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last polygon can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = apolygs
        lword(2) = lapolygs
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, apolygs
          write (iout, 8128) ainput, ninput, apolygs
          go to 999
        endif
      endif

 8129 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last polygon can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = apolygs
        lword(2) = lapolygs
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, apolygs
          write (iout, 8129) ainput, ninput, apolygs
          go to 999
        endif
      endif

 8905 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of polygon too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if polygon exists.

 8160 format ('  WARNING:  replacing existing polygon:  ',a)

      nrep = 0
      do 136 n = 1, npolygs
        if (aword(2) .eq. apolyg(n)) then
          nrep = 1
          write (   6, 8160) apolyg(n)
          write (iout, 8160) apolyg(n)
          call pgout (n, n, 'min')
          go to 138
        endif
  136 continue

      do 137 n = 1, npolygs
        if (apolyg(n) .eq. ' ') go to 138
  137 continue

      npolygs = npolygs + 1

 8165 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of regular polygons exceeds limit ',i5)

      if (npolygs .gt. npolygm) then
        npolygs = npolygm
        write (   6, 8165) ainput, ninput, npolygm
        write (iout, 8165) ainput, ninput, npolygm
        go to 999
      endif

      n = npolygs

  138 nerr = 0

c===============================================================================

c.... Input line:  polygon PGNAME fit NVER PCEN PVER PCOP

c.... Find the three points defining the polygon.

      call ptfind (5, ncen)

      if (ncen .eq. 0) then
        nerr = 1
        go to 150
      endif

  150 call ptfind (6, nver)

      if (nver .eq. 0) then
        nerr = 1
        go to 170
      endif

  170 call ptfind (7, ncop)

      if (ncop .eq. 0) then
        nerr = 1
        go to 190
      endif

c.... Find the vertex points and other polygon data.

 8190 format ('  polygon ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Regular polygon is bad.  nerra = ',i3,'  area = ',1pe20.12)

  190 call aptpoly (x(ncen), y(ncen), z(ncen),
     &              x(nver), y(nver), z(nver),
     &              x(ncop), y(ncop), z(ncop), nverpgx, tol,
     &              xver, yver, zver, sedge, rin, rout, area, nerra)
  
      if ((nerra .ne. 0) .or. (area .eq. 0.0)) then
        nerr = 1
        write (   6, 8190) ainput, ninput, nerra, area
        write (iout, 8190) ainput, ninput, nerra, area
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          apolyg(n) = ' '
          if (n .eq. npolygs) npolygs = npolygs - 1
        endif
        go to 999
      endif

c.... Store and display the polygon specifications.

      apolyg(n) = aword(2)
      apolygs   = aword(2)
      lapolygs  = lword(2)

      nverpg(n) = nverpgx
      acenpg(n) = apoint(ncen)
      averpg(n) = apoint(nver)
      acoppg(n) = apoint(ncop)

      amount = 'maximum'
      call pgout (n, n, amount)

  999 return

c.... End of subroutine polygon.      (+1 line)
      end
