      subroutine ptdist (np, nq)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ptdist   Updated     1996 November 11 15:20.
c               ptdist   Originated  1994 September 19 13:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find which side of a quadric surface a point is on,
c               the minimum distance from the point to the surface, and the
c               coordinates of, and direction to, the proximal point.
c
c     Logic:    Entry points:  ptdist.
c ptdist        Called by:  distance, ptzone, side.
c
c ptdist        Calls:  aptqexv, aptqnor, aptqprt, aptrkis, aptscat, aptvdis,
c               aptvusz, aptwhis, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     np        Input    Index of point.
c
c     nq        Input    Index of quadric surface.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*16 aside
      common /lptdist/ cx(4)          ! X coordinate of surface point.
      common /lptdist/ cy(4)          ! Y coordinate of surface point.
      common /lptdist/ cz(4)          ! Z coordinate of surface point.
      common /lptdist/ dist(4)        ! Distance to quadric surface.
      common /lptdist/ ex(2)          ! X coordinate of extreme point.
      common /lptdist/ ey(2)          ! Y coordinate of extreme point.
      common /lptdist/ ez(2)          ! Z coordinate of extreme point.
      common /lptdist/ adir(2)        ! Type of extremum.
      character*8 adir                ! Type of extremum.

c=======================================================================********

c.... Initialize.

      pdist   = -1.e99

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx  = -1.e99
      proxvy  = -1.e99
      proxvz  = -1.e99

c=======================================================================********

c.... Find the proximal point on the quadric surface.

      call aptwhis (x(np), y(np), z(np), qc(nq),
     &              qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              dside, vgoodx, vgoody, vgoodz, nprox, cx, cy, cz,
     &              dist, nerra)

c.... Note: (vgoodx, vgoody, vgoodz) is the normal vector at external point P.

 8170 format (/ 'Side measure      ',1pe20.12,2x,a /
     &        'Normal pnx,pny,pnz',1p3e20.12 )
      aside = ' '
      if (dside .eq. 0.0) then
        aside = '***ON SURFACE***'

        pdist   = 0.0

        proxptx = x(np)
        proxpty = y(np)
        proxptz = z(np)

      endif

      proxvx  = vgoodx
      proxvy  = vgoody
      proxvz  = vgoodz

      write (   6, 8170) dside, aside, vgoodx, vgoody, vgoodz
      write (iout, 8170) dside, aside, vgoodx, vgoody, vgoodz

      if (dside .eq. 0.0) go to 999

c.... The method above should always work, unless the coefficients of the
c....   quadric surface are really strange.

      if (nprox .ge. 1) then          ! Found proximal point(s).

        pdist = dist(1)

 8194   format ('Three points on a proximal circle:')
 8196   format ('Four points on a proximal sphere:')
 8190   format ('Proximal x,y,z    ',1p3e20.12)
 8191   format ('Proximal dx,dy,dz ',1p3e20.12)
 8192   format ('  Exact distance  ',1pe20.12)
 8193   format ('Normal qnx,qny,qnz',1p3e20.12)

        if (nprox .eq. 3) then
          write (   6, 8194)
          write (iout, 8194)
        elseif (nprox .eq. 4) then
          write (   6, 8196)
          write (iout, 8196)
        endif

        do 210 n = 1, nprox

c....     Find normal vector on surface, at proximal point.

          call aptqnor (cx(n), cy(n), cz(n),
     &                  qc(nq), qx(nq), qy(nq), qz(nq),
     &                  qxy(nq), qyz(nq), qzx(nq),
     &                  qxx(nq), qyy(nq), qzz(nq), tol,
     &                  sc, sx, sy, sz, slen, nerr)

          write (   6, 8193) sx, sy, sz
          write (iout, 8193) sx, sy, sz

c....     Find distance from point to proximal point.

          call aptvdis (x(np), y(np), z(np), cx(n), cy(n), cz(n),
     &                  1, tol, dx, dy, dz, distx, nerra)

          write (   6, 8190) cx(n), cy(n), cz(n)
          write (iout, 8190) cx(n), cy(n), cz(n)
          call plotpt (cx(n), cy(n), cz(n), nerrg)

          write (   6, 8191) dx, dy, dz
          write (iout, 8191) dx, dy, dz


          if (n .eq. 1) then
            proxptx = cx(n)
            proxpty = cy(n)
            proxptz = cz(n)
  
            proxvx = dx
            proxvy = dy
            proxvz = dz
          endif

          write (   6, 8192) dist(n)
          write (iout, 8192) dist(n)

  210   continue

        go to 999

      endif                           ! Found proximal point(s).

c=======================================================================********

c.... Try to find the proximal point using extrema.

  110 if (nprox .eq. -1) then         ! No track intersection.

 8110   format (/ 'Iterate using extrema.')

        write (   6, 8110)
        write (iout, 8110)

 8175   format ('  Estimated dist  ',1pe20.12)

        write (   6, 8175) dist(1)
        write (iout, 8175) dist(1)

c....   Find the first test vector.

        call aptvdis (x(np), y(np), z(np),
     &                cx(1), cy(1), cz(1), 1, tol,
     &                dx, dy, dz, vlength, nerra)

 8178   format ('  Vector (test)   ',1p3e20.12 /
     &          '  Dist   (test)   ',1pe20.12 )
 8177   format ('  Use random vect ',1p3e20.12)

        write (   6, 8178) dx, dy, dz, vlength
        write (iout, 8178) dx, dy, dz, vlength

        if (vlength .eq. 0.0) then
          call aptscat (1, dx, dy, dz, nerra)
          vlength = 1.0
          write (   6, 8177) dx, dy, dz
          write (iout, 8177) dx, dy, dz
        endif

c....   Slightly randomize the test vector.

        dx = dx - tol * vlength
        dz = dz + tol * vlength

        distmin = 1.e99

c....   Find any intersection in the direction of the test vector.

        dintmn = -1.e99
        dintmx =  1.e99

        call aptrkis (x(np), y(np), z(np), dx, dy, dz,
     &                qc(nq), qx(nq), qy(nq), qz(nq),
     &                qxy(nq), qyz(nq), qzx(nq),
     &                qxx(nq), qyy(nq), qzz(nq),
     &                dintmn, dintmx, 1, tol,
     &                nint, cx(1), cy(1), cz(1), dist(1),
     &                snx, sny, snz, vcos, nerra)

 8181   format ('  No intersection in test direction.')

        if (nint .eq. 0) then
          write (   6, 8181)
          write (iout, 8181)
          go to 115
        endif

        if (dist(1) .lt. 0.0) then
          dist(1) = -dist(1)
          vcos    = -vcos
          dx      = -dx
          dy      = -dy
          dz      = -dz
        endif
cbugc____ DEBUG begins.
cbug 8202 format ('* norm vect at int',1p3e20.12)
cbug      write (   6, 8202) snx, sny, snz
cbug      write (iout, 8202) snx, sny, snz
cbug      write (   6, 8215) vcos
cbug      write (iout, 8215) vcos
cbugc____ DEBUG ends.

        call aptvusz (dx, dy, dz, tol, udx, udy, udz, dlen)

        distmin = abs (dist(1))
        vgoodx  = udx * dist(1)
        vgoody  = udy * dist(1)
        vgoodz  = udz * dist(1)

 8112   format ('  Vector (test)   ',1p3e20.12 /
     &          '  Dist   (test)   ',1pe20.12 )

        write (   6, 8112) vgoodx, vgoody, vgoodz, distmin
        write (iout, 8112) vgoodx, vgoody, vgoodz, distmin

c....   Find any extrema in test direction, last extrema.  Iterate.

  115   nextrs  = 0

        ntry = 0
  120   ntry = ntry + 1

        call aptqexv (dx, dy, dz, qc(nq), qx(nq), qy(nq), qz(nq),
     &                qxy(nq), qyz(nq), qzx(nq),
     &                qxx(nq), qyy(nq), qzz(nq), tol,
     &                nextr, ex, ey, ez, adir, nerr)

 8160   format ('No additional extrema exist.  Found ',i3,'.')
 8179   format ('  Extremum ',i3,' in test direction or direction',
     &    ' of last extremum:')
 8150   format ('  ',a,'   (xyz)',1p3e20.12)

        if (nextr .eq. 0) then        ! No extrema.

          ntry = ntry - 1
          write (   6, 8160) ntry
          write (iout, 8160) ntry

          nprox   = 0
          dist(1) = vlength
cbugc____ DEBUG begins.
cbug 9601 format ('  No more extrema.  dist(1) = ',1pe20.12)
cbug      write (   6, 9601) dist(1)
cbug      write (iout, 9601) dist(1)
cbugc____ DEBUG ends.

          if (nextrs .eq. 1) then
            go to 110
          endif

        else                          ! Found extrema.

          nextrs  = 1
          write (   6, 8179) ntry
          write (iout, 8179) ntry

          distsq1 = (ex(1) - x(np))**2 +
     &              (ey(1) - y(np))**2 +
     &              (ez(1) - z(np))**2

          distsq2 = (ex(2) - x(np))**2 +
     &              (ey(2) - y(np))**2 +
     &              (ez(2) - z(np))**2


          if (distsq1 .le. distsq2) then
            n = 1
            write (   6, 8150) adir(1), ex(1), ey(1), ez(1)
            write (iout, 8150) adir(1), ex(1), ey(1), ez(1)
          else
            n = 2
            write (   6, 8150) adir(2), ex(2), ey(2), ez(2)
            write (iout, 8150) adir(2), ex(2), ey(2), ez(2)
          endif

c....     Find vector, distance from point to extremum.

          call aptvdis (x(np), y(np), z(np), ex(n), ey(n), ez(n),
     &                  1, tol, dx, dy, dz, vlength, nerra)

 8176     format ('  Vector to extr  ',1p3e20.12 /
     &            '  Dist   to extr  ',1pe20.12 )

          if (vlength .lt. distmin) then
            write (   6, 8176) dx, dy, dz, vlength
            write (iout, 8176) dx, dy, dz, vlength
            distmin = vlength
            dist(1) = vlength
            pdist   = vlength
            vgoodx  = dx
            vgoody  = dy
            vgoodz  = dz
            go to 120
          endif

          nprox   = 0
          dist(1) = distmin

          go to 110                   ! Try general iterative method next.

        endif                         ! No track intersection.  Used extrema.

c=======================================================================********

c.... No exact proximal point.  Only know intersection of normal track,
c....   or no extrema, or last extremum was not at minimum distance.

      elseif (nprox .eq. 0) then      ! Try general iterative method.

 8310 format (/ 'Iterate with center of curvature and tangent plane',
     &  ' method.')

      write (   6, 8310)
      write (iout, 8310)

      vtestx = vgoodx
      vtesty = vgoody
      vtestz = vgoodz

      call aptqprt (x(np), y(np), z(np), vtestx, vtesty, vtestz,
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              niter, nprox, sgoodx, sgoody, sgoodz, distgood,
     &              vgoodx, vgoody, vgoodz, snx, sny, snz,
     &              vcosmax, nerr)

      if     (nerr  .ne.  0) then

 8200   format ('  ptdist ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  Error in subroutine aptqprt.  nerr = ',i2)

        write (   6, 8200) ainput, ninput, nerr
        write (iout, 8200) ainput, ninput, nerr

      elseif (nprox .eq.  1) then

 8245   format (/ '  Distance and angle converged in',i6,
     &    ' iterations.')
 8212   format (/ 'Proximal x,y,z    ',1p3e20.12 /
     &            'Proximal dx,dy,dz ',1p3e20.12 /
     &            '  Exact distance  ',1pe20.12 )
 8215   format ('  Cosine of angle ',1pe20.12,'  (from normal)' )
 8220   format ('  Angle (',a,')',1pe20.12,'  (from normal)' )

        write (   6, 8245) niter
        write (iout, 8245) niter
        write (   6, 8212) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        write (iout, 8212) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        call plotpt (sgoodx, sgoody, sgoodz, nerrg)

        proxptx = sgoodx
        proxpty = sgoody
        proxptz = sgoodz

        proxvx = vgoodx
        proxvy = vgoody
        proxvz = vgoodz

        write (   6, 8215) vcosmax
        write (iout, 8215) vcosmax

c....   Find the angle between "d" and "sn".

        vang = acos (vcosmax)
        if (abs (vang) .le. tol) vang = 0.0
        if (abs (vang - pi) .le. tol) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        write (   6, 8220) angunit, vang
        write (iout, 8220) angunit, vang

        pdist = distgood

      elseif (nprox .eq.  0) then

 8275   format (/ '  Distance and angle did not converge in',
     &    i6,' iterations.')

        write (   6, 8275) niter
        write (iout, 8275) niter

 8400   format (/ 'Proximal x,y,z   ?',1p3e20.12 /
     &            'Proximal dx,dy,dz?',1p3e20.12 /
     &            '  Approx distance?',1pe20.12 )

        write (   6, 8400) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        write (iout, 8400) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood

        write (   6, 8215) vcosmax
        write (iout, 8215) vcosmax

c....   Find the angle between "d" and "sn".

        vang = acos (vcosmax)
        if (abs (vang) .le. tol) vang = 0.0
        if (abs (vang - pi) .le. tol) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        write (   6, 8220) angunit, vang
        write (iout, 8220) angunit, vang

      elseif (nprox .eq. -1) then

 8450   format (/ 'First trial direction missed quadric surface.' /
     &            '  vector     (xyz)',1p3e20.12)

        write (   6, 8450) vtestx, vtesty, vtestz
        write (iout, 8450) vtestx, vtesty, vtestz

      elseif (nprox .eq. -2) then

 8340   format (/ '  Normal vector has zero length.  Cannot proceed.')

        write (   6, 8340)
        write (iout, 8340)

      elseif (nprox .eq. -3) then

 8350   format (/ '  Did not converge.  Correction too small',
     &    ' to continue.')

        write (   6, 8350)
        write (iout, 8350)

      elseif (nprox .eq. -4) then

 8380   format (/  '  Missed quadric entirely 100 times.  Quit.')

        write (   6, 8380)
        write (iout, 8380)

      elseif (nprox .eq. -5) then

 8430   format (/  '  Failed to decrease distance 100',
     &    ' times.  Quit.')

        write (   6, 8430)
        write (iout, 8430)

      elseif (nprox .eq. -6) then

 8360   format (/ '  Test vector has zero length.  Cannot proceed.')

        write (   6, 8360)
        write (iout, 8360)

      endif                           ! Tested nerr, nprox from aptqprt.

      endif                           ! Tested nprox from aptwhis.

c=======================================================================********

  999 return

c.... End of subroutine ptdist.       (+1 line)
      end
