      subroutine ptzone (kopt, np, nz, amount, dside, nside, nerra)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ptzone   Updated     2007 January 24 16:40.
c               ptzone   Originated  1994 March 17 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To see if a point is in a zone, and the distances from the point
c               to each of the bounding surfaces of the zone.
c
c     Logic:    Entry points:  ptzone
c ptzone        Called by:  cluster, distance, where.
c
c ptzone        Calls:  ptdist, quout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     amount    Input    Amount of output, "maximum" or "minimum".
c                          Use "maximum" for one point, one zone.
c
c     kopt      Input    Surface distance display option, 0 for none, 1 for all.
c
c     nerra     Output   1 if a zone surface is bad or does not exist.
c
c     np        Input    Index of a point.
c
c     nside     Output   1 if point is in zone.
c
c     nz        Input    Index of a zone.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Arguments.

      character*8 amount              ! Output control, "maximum" or "minimum".
      dimension dside (nzquadm)       ! Value of F(x,y,z) at point.

      nerra = 0

      nside = 1                       ! Assume point is in zone.

c.... Test point against each boundary surface of this zone.

      do 170 nzq = 1, nzquads(nz)     ! Loop over boundary surfaces.

 8140  format ('  ptzone ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  Named zone boundary surface does not exist.',
     &   '  Zone:  ',a,2x,a)

        do 150 nq = 1, nquads         ! Loop over all quadrics.
          if (aquad(nq) .eq. azquad(nzq,nz)) then
            n = nq
            go to 160
          endif
  150   continue                      ! End of loop over all quadrics
        write (   6, 8140) ainput, ninput, azone(nz), azquad(nzq,nz)
        write (iout, 8140) ainput, ninput, azone(nz), azquad(nzq,nz)
        nerra = 1
        go to 999

  160   continue

 8160   format ('  ptzone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Zone boundary surface is bad.',
     &    '  Zone:  ',a,2x,a)

        if (amount .eq. 'maximum') then

          call quout (n, n, 'min', nerrq)

          if (nerrq .ne. 0) then
            write (   6, 8160) ainput, ninput, azone(nz), azquad(nzq,nz)
            write (iout, 8160) ainput, ninput, azone(nz), azquad(nzq,nz)
            nerra = 1
            go to 999
          endif
        endif

c....   Find the side of the quadric surface the point is on.

        dside(nzq) = qc(n) +
     &          qx(n)  * x(np) +
     &          qy(n)  * y(np) +
     &          qz(n)  * z(np) +
     &          qxy(n) * x(np) * y(np) +
     &          qyz(n) * y(np) * z(np) +
     &          qzx(n) * z(np) * x(np) +
     &          qxx(n) * x(np)**2 +
     &          qyy(n) * y(np)**2 +
     &          qzz(n) * z(np)**2

        derr  = tol * (abs (qc(n)) +
     &          abs (qx(n)   * x(np)) +
     &          abs (qy(n)   * y(np)) +
     &          abs (qz(n)   * z(np)) +
     &          abs (qxy(n)  * x(np) * y(np)) +
     &          abs (qyz(n)  * y(np) * z(np)) +
     &          abs (qzx(n)  * z(np) * x(np)) +
     &          abs (qxx(n)) * x(np)**2 +
     &          abs (qyy(n)) * y(np)**2 +
     &          abs (qzz(n)) * z(np)**2)

        if (abs (dside(nzq)) .lt. derr) then
          dside(nzq) = 0.0
        endif

 8170   format ('  dside, err      ',1p2e20.12 )

        if (kopt .eq. 0) then
          if (amount .eq. 'maximum') then
            write (   6, 8170) dside(nzq), derr
            write (iout, 8170) dside(nzq), derr
          endif
        endif

c....   See if the point is inside (dside(nzq) has opposite sign of azsign)
c....     or outside the zone.

        if ((dside(nzq) .gt. 0.0)      .and.
     &      (azsign(nzq,nz) .eq. '+')) then
          nside = 0
        endif

        if ((dside(nzq) .lt. 0.0)      .and.
     &      (azsign(nzq,nz) .eq. '-')) then
          nside = 0
        endif

        if (kopt .eq. 1) then

          call ptdist (np, n)

        endif

  170 continue                        ! End of loop over boundary surfaces.

  999 return

c.... End of subroutine ptzone.       (+1 line)
      end
