      subroutine qumove

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  qumove   Updated     2007 January 19 14:30.
c               qumove   Originated  1994 July 26 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To create a family of new quadric surfaces by repeatedly moving
c               a base quadric surface.
c
c     Logic:    Entry points:  qumove.
c qumove        Called by:  axisym, cone, cylinder, ellipsoid, plane, quadric,
c               sphere.
c
c qumove        Calls:  apttris, incrsubs, namechk, quout, qustore, qutype,
c               varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 anamex             ! Name of a quadric surface.

c=======================================================================********

c.... Input line:  axisym    AX02    move NUMAX   INC AX01    VMOVE
c.... Input line:  cone      CONE02  move NUMCONE INC CONE01  VMOVE
c.... Input line:  cylinder  CYL02   move NUMCYL  INC CYL01   VMOVE
c.... Input line:  ellipsoid ELL02   move NUMELL  INC ELL01   VMOVE
c.... Input line:  hyperb    HYP02   move NUMHYP  INC HYP01   VMOVE
c.... Input line:  plane     PL02    move NUMPL   INC PL01    VMOVE
c.... Input line:  quadric   QNAME02 move NUMQ    INC QNAME01 VMOVE
c.... Input line:  sphere    SPH02   move NUMSPH  INC SPH01   VMOVE

      nerr = 0

 8505 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words in input line:  ',i3)

      if (nword .ne. 7) then
        write (   6, 8505) ainput, ninput, nword
        write (iout, 8505) ainput, ninput, nword
        go to 999
      endif

 8520 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMQ is not a positive integer:  ',a)
 8525 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMQ is not an integer or integer variable:  ',a)

      call varfind (4, 4)

      if (mtype(4) .eq. 2) then
        numq = iword(4)
        if (numq .lt. 1) then
          write (   6, 8520) ainput, ninput, aword(4)
          write (iout, 8520) ainput, ninput, aword(4)
          go to 999
        endif
      else
        write (   6, 8525) ainput, ninput, aword(4)
        write (iout, 8525) ainput, ninput, aword(4)
        go to 999
      endif

 8510 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is zero.')
 8515 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is not an integer or integer variable:  ',a)

      call varfind (5, 5)

      if (mtype(5) .eq. 2) then
        inc = iword(5)
        if (inc .eq. 0) then
          write (   6, 8510) ainput, ninput
          write (iout, 8510) ainput, ninput
          go to 999
        endif
      else
        write (   6, 8515) ainput, ninput, aword(5)
        write (iout, 8515) ainput, ninput, aword(5)
        go to 999
      endif

c.... Find the base quadric.

 8530 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      nbase = 0
      do 530 nq = 1, nquads
        if (aword(6) .eq. aquad(nq)) then
          nbase = nq
          call quout (nbase, nbase, 'min', nerrq)
          go to 540
        endif
  530 continue
      write (   6, 8530) ainput, ninput, aword(6)
      write (iout, 8530) ainput, ninput, aword(6)
      go to 999

c.... See if the quadric type matches.

  540 call qutype (aword(1), nbase, 1, nerrq)

      if (nerrq .ne. 0) go to 999

c.... Find the translation vector.

      call vectfind (7, nv)

      if (nv .eq. 0) go to 999

c.... See if the translation vector has nonzero length.

 8560 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Translation vector has zero length:  ',a)

      if (nv .gt. 0) then             ! Translation vector exists.
        if (vlen(nv) .eq. 0.0) then
          write (   6, 8560) ainput, ninput, aword(7)
          write (iout, 8560) ainput, ninput, aword(7)
          go to 999
        endif
      endif                           ! Translation vector exists.

c.... See if the quadric names can be generated uniquely.

 8570 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' quadric names from:  ',a)

 8905 format ('  qumove ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of quadric too dangerous to use:  ',a)

      anamex  = aword(2)
      lanamex = lword(2)
      namex   = inc * (numq - 1)

      call incrsubs (namex, anamex, 1, lanamex, nerra)

      if (nerra .ne. 0) then
        write (   6, 8570) ainput, ninput, numq, aword(2)
        write (iout, 8570) ainput, ninput, numq, aword(2)
        go to 999
      endif

      call namechk (anamex, lanamex, nerrchk)

      if (nerrchk .ne. 0) then
        go to 999
        write (   6, 8905) ainput, ninput, anamex
        write (iout, 8905) ainput, ninput, anamex
      endif

      if (nerr .ne. 0) go to 999

c.... Save the base quadric coefficients.

      ac  = qc(nbase)
      ax  = qx(nbase)
      ay  = qy(nbase)
      az  = qz(nbase)
      axy = qxy(nbase)
      ayz = qyz(nbase)
      azx = qzx(nbase)
      axx = qxx(nbase)
      ayy = qyy(nbase)
      azz = qzz(nbase)

c.... Create the translated quadrics.

      anamex  = aword(2)              ! Initial quadric name.
      lanamex = lword(2)              ! Initial quadric name length.

      do 580 nqt = 1, numq            ! Loop over the new quadrics.

c....   Find the index in the quadric surface list for the new quadric.

        aword(2) = anamex
        lword(2) = lanamex

        call qustore (-1, nrep, n)

        if (n .eq. 0) go to 999

c....   Translate the base quadric by vector vname.

        call apttris (1, xcom(nv), ycom(nv), zcom(nv),
     &                ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &                tol, nerra)

        aquads    = anamex
        laquads   = lanamex

        aquad(n)  = anamex
        laquad(n) = lanamex
        qc(n)     = ac
        qx(n)     = ax
        qy(n)     = ay
        qz(n)     = az
        qxy(n)    = axy
        qyz(n)    = ayz
        qzx(n)    = azx
        qxx(n)    = axx
        qyy(n)    = ayy
        qzz(n)    = azz

        call quout (n, n, 'min', nerrq)

c....   Find the next quadric name, by incrementing by inc.

        call incrsubs (inc, anamex, 1, lanamex, nerra)

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
          go to 999
        endif

  580 continue                        ! End of loop over the new quadrics.

  999 return

c.... End of subroutine qumove.       (+1 line)
      end
