      subroutine quoper

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  quoper   Updated     2007 January 19 15:30.
c               quoper   Originated  1994 July 26 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To create a family of new quadric surfaces by repeatedly
c               rotating or scaling a base quadric surface.
c
c     Logic:    Entry points:  quoper.
c quoper        Called by:  axisym, cone, cylinder, ellipsoid, plane, quadric,
c               sphere.
c
c quoper        Calls:  aptrois, incrsubs, namechk, opout, ptfind, quout,
c               qustore, qutype, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 anamex             ! Name of a quadric surface.
      character*8  aopx               ! Operation type, "rotate" or "scale".

c=======================================================================********

c.... Input line:  cone      CONE02  rotate NUMCONE INC CONE01  OPNAME PINV
c.... Input line:  cylinder  CYL02   rotate NUMCYL  INC CYL01   OPNAME PINV
c.... Input line:  ellipsoid ELL02   rotate NUMELL  INC ELL01   OPNAME PINV
c.... Input line:  hyperb    HPNAME2 rotate NUMHYP  INC HPNAME1 OPNAME PINV
c.... Input line:  quadric   QNAME02 rotate NUMQ    INC QNAME01 OPNAME PINV
c.... Input line:  sphere    SPH02   rotate NUMSPH  INC SPH01   OPNAME PINV
c.... Input line:  axisym    AXQ02   scale  NUMAX   INC AXQ01   OPNAME PINV
c.... Input line:  cone      CONE02  scale  NUMCONE INC CONE01  OPNAME PINV
c.... Input line:  cylinder  CYL02   scale  NUMCYL  INC CYL01   OPNAME PINV
c.... Input line:  ellipsoid ELL02   scale  NUMELL  INC ELL01   OPNAME PINV
c.... Input line:  hyperb    HPNAME2 scale  NUMHYP  INC HPNAME1 OPNAME PINV
c.... Input line:  quadric   QNAME02 scale  NUMQ    INC QNAME01 OPNAME PINV
c.... Input line:  sphere    SPH02   scale  NUMSPH  INC SPH01   OPNAME PINV

      nerr = 0

 8605 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 8 words in input line:  ',i3)

      if (nword .ne. 8) then
        write (   6, 8605) ainput, ninput, nword
        write (iout, 8605) ainput, ninput, nword
        go to 999
      endif

 8620 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMQ is not a positive integer:  ',a)
 8625 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMQ is not an integer or integer variable:  ',a)

      call varfind (4, 4)

      if (mtype(4) .eq. 2) then
        numq = iword(4)
        if (numq .lt. 1) then
          write (   6, 8620) ainput, ninput, aword(4)
          write (iout, 8620) ainput, ninput, aword(4)
          go to 999
        endif
      else
        write (   6, 8625) ainput, ninput, aword(4)
        write (iout, 8625) ainput, ninput, aword(4)
        go to 999
      endif

 8610 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is zero.')
 8615 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is not an integer or integer variable:  ',a)

      call varfind (5, 5)

      if (mtype(5) .eq. 2) then
        inc = iword(5)
        if (inc .eq. 0) then
          write (   6, 8610) ainput, ninput
          write (iout, 8610) ainput, ninput
          go to 999
        endif
      else
        write (   6, 8615) ainput, ninput, aword(5)
        write (iout, 8615) ainput, ninput, aword(5)
        go to 999
      endif

c.... Find the base quadric.

 8630 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      nbase = 0
      do 630 nq = 1, nquads
        if (aword(6) .eq. aquad(nq)) then
          nbase = nq
          call quout (nbase, nbase, 'min', nerrq)
          go to 640
        endif
  630 continue
      write (   6, 8630) ainput, ninput, aword(6)
      write (iout, 8630) ainput, ninput, aword(6)
      go to 999

c.... See if the base quadric is the right type.

  640 call qutype (aword(1), nbase, 1, nerrq)

      if (nerrq .ne. 0) go to 999

c.... Find the operator.

 8650 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)

      do 650 n = 1, nopers
        if (aword(7) .eq. aoper(n)) then
          nop = n
          call opout (nop, nop, 'min')
          go to 660
        endif
  650 continue
      write (   6, 8650) ainput, ninput, aword(7)
      write (iout, 8650) ainput, ninput, aword(7)
      go to 999

c.... See if the operator type agrees with scale or rotate.

 8660 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Operator type is not ',a,':  ',a)

  660 if ((aword(3) .eq. 'rotate') .or.
     &    (aword(3) .eq. 'rot'))   then

        aopx = 'rotate'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8660) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8660) ainput, ninput, aopx, aoptype(nop)
          go to 999
        endif

      elseif ((aword(3) .eq. 'scale') .or.
     &        (aword(3) .eq. 'sc'))   then

        aopx = 'scale'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8660) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8660) ainput, ninput, aopx, aoptype(nop)
          go to 999
        endif

      endif

c.... Find the invariant point.

      call ptfind (8, np)

      if (np .eq. 0) go to 999

c.... See if the quadric names can be generated uniquely.

 8680 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' quadric names from:  ',a)
 8905 format ('  quoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of quadric too dangerous to use:  ',a)

      anamex  = aword(2)
      lanamex = lword(2)
      namex   = inc * (numq - 1)

      call incrsubs (namex, anamex, 1, lanamex, nerra)

      if (nerra .ne. 0) then
        write (   6, 8680) ainput, ninput, numq, aword(2)
        write (iout, 8680) ainput, ninput, numq, aword(2)
        go to 999
      endif

      call namechk (anamex, lanamex, nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, anamex
        write (iout, 8905) ainput, ninput, anamex
        go to 999
      endif

      if (nerr .ne. 0) go to 999

c.... Save the base quadric coefficients.

      ac  = qc(nbase)
      ax  = qx(nbase)
      ay  = qy(nbase)
      az  = qz(nbase)
      axy = qxy(nbase)
      ayz = qyz(nbase)
      azx = qzx(nbase)
      axx = qxx(nbase)
      ayy = qyy(nbase)
      azz = qzz(nbase)

c.... Create the rotated or scaled quadrics.

      anamex  = aword(2)              ! Initial quadric name.
      lanamex = lword(2)              ! Initial quadric name.

      do 690 nqr = 1, numq            ! Loop over the new quadrics.

c....   Find the index in the quadric surface list for the new quadric.

        aword(2) = anamex
        lword(2) = lanamex

        call qustore (-1, nrep, n)

        if (n .eq. 0) go to 999

c....   Rotate or scale the base quadric.

        if (aopx .eq. 'rotate') then  ! Rotate.

          call aptrois (toper(1,1,nop), 0, x(np), y(np), z(np),
     &                  ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &                  tol, nerra)

        else                          ! Scale.  

          call aptrois (topin(1,1,nop), 0, x(np), y(np), z(np),
     &                  ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &                  tol, nerra)

        endif
cbugc***DEBUG begins.
cbug 9901 format ('quoper DEBUG.  nbug, n, aword, lword, aquad, laquad=' /
cbug     &  2i3,2x,a24,2x,i3,2x,a24,2x,i3)
cbug      nbug = 1
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

        aquads    = anamex
        laquads   = lanamex
 
        aquad(n)  = anamex
        laquad(n) = lanamex
        qc(n)     = ac
        qx(n)     = ax
        qy(n)     = ay
        qz(n)     = az
        qxy(n)    = axy
        qyz(n)    = ayz
        qzx(n)    = azx
        qxx(n)    = axx
        qyy(n)    = ayy
        qzz(n)    = azz

cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.
        call quout (n, n, 'min', nerrq)
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

c....   Find the next quadric name, by incrementing by inc.

        call incrsubs (inc, anamex, 1, lanamex, nerra)

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
          go to 999
        endif
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

  690 continue                        ! End of loop over the new quadrics.

  999 return

c.... End of subroutine quoper.       (+1 line)
      end
