      subroutine quout (n1, n2, amount, nerrq)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  quout    Updated     2007 January 24 16:40.
c               quout    Originated  1993 December 1 13:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the data for quadric surfaces n1 to n2.
c
c     Logic:    Entry points:  quout
c quout         Called by:  accel, axisym, cone, copy, cylinder, delete,
c               distance, ellipsoid, extrema, hyperb, matrix, movex, operator,
c               plane, point, prox, ptzone, quadfind, quadric, qumove, quoper,
c               qustore, renamea, renamex, search, side, sliceq, sphere,
c               track, vector, znmake, zout.
c
c quout         Calls:  aptaxis, aptmopv, aptqext, aptqrts, aptrois, apttran,
c               apttris, aptvunz, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     nerrq     Output   Error indicator.  0 for none.  1 if all coefficients
c                          are zero.  2 if quadric surface is imaginary.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

c.... Real quadric surface types.  Coefficients must have the preceding sign.

c.... Plane, simple (type 0):                                 x = 0  (type  0).
c.... Planes, coincident:                                  x**2 = 0  (type  1).
c.... Planes, parallel:                           -1 + qxx*x**2 = 0  (type  2).
c.... Planes, intersecting:                   x**2 - |qyy|*y**2 = 0  (type  3).
c.... Cylinder, parabolic:  (u = y or z)         -|qy|*y + x**2 = 0  (type  4).
c.... Cylinder, hyperbolic:           1 + qxx*x**2 - |qyy|*y**2 = 0  (type  5).
c.... Cylinder, elliptic:            -1 + qxx*x**2 +  qyy *y**2 = 0  (type  6).
c.... Cylinder, circular:              -1 + qxx * (x**2 + y**2) = 0  (type  7).
c.... Paraboloid, hyperbolic:       -|qz|*z + x**2 - |qyy|*y**2 = 0  (type  8).
c.... Paraboloid, elliptic:         -|qz|*z + x**2 +  qyy *y**2 = 0  (type  9).
c.... Paraboloid, circular:               -|qz|*z + x**2 + y**2 = 0  (type 10).
c.... Cone, elliptic:              x**2 + qyy*y**2 - |qzz|*z**2 = 0  (type 11).
c.... Cone, circular:                  x**2 + y**2 - |qzz|*z**2 = 0  (type 12).
c.... Hyperboloid of 1 sheet:   -1+qxx*x**2+qyy*y**2-|qzz|*z**2 = 0  (type 13).
c.... Hyperboloid of 2 sheets:   1+qxx*x**2+qyy*y**2-|qzz|*z**2 = 0  (type 14).
c.... Ellipsoid:            -1 + qxx*x**2 + qyy*y**2 + qzz*z**2 = 0  (type 15).
c.... Sphere:                   -1 + qxx * (x**2 + y**2 + z**2) = 0  (type 16).

      include 'store.h'

      character*8 amount

c.... Local variables.

      common /lquout/ ex(6)           ! Coordinate x of extreme point.
      common /lquout/ ey(6)           ! Coordinate y of extreme point.
      common /lquout/ ez(6)           ! Coordinate z of extreme point.
      common /lquout/ adir(6)         ! Type of extreme point.
      character*8 adir                ! Type of extreme point.

c.... Initialize.
cbugc***DEBUG begins.
cbug 9901 format ('quout DEBUG 1.  n1=',i4,' n2=',i4,' aquad(n1)="',a,'"')
cbug      write (   6, 9901) n1, n2, aquad(n1)
cbug      write (iout, 9901) n1, n2, aquad(n1)
cbugc***DEBUG ends.

      nerrq = 0

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. nquads) nn2 = nquads

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1
cbugc***DEBUG begins.
cbug 9902 format ('quout DEBUG 2.  nn1=',i4,' nn2=',i4,
cbug     &  ' aquad(nn1)="',a,'"')
cbug      write (   6, 9902) nn1, nn2, aquad(nn1)
cbug      write (iout, 9902) nn1, nn2, aquad(nn1)
cbugc***DEBUG ends.

      do 120 n = nn1, nn2
cbugc***DEBUG begins.
cbug 9903 format ('quout DEBUG 3.  n=',i4,' aquad(n)="',a,'"')
cbug      write (   6, 9903) n, aquad(n)
cbug      write (iout, 9903) n, aquad(n)
cbugc***DEBUG ends.

      if (aquad(n) .eq. ' ') go to 120

c=======================================================================********

c.... Find the general form of the quadric surface.

      qlinear = qx(n)**2  + qy(n)**2  + qz(n)**2
      qcross  = qxy(n)**2 + qyz(n)**2 + qzx(n)**2
      qsquare = qxx(n)**2 + qyy(n)**2 + qzz(n)**2
cbugc***DEBUG begins.
cbug 9210 format ('quout DEBUG.  qlinear, qcross, qsquare=',1p3e12.4)
cbug      write (iout, 9210) qlinear, qcross, qsquare
cbugc***DEBUG ends.

      if ((qcross  .eq. 0.0)  .and.
     &    (qsquare .eq. 0.0)) then

c....   The quadric surface is a plane.

        if ((qy(n) .eq. 0.0)  .and.
     &      (qz(n) .eq. 0.0)  .and.
     &      (qx(n) .ne. 0.0)) then

          aqform(n) = 'planex'
          qc(n) = qc(n) / abs (qx(n))
          qx(n) = qx(n) / abs (qx(n))
          go to 130

        elseif ((qz(n) .eq. 0.0)  .and.
     &          (qx(n) .eq. 0.0)  .and.
     &          (qy(n) .ne. 0.0)) then

          aqform(n) = 'planey'
          qc(n) = qc(n) / abs (qy(n))
          qy(n) = qy(n) / abs (qy(n))
          go to 130

        elseif ((qx(n) .eq. 0.0)  .and.
     &          (qy(n) .eq. 0.0)  .and.
     &          (qz(n) .ne. 0.0)) then

          aqform(n) = 'planez'
          qc(n) = qc(n) / abs (qz(n))
          qz(n) = qz(n) / abs (qz(n))
          go to 130

        else                          ! Skew plane.

          aqform(n) = 'planegen'
          ax = qx(n)
          ay = qy(n)
          az = qz(n)

          call aptvunz (ax, ay, az, tol, vlength)

          if (vlength .ne. 0.0) then
            qc(n) = qc(n) / vlength
            qx(n) = ax
            qy(n) = ay
            qz(n) = az
          endif

          go to 130

        endif

      elseif (qcross .eq. 0.0) then

c....   The quadric surface is curved, and aligned with the major axes.

        aqform(n) = 'qaligned'
        go to 130
      else

c....   The quadric surface is curved, and not aligned with the major axes.

        aqform(n) = 'qnotalig'
      endif
cbugc***DEBUG begins.
cbug 9214 format ('quout DEBUG.  aqform=',a16)
cbug      write (iout, 9214) aqform(n)
cbugc***DEBUG ends.

c=======================================================================********

c.... Do the principle axis transformation on the quadric surface.

  130 nerrq = 0

      ac  = qc(n)
      ax  = qx(n)
      ay  = qy(n)
      az  = qz(n)
      axy = qxy(n)
      ayz = qyz(n)
      azx = qzx(n)
      axx = qxx(n)
      ayy = qyy(n)
      azz = qzz(n)

      call aptaxis (ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &              tol, xcen(n), ycen(n), zcen(n), rotq, ntype,
     &              nerraxis)

      dcen(n) = sqrt (xcen(n)**2 + ycen(n)**2 + zcen(n)**2)

c*************Consider ntype 18 -> 3, 21 -> 11, 22-> 12.

 9211 format ('  quout ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric surface is bad.',
     &  '  ***DELETE OR RESPECIFY***')
 8212 format ('  quout ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric surface is a point, a line, or imaginary.',
     &  '  ***DELETE OR RESPECIFY***')

c.... See if quadric surface is already in standard form, and if sign changed.

      kstd = 0
      if ((rotq(1,1) .eq. 1.0)  .and.
     &    (rotq(2,2) .eq. 1.0)  .and.
     &    (rotq(3,3) .eq. 1.0)  .and.
     &    (xcen(n)   .eq. 0.0)  .and.
     &    (ycen(n)   .eq. 0.0)  .and.
     &    (zcen(n)   .eq. 0.0)) then
                                      ! No rotation or translation.
        if (qc(n) .eq. ac) then       ! No change of sign.
          kstd   = 1
        endif                         ! Tested for change of sign.
     
      endif

      if (ntype .eq. -1) then
        write (   6, 9211) ainput, ninput
        write (iout, 9211) ainput, ninput
        nerrq = 2
        go to 140
      endif

      if (ntype .gt. 16) then         ! Imaginary quadric surface.
        write (   6, 8212) ainput, ninput
        write (iout, 8212) ainput, ninput
        nerrq = 2
      endif

      if (ntype .ge. 0) then
        aqtype(n) = aqtypes(ntype)
      else
        aqtype(n) = 'BAD'
        ntype = 25
      endif
cbugc***DEBUG begins.
cbug 9216 format ('quout DEBUG.  aqtype=',a,'  ntype=',i3)
cbug      write (iout, 9216) aqtype(n), ntype
cbugc***DEBUG ends.

c=======================================================================********

c.... See if the quadric surface is axisymmetric.

      asym(n) = ' '
      nn = 1 + ntype

      go to (101, 101, 101, 140, 140, 140, 140, 107, 140, 140, 101,
     &       140, 112, 102, 102, 103, 116, 140, 140, 140, 140, 140,
     &       140, 140, 140, 140), nn

c.... Simple, coincident and parallel planes.

  101 asym(n) = 'axisymmetric'
      go to 140

c.... Hyperboloids of one or two sheets.

  102 errxy = tol * (abs (axx) + abs (ayy))
      if (abs (axx - ayy) .le. errxy) then
        asym(n) = 'axisymmetric'
      endif
      go to 140

c.... Ellipsoids.

  103 errxy = tol * (abs (axx) + abs (ayy))
      erryz = tol * (abs (ayy) + abs (azz))
      if ((abs (axx - ayy) .le. errxy)  .or.
     &    (abs (ayy - azz) .le. erryz)) then
        asym(n) = 'axisymmetric'
      endif
      go to 140

c.... Circular cylinder.

  107 asym(n) = 'axisymmetric'
      go to 140

c.... Circular cone.

  112 asym(n) = 'axisymmetric'
      go to 140

c.... Sphere.

  116 asym(n) = 'axisymmetric'

  140 continue

c.... Save the last name of various quadric surface types.

      if (aquad(n) .eq. aquads) then

        if (aqtype(n) .eq. aqtypes(0))  then
          aplanes  = aquads
          laplanes = laquads
        endif

        if (aqtype(n) .eq. aqtypes(16)) then
          aspheres  = aquads
          laspheres = laquads
        endif

        if (aqtype(n) .eq. aqtypes(7))  then
          acyls  = aquads
          lacyls = laquads
        endif

        if (aqtype(n) .eq. aqtypes(8)) then
          ahyperbs  = aquads
          lahyperbs = laquads
        endif

        if (aqtype(n) .eq. aqtypes(12)) then
          acones  = aquads
          lacones = laquads
        endif

        if (aqtype(n) .eq. aqtypes(13)) then
          ahyperbs  = aquads
          lahyperbs = laquads
        endif

        if (aqtype(n) .eq. aqtypes(15)) then
          aellips  = aquads
          laellips = laquads
        endif

        if (asym(n) .eq. 'axisymmetric') then
          axisyms  = aquads
          laxisyms = laquads
        endif

      endif

c=======================================================================********

c.... Print the data for the original quadric surface.

c=======================================================================********

 8120 format ('plane   ',a / 8x,a62)
 8121 format ('plane   ',a / a16,2x,a62)
 8122 format ('quadric ',a / 8x,a62)
 8123 format ('quadric ',a / a16,2x,a62)
 8125 format ('  General type:     ',a16,4x,a)
 8130 format ('  Center     (xyz)',1p3e20.12)
 8132 format ('  Center to origin',1pe20.12)
 8135 format ('  Center    (',a4,')',1p3e20.12)

c.... Plane, simple (type 0):                                 x = 0  (type  0).

      if (ntype .eq. 0) then
        if (laquad(n) .gt. 16) then
          write (   6, 8120) aquad(n), aqtype(n)
          write (iout, 8120) aquad(n), aqtype(n)
        else
          write (   6, 8121) aquad(n), aquad(n), aqtype(n)
          write (iout, 8121) aquad(n), aquad(n), aqtype(n)
        endif
      else
        if (laquad(n) .gt. 16) then
          write (   6, 8122) aquad(n), aqtype(n)
          write (iout, 8122) aquad(n), aqtype(n)
        else
          write (   6, 8123) aquad(n), aquad(n), aqtype(n)
          write (iout, 8123) aquad(n), aquad(n), aqtype(n)
        endif
        write (   6, 8125) aqform(n), asym(n)
        write (iout, 8125) aqform(n), asym(n)
      endif

c.... Quadric types 0 to 3 and 8 to 12 (not cylinders).

      if ((ntype .lt. 4) .or. (ntype .gt. 7)) then
        write (   6, 8130) xcen(n), ycen(n), zcen(n)
        write (iout, 8130) xcen(n), ycen(n), zcen(n)
        call plotpt (xcen(n), ycen(n), zcen(n), nerrg)
      endif

c.... All quadrics.

      write (   6, 8132) dcen(n)
      write (iout, 8132) dcen(n)

c=======================================================================********

 8140 format ('  Center x=0 (xyz)',1p3e20.12)
 8141 format ('  Center y=0 (xyz)',1p3e20.12)
 8142 format ('  Center z=0 (xyz)',1p3e20.12)
 8196 format ('  Cyl axis   (xyz)',1p3e20.12)

      if ((ntype .ge. 4) .and. (ntype .le. 7)) then  ! All kinds of cylinders.

        if (rotq(3,1) .ne. 0.0) then
          cenx = 0.0
          ceny = ycen(n) - (xcen(n) / rotq(3,1)) * rotq(3,2)
          cenz = zcen(n) - (xcen(n) / rotq(3,1)) * rotq(3,3)
          write (   6, 8140) cenx, ceny, cenz
          write (iout, 8140) cenx, ceny, cenz
          call plotpt (cenx, ceny, cenz, nerrg)
        endif

        if (rotq(3,2) .ne. 0.0) then
          cenx = xcen(n) - (ycen(n) / rotq(3,2)) * rotq(3,1)
          ceny = 0.0
          cenz = zcen(n) - (ycen(n) / rotq(3,2)) * rotq(3,3)
          write (   6, 8141) cenx, ceny, cenz
          write (iout, 8141) cenx, ceny, cenz
          call plotpt (cenx, ceny, cenz, nerrg)
        endif

        if (rotq(3,3) .ne. 0.0) then
          cenx = xcen(n) - (zcen(n) / rotq(3,3)) * rotq(3,1)
          ceny = ycen(n) - (zcen(n) / rotq(3,3)) * rotq(3,2)
          cenz = 0.0
          write (   6, 8142) cenx, ceny, cenz
          write (iout, 8142) cenx, ceny, cenz
          call plotpt (cenx, ceny, cenz, nerrg)
        endif

        write (   6, 8196) (rotq(3,j), j = 1, 3)
        write (iout, 8196) (rotq(3,j), j = 1, 3)

      endif

c=======================================================================********

 8213 format ('  Center to focus ',1pe20.12 )
 8770 format ('  Center to foci  ',1p3e20.12 )
 8193 format ('  Latus rectum    ',1pe20.12 )

c.... Cylinder, parabolic:  (u = y or z)     (+/-)|qu|*u + x**2 = 0  (type  4).

      if (ntype .eq. 4) then          ! Parabolic cylinder.
        if (ay .eq. 0.0) then
          dfocus   = 0.25 * abs (az / (axx + small))
          dlatrect = abs (az / (axx + small))
        else
          dfocus   = 0.25 * abs (ay / (axx + small))
          dlatrect = abs (ay / (axx + small))
        endif
        write (   6, 8213) dfocus
        write (iout, 8213) dfocus
        write (   6, 8193) dlatrect
        write (iout, 8193) dlatrect
      endif

c=======================================================================********

 8211 format ('  Eccentricity    ',1pe20.12 )
 8192 format ('  Trans semiaxis  ',1pe20.12 )
 8760 format ('  Conj  semiaxis  ',1pe20.12 )

c.... Cylinder, hyperbolic:      (+/-)1 + qxx*x**2 - |qyy|*y**2 = 0  (type  5).

      if (ntype .eq. 5) then          ! Hyperbolic cylinder.
        if (ac .gt. 0) then
          dtran    =  sqrt (-ac / (ayy + small))
          dconj    =  sqrt ( ac / (axx + small))
          dlatrect =  2.0 * sqrt (-ac * ayy) / (axx + small)
          eccen    = sqrt (1.0 - ayy / (axx + small))
          dfocus   = sqrt (ac / (axx + small) - ac / (ayy + small))
        else
          dtran    =  sqrt (-ac / (axx + small))
          dconj    =  sqrt ( ac / (ayy + small))
          dlatrect = -2.0 * sqrt (-ac * axx) / (ayy + small)
          eccen    = sqrt (1.0 - axx / (ayy + small))
          dfocus   = sqrt (ac / (ayy + small) - ac / (axx + small))
        endif
        write (   6, 8211) eccen
        write (iout, 8211) eccen
        write (   6, 8192) dtran
        write (iout, 8192) dtran
        write (   6, 8760) dconj
        write (iout, 8760) dconj
        write (   6, 8213) dfocus
        write (iout, 8213) dfocus
        write (   6, 8193) dlatrect
        write (iout, 8193) dlatrect
      endif

c=======================================================================********

c.... Cylinder, elliptic:              -1 + qxx*x**2 + qyy*y**2 = 0  (type  6).

 8191 format ('  Ellip semiaxes  ',1p3e20.12 )

      if (ntype .eq. 6) then          ! Elliptic cylinder.
        eccen    = sqrt (1.0 - ayy / (axx + small))
        semix    = sqrt (-ac / (axx + small))
        semiy    = sqrt (-ac / (ayy + small))
        dlatrect = 2.0 * sqrt (-ac * ayy) / (axx + small)
        write (   6, 8211) eccen
        write (iout, 8211) eccen
        write (   6, 8191) semix, semiy
        write (iout, 8191) semix, semiy
        write (   6, 8193) dlatrect
        write (iout, 8193) dlatrect
      endif

c=======================================================================********

c.... Cylinder, circular (type 7):     -1 + qxx * (x**2 + y**2) = 0  (type  7).

 8194 format ('  Cylinder radius ',1pe20.12 )

      if (ntype .eq. 7) then          ! Circular cylinder.
        radius  = sqrt (-ac / (axx + small))
        write (   6, 8194) radius
        write (iout, 8194) radius
      endif

c=======================================================================********

c.... Paraboloid, hyperbolic:   (+/-)|qz|*z + x**2 - |qyy|*y**2 = 0  (type  8).

      if (ntype .eq. 8) then          ! Hyperbolic paraboloid.
        dlatrec1 = abs (az / (ayy + small))
        dlatrec2 = abs (az / (axx + small))
        write (   6, 8710) dlatrec1, dlatrec2
        write (iout, 8710) dlatrec1, dlatrec2
      endif

c=======================================================================********

c.... Paraboloid, elliptic:       (+/-)|qz|*z + x**2 + qyy*y**2 = 0  (type  9).

 8710 format ('  Latera recta    ',1p3e20.12 )

      if (ntype .eq. 9) then          ! Elliptic paraboloid.
        dlatrec1 = abs (az / (ayy + small))
        dlatrec2 = abs (az / (axx + small))
        write (   6, 8710) dlatrec1, dlatrec2
        write (iout, 8710) dlatrec1, dlatrec2
      endif

c=======================================================================********

c.... Paraboloid, circular:           (+/-)|qz|*z + x**2 + y**2 = 0  (type 10).

      if (ntype .eq. 10) then         ! Circular paraboloid.
        dfocus  = 0.25 * abs (az / (axx + small))
        dlatrect = abs (az / (axx + small))
        write (   6, 8213) dfocus
        write (iout, 8213) dfocus
        write (   6, 8193) dlatrect
        write (iout, 8193) dlatrect
      endif

c=======================================================================********

c.... Cone, circular (type 12):        x**2 + y**2 - |qzz|*z**2 = 0  (type 12).

 8197 format ('  Cone axis  (xyz)',1p3e20.12)

      if (ntype .eq. 12) then         ! Circular cone.
        write (   6, 8197) (rotq(3,j), j = 1, 3)
        write (iout, 8197) (rotq(3,j), j = 1, 3)
      endif

c=======================================================================********

c.... Hyperboloid of 1 sheet:   -1+qxx*x**2+qyy*y**2-|qzz|*z**2 = 0  (type 13).

 8720 format ('  Trans semiaxes  ',1p2e20.12 )

      if (ntype .eq. 13) then         ! Hyperboloid of one sheet.
        dtran1   =  sqrt (-ac / (ayy + small))
        dtran2   =  sqrt (-ac / (axx + small))
        dfocus1  = sqrt (ac / (azz + small) - ac / (ayy + small))
        dfocus2  = sqrt (ac / (azz + small) - ac / (axx + small))
        dconj    =  sqrt (ac / (azz + small))
        dlatrec1 = -2.0 * sqrt (-ac * ayy) / (azz + small)
        dlatrec2 = -2.0 * sqrt (-ac * axx) / (azz + small)
        write (   6, 8720) dtran1, dtran2
        write (iout, 8720) dtran1, dtran2
        write (   6, 8760) dconj
        write (iout, 8760) dconj
        write (   6, 8770) dfocus1, dfocus2
        write (iout, 8770) dfocus1, dfocus2
        write (   6, 8710) dlatrec1, dlatrec2
        write (iout, 8710) dlatrec1, dlatrec2
      endif

c=======================================================================********

c.... Hyperboloid of 2 sheets:   1+qxx*x**2+qyy*y**2-|qzz|*z**2 = 0  (type 14).

 8740 format ('  Conj  semiaxes  ',1p2e20.12 )
 8750 format ('  Eccentricities  ',1p2e20.12 )

      if (ntype .eq. 14) then         ! Hyperboloid of two sheets.
        eccen1   = sqrt (1.0 - azz / (ayy + small))
        eccen2   = sqrt (1.0 - azz / (axx + small))
        dtran    = sqrt (-ac / (azz + small))
        dfocus1  = sqrt (ac / (ayy + small) - ac / (azz + small))
        dfocus2  = sqrt (ac / (axx + small) - ac / (azz + small))
        dconj1   = sqrt (ac / (ayy + small))
        dconj2   = sqrt (ac / (axx + small))
        dlatrec1 = 2.0 * sqrt (-ac * azz) / (ayy + small)
        dlatrec2 = 2.0 * sqrt (-ac * azz) / (axx + small)
        write (   6, 8750) eccen1, eccen2
        write (iout, 8750) eccen1, eccen2
        write (   6, 8192) dtran
        write (iout, 8192) dtran
        write (   6, 8740) dconj1, dconj2
        write (iout, 8740) dconj1, dconj2
        write (   6, 8770) dfocus1, dfocus2
        write (iout, 8770) dfocus1, dfocus2
        write (   6, 8710) dlatrec1, dlatrec2
        write (iout, 8710) dlatrec1, dlatrec2
      endif

c=======================================================================********

c.... Ellipsoid:            -1 + qxx*x**2 + qyy*y**2 + qzz*z**2 = 0  (type 15).

 8755 format ('  Eccentricities  ',1p3e20.12 )

      if (ntype .eq. 15) then         ! Ellipsoid.

        erryz = tol * (abs (ayy) + abs (azz))
        if (abs (ayy - azz) .le. erryz) then
          eccenx  = 0.0
          dfocusx = 0.0
        else
          eccenx  = sqrt (1.0 - azz / (ayy + small))
          dfocusx = sqrt (ac / (ayy + small) - ac / (azz + small))
        endif

        errxz = tol * (abs (axx) + abs (azz))
        if (abs (axx - azz) .le. errxz) then
          ecceny  = 0.0
          dfocusy = 0.0
        else
          ecceny  = sqrt (1.0 - azz / (axx + small))
          dfocusy = sqrt (ac / (axx + small) - ac / (azz + small))
        endif

        errxy = tol * (abs (axx) + abs (ayy))
        if (abs (axx - ayy) .le. errxy) then
          eccenz  = 0.0
          dfocusz = 0.0
        else
          eccenz  = sqrt (1.0 - ayy / (axx + small))
          dfocusz = sqrt (ac / (axx + small) - ac / (ayy + small))
        endif

        semix    = sqrt (-ac / (axx + small))
        semiy    = sqrt (-ac / (ayy + small))
        semiz    = sqrt (-ac / (azz + small))
        dlatrecx = 2.0 * sqrt (-ac * azz) / (ayy + small)
        dlatrecy = 2.0 * sqrt (-ac * azz) / (axx + small)
        dlatrecz = 2.0 * sqrt (-ac * ayy) / (axx + small)

        write (   6, 8755) eccenx, ecceny, eccenz
        write (iout, 8755) eccenx, ecceny, eccenz
        write (   6, 8191) semix, semiy, semiz
        write (iout, 8191) semix, semiy, semiz
        write (   6, 8770) dfocusx, dfocusy, dfocusz
        write (iout, 8770) dfocusx, dfocusy, dfocusz
        write (   6, 8710) dlatrecx, dlatrecy, dlatrecz
        write (iout, 8710) dlatrecx, dlatrecy, dlatrecz

      endif                           ! Ellipsoid. 

c=======================================================================********

c.... Sphere (type 16):         -1 + qxx * (x**2 + y**2 + z**2) = 0  (type 16).

 8195 format ('  Sphere radius   ',1pe20.12 )

      if (ntype .eq. 16) then         ! Sphere.
        radius  = sqrt (-ac / (axx + small))
        write (   6, 8195) radius
        write (iout, 8195) radius
      endif                           ! Sphere.

c=======================================================================********

c.... Find half-angle at vertex of cones and hyperbolic surfaces.

      fact = 1.0
      if (angunit .eq. 'degrees') fact = 180.0 / pi

 8290 format ('  Half-angle      ',1pe20.12,2x,a)
 8291 format ('  Half-angle      ',1pe20.12,2x,a)
 8292 format ('  Half-angle (xy) ',1pe20.12,2x,a)
 8293 format ('  Half-angle(x''y'')',1pe20.12,2x,a)
 8294 format ('  Half-angle (yx) ',1pe20.12,2x,a)
 8295 format ('  Half-angle(y''x'')',1pe20.12,2x,a)
 8296 format ('  Half-angle (zx) ',1pe20.12,2x,a)
 8297 format ('  Half-angle(z''x'')',1pe20.12,2x,a)
 8298 format ('  Half-angle (zy) ',1pe20.12,2x,a)
 8299 format ('  Half-angle(z''y'')',1pe20.12,2x,a)
cbugc***DEBUG begins.
cbug 9212 format ('quout DEBUG.  axx, ayy, azz =',1p3e12.4)
cbug      write (iout, 9212) axx, ayy, azz
cbugc***DEBUG ends.

      if (azz .eq. 0.0) then          ! Cylinders, hyperbolic paraboloids.

        if (ayy .lt. 0.0) then        ! Intersecting planes, hyperbolics.

          thhalf = fact * atan (sqrt (-axx / (ayy + small)))
          if (kstd .eq. 1) then
            write (   6, 8292) thhalf, angunit
            write (iout, 8292) thhalf, angunit
          else
            write (   6, 8293) thhalf, angunit
            write (iout, 8293) thhalf, angunit
          endif

          thhalf = fact * atan (sqrt (-ayy / (axx + small)))
          if (kstd .eq. 1) then
            write (   6, 8294) thhalf, angunit
            write (iout, 8294) thhalf, angunit
          else
            write (   6, 8295) thhalf, angunit
            write (iout, 8295) thhalf, angunit
          endif

        endif

      elseif (azz .lt. 0.0) then      ! Cones and hyperboloids of 1 or 2 sheets.

        errxy = tol * (abs (axx) + abs (ayy))

        if (abs (axx - ayy) .le. errxy) then  ! Circular cones and hyperboloids.

          thhalf = fact * atan (sqrt (-azz / (axx + small)))
          if (kstd .eq. 1) then
            write (   6, 8290) thhalf, angunit
            write (iout, 8290) thhalf, angunit
          else
            write (   6, 8291) thhalf, angunit
            write (iout, 8291) thhalf, angunit
          endif

        else                          ! Elliptic cones and hyperboloids.

          thhalf = fact * atan (sqrt (-azz / (axx + small)))
          if (kstd .eq. 1) then
            write (   6, 8296) thhalf, angunit
            write (iout, 8296) thhalf, angunit
          else
            write (   6, 8297) thhalf, angunit
            write (iout, 8297) thhalf, angunit
          endif

          thhalf = fact * atan (sqrt (-azz / (ayy + small)))
          if (kstd .eq. 1) then
            write (   6, 8298) thhalf, angunit
            write (iout, 8298) thhalf, angunit
          else
            write (   6, 8299) thhalf, angunit
            write (iout, 8299) thhalf, angunit
          endif

        endif

      endif                           ! Tested for azz magnitude and sign.

c=======================================================================********

 8160 format ('  Original    qc  ',1pe20.12 )

      if (aqtype(n) .ne. aqtypes(0)) then
        write (   6, 8160) qc(n)
        write (iout, 8160) qc(n)
      endif

 8180 format ('    qx,  qy,  qz  ',1p3e20.12 )
 8182 format ('  qc = -N dot P   ',1pe20.12 /
     &        '    qx,  qy,  qz  ',1p3e20.12 /
     &        '  Norm vect  (xyz)',1p3e20.12 )
cold 8184 format ('  Norm vect (',a4,')',1p3e20.12 )

      if (qlinear .gt. 0.0) then
        if (aqtype(n) .eq. aqtypes(0)) then
          write (   6, 8182) qc(n), qx(n), qy(n), qz(n),
     &                              qx(n), qy(n), qz(n)
          write (iout, 8182) qc(n), qx(n), qy(n), qz(n),
     &                              qx(n), qy(n), qz(n)
cold          if (asys .ne. 'rect') then
cold            write (   6, 8184) asys, du, dv, dw
cold            write (iout, 8184) asys, du, dv, dw
cold          endif
        else
          write (   6, 8180) qx(n), qy(n), qz(n)
          write (iout, 8180) qx(n), qy(n), qz(n)
        endif
      endif

 8185 format ('    qxy, qyz, qzx ',1p3e20.12 )

      if (qcross .gt. 0.0) then
        write (   6, 8185) qxy(n), qyz(n), qzx(n)
        write (iout, 8185) qxy(n), qyz(n), qzx(n)
      endif

 8190 format ('    qxx, qyy, qzz ',1p3e20.12 )

      if (qsquare .gt. 0.0) then
        write (   6, 8190) qxx(n), qyy(n), qzz(n)
        write (iout, 8190) qxx(n), qyy(n), qzz(n)
      endif

 8210 format ('  quout ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric has bad coefficients.  ***DELETE OR RESPECIFY***')

      if (nerraxis .ne. 0) then
        write (   6, 8210) ainput, ninput
        write (iout, 8210) ainput, ninput
        nerrq = 1
      endif

c=======================================================================********

c.... See if the maximum amount of data should be displayed.

      if ((amount .eq. 'maximum') .or. (amount .eq. 'max')) then

c=======================================================================********

c.... Find extreme points on quadric surface.

      call aptqext (qc(n), qx(n), qy(n), qz(n), qxy(n), qyz(n),
     &              qzx(n), qxx(n), qyy(n), qzz(n),
     &              tol, ntyps, cenx, ceny, cenz, np, ex, ey, ez,
     &              adir, nerra)

 8220 format ('  ',a,'   (xyz)',1p3e20.12 )

      do 220 nex = 1, np
        write (   6, 8220) adir(nex), ex(nex), ey(nex), ez(nex)
        write (iout, 8220) adir(nex), ex(nex), ey(nex), ez(nex)
        call plotpt (ex(nex), ey(nex), ez(nex), nerrg)
  220 continue

c=======================================================================********

c.... Find intercepts of original form on x axis.

      call aptqrts (0, qxx(n), qx(n), qc(n), qq, tol,
     &              nroots, xint1, xint2, itrun)

      zero = 0.0

 8262 format ('  x  intercept    ',1p3e20.12)

      if (nroots .ge. 1) then
        write (   6, 8262) xint1, zero, zero
        write (iout, 8262) xint1, zero, zero
        call plotpt (xint1, zero, zero, nerrg)
      endif
      if (nroots .eq. 2) then
        write (   6, 8262) xint2, zero, zero
        write (iout, 8262) xint2, zero, zero
        call plotpt (xint2, zero, zero, nerrg)
      endif

c.... Find intercepts of original form on y axis.

      call aptqrts (0, qyy(n), qy(n), qc(n), qq, tol,
     &              nroots, yint1, yint2, itrun)

 8272 format ('  y  intercept    ',1p3e20.12)

      if (nroots .ge. 1) then
        write (   6, 8272) zero, yint1, zero
        write (iout, 8272) zero, yint1, zero
        call plotpt (zero, yint1, zero, nerrg)
      endif
      if (nroots .eq. 2) then
        write (   6, 8272) zero, yint2, zero
        write (iout, 8272) zero, yint2, zero
        call plotpt (zero, yint2, zero, nerrg)
      endif

c....   Find intercepts of original form on z axis.

      call aptqrts (0, qzz(n), qz(n), qc(n), qq, tol,
     &              nroots, zint1, zint2, itrun)

 8282 format ('  z  intercept    ',1p3e20.12)

      if (nroots .ge. 1) then
        write (   6, 8282) zero, zero, zint1
        write (iout, 8282) zero, zero, zint1
        call plotpt (zero, zero, zint1, nerrg)
      endif
      if (nroots .eq. 2) then
        write (   6, 8282) zero, zero, zint2
        write (iout, 8282) zero, zero, zint2
        call plotpt (zero, zero, zint2, nerrg)
      endif

      if (nerrq .ne. 0) go to 120

c=======================================================================********

c.... Scale the standard equation.
cbugc***DEBUG begins.
cbug 9235 format ('quout DEBUG kstd=',i3)
cbug 9230 format ('quout DEBUG ac, ax, axx=',1p3e12.4)
cbug      write (iout, 9235) kstd
cbug      write (iout, 9230) ac, ax, axx
cbugc***DEBUG ends.

      fact = 1.0
      if (ac .ne. 0.0) then
        fact = 1.0 / abs (ac)
      elseif (axx .ne. 0.0) then
        fact = 1.0 / abs (axx)
      elseif (ax .ne. 0.0) then
        fact = 1.0 / abs (ax)
      endif
cbugc***DEBUG begins.
cbug      write (iout, 9230) ac, ax, axx
cbugc***DEBUG ends.

      ac  = fact * ac
      ax  = fact * ax
      ay  = fact * ay
      az  = fact * az
      axy = fact * axy
      ayz = fact * ayz
      azx = fact * azx
      axx = fact * axx
      ayy = fact * ayy
      azz = fact * azz

c=======================================================================********

c.... Skip stuff for standard form if quadric is already standard.

      if (kstd .eq. 1) go to 260

c=======================================================================********

c.... Print the principle axis rotation operator.

 8240 format ('  Operator to rotate ',a,' to the x, y and z axes' /
     &  '  (rows are initial axes):')
 8250 format ('    x'' axis       ',1p3e20.12 /
     &        '    y'' axis       ',1p3e20.12 /
     &        '    z'' axis (main)',1p3e20.12)

      write (   6, 8240) aquad(n)
      write (iout, 8240) aquad(n)
      write (   6, 8250) ((rotq(i,j), j = 1, 3), i = 1, 3)
      write (iout, 8250) ((rotq(i,j), j = 1, 3), i = 1, 3)

c=======================================================================********

c.... Print the transformed coefficients.

 8170 format ('  Standard    qc  ',1pe20.12 )

      alinear = ax**2  + ay**2  + az**2
      across  = axy**2 + ayz**2 + azx**2
      asquare = axx**2 + ayy**2 + azz**2

      if (aqtype(n) .ne. aqtypes(0)) then
        write (   6, 8170) ac
        write (iout, 8170) ac
      endif

      if (alinear .gt. 0.0) then
        if (aqtype(n) .eq. aqtypes(0)) then
          write (   6, 8182) ac, ax, ay, az, ax, ay, az
          write (iout, 8182) ac, ax, ay, az, ax, ay, az
        else
          write (   6, 8180) ax, ay, az
          write (iout, 8180) ax, ay, az
        endif
      endif

      if (across .gt. 0.0) then
        write (   6, 8185) axy, ayz, azx
        write (iout, 8185) axy, ayz, azx
      endif

      if (asquare .gt. 0.0) then
        write (   6, 8190) axx, ayy, azz
        write (iout, 8190) axx, ayy, azz
      endif

      if (ntype .gt. 16) then         ! Imaginary quadric surface.
        write (   6, 8212) ainput, ninput
        write (iout, 8212) ainput, ninput
      endif

c=======================================================================********

c.... Find intercepts of standard form on x' axis, original coordinates.

      dx = -xcen(n)
      dy = -ycen(n)
      dz = -zcen(n)

      call aptqrts (0, axx, 0., ac, qq, tol,
     &              nroots, xint1, xint2, itrun)

 8260 format ('  x'' intercept    ',1p3e20.12)
 8261 format ('  ", original xyz ',1p3e20.12)

      if (nroots .ge. 1) then
        yr = 0.0
        zr = 0.0
        write (   6, 8260) xint1, yr, zr
        write (iout, 8260) xint1, yr, zr
        semix = abs (xint1)
        call aptmopv (rotq, 1, 0., 0., 0., xint1, yr, zr, 1, tol,
     &                nerra)
        call apttran (dx, dy, dz, xint1, yr, zr, 1, tol, nerra)
        write (   6, 8261) xint1, yr, zr
        write (iout, 8261) xint1, yr, zr
        call plotpt (xint1, yr, zr, nerrg)
      endif

      if (nroots .eq. 2) then
        yr = 0.0
        zr = 0.0
        write (   6, 8260) xint2, yr, zr
        write (iout, 8260) xint2, yr, zr
        call aptmopv (rotq, 1, 0., 0., 0., xint2, yr, zr, 1, tol,
     &                nerr)
        call apttran (dx, dy, dz, xint2, yr, zr, 1, tol, nerra)
        write (   6, 8261) xint2, yr, zr
        write (iout, 8261) xint2, yr, zr
        call plotpt (xint2, yr, zr, nerrg)
      endif

c=======================================================================********

c.... Find intercepts of standard form on y' axis, original coordinates.

      call aptqrts (0, ayy, 0., ac, qq, tol,
     &              nroots, yint1, yint2, itrun)

 8270 format ('  y'' intercept    ',1p3e20.12)
 8271 format ('  ", original xyz ',1p3e20.12)

      if (nroots .ge. 1) then
        xr = 0.0
        zr = 0.0
        semiy = abs (yint1)
        write (   6, 8270) xr, yint1, zr
        write (iout, 8270) xr, yint1, zr
        call aptmopv (rotq, 1, 0., 0., 0., xr, yint1, zr, 1, tol,
     &                nerra)
        call apttran (dx, dy, dz, xr, yint1, zr, 1, tol, nerra)
        write (   6, 8271) xr, yint1, zr
        write (iout, 8271) xr, yint1, zr
        call plotpt (xr, yint1, zr, nerrg)
      endif
      if (nroots .eq. 2) then
        xr = 0.0
        zr = 0.0
        write (   6, 8270) xr, yint2, zr
        write (iout, 8270) xr, yint2, zr
        call aptmopv (rotq, 1, 0., 0., 0., xr, yint2, zr, 1, tol,
     &                nerra)
        call apttran (dx, dy, dz, xr, yint2, zr, 1, tol, nerra)
        write (   6, 8271) xr, yint2, zr
        write (iout, 8271) xr, yint2, zr
        call plotpt (xr, yint2, zr, nerrg)
      endif

c=======================================================================********

c.... Find intercepts of standard form on z' axis, original coordinates.

      call aptqrts (0, azz, az, ac, qq, tol,
     &              nroots, zint1, zint2, itrun)

 8280 format ('  z'' intercept    ',1p3e20.12)
 8281 format ('  ", original xyz ',1p3e20.12)

      if (nroots .ge. 1) then
        xr = 0.0
        yr = 0.0
        semiz = abs (zint1)
        write (   6, 8280) xr, yr, zint1
        write (iout, 8280) xr, yr, zint1
        call aptmopv (rotq, 1, 0., 0., 0., xr, yr, zint1, 1, tol,
     &                nerra)
        call apttran (dx, dy, dz, xr, yr, zint1, 1, tol, nerra)
        write (   6, 8281) xr, yr, zint1
        write (iout, 8281) xr, yr, zint1
        call plotpt (xr, yr, zint1, nerrg)
      endif
      if (nroots .eq. 2) then
        xr = 0.0
        yr = 0.0
        write (   6, 8280) xr, yr, zint2
        write (iout, 8280) xr, yr, zint2
        call aptmopv (rotq, 1, 0., 0., 0., xr, yr, zint2, 1, tol,
     &                nerra)
        call apttran (dx, dy, dz, xr, yr, zint2, 1, tol, nerra)
        write (   6, 8281) xr, yr, zint2
        write (iout, 8281) xr, yr, zint2
        call plotpt (xr, yr, zint2, nerrg)
      endif

c=======================================================================********

c.... Skipped here when original quadric surface is in standard form.

  260 continue

c=======================================================================********

c.... Display simple planes for quadrics representing two planes.

      if (ntype .eq. 1) then          ! Two coincident simple planes.

        sc  = 0.0
        sx  = 1.0
        sy  = 0.0
        sz  = 0.0
        sxy = 0.0
        syz = 0.0
        szx = 0.0
        sxx = 0.0
        syy = 0.0
        szz = 0.0

c....   Rotate the plane back to the original axes.

        call aptrois (rotq, 1, 0., 0., 0., sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Translate the plane back to the center point.

        call apttris (1, xcen(n), ycen(n), zcen(n), sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Find the center of the simple plane (nearest origin).

        vnor = sx**2 + sy**2 + sz**2
        cenx1 = -sc * sx / vnor
        ceny1 = -sc * sy / vnor
        cenz1 = -sc * sz / vnor

 8340 format ('Two coincident simple planes:' /
     &        '  Center     (xyz)',1p3e20.12 /
     &        '  qc = -N dot P   ',1pe20.12  /
     &        '    qx,  qy,  qz  ',1p3e20.12 /
     &        '  Norm vect  (xyz)',1p3e20.12 )

        write (   6, 8340) cenx1, ceny1, cenz1, sc, sx, sy, sz,
     &    sx, sy, sz
        write (iout, 8340) cenx1, ceny1, cenz1, sc, sx, sy, sz,
     &    sx, sy, sz

      endif                           ! Two coincident simple planes.

      if (ntype .eq. 2) then          ! Two parallel simple planes.

        fact = sqrt (-ac / (axx + small))

        sc  =  fact
        sx  =  1.0
        sy  =  0.0
        sz  =  0.0
        sxy =  0.0
        syz =  0.0
        szx =  0.0
        sxx =  0.0
        syy =  0.0
        szz =  0.0

c....   Rotate the first simple plane back to the original axes.

        call aptrois (rotq, 1, 0., 0., 0., sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Translate the first simple plane back to the center point.

        call apttris (1, xcen(n), ycen(n), zcen(n), sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Find the center of the first simple plane (nearest origin).

        vnor  = sx**2 + sy**2 + sz**2
        cenx1 = -sc * sx / vnor
        ceny1 = -sc * sy / vnor
        cenz1 = -sc * sz / vnor

 8350 format ('First  parallel simple plane:' /
     &        '  Center     (xyz)',1p3e20.12 /
     &        '  qc = -N dot P   ',1pe20.12  /
     &        '    qx,  qy,  qz  ',1p3e20.12 /
     &        '  Norm vect  (xyz)',1p3e20.12 )

        write (   6, 8350) cenx1, ceny1, cenz1, sc, sx, sy, sz,
     &    sx, sy, sz
        write (iout, 8350) cenx1, ceny1, cenz1, sc, sx, sy, sz,
     &    sx, sy, sz

c....   Find the coefficients of the second simple plane.

        sc  = -fact
        sx  =  1.0
        sy  =  0.0
        sz  =  0.0

c....   Rotate the second simple plane back to the original axes.

        call aptrois (rotq, 1, 0., 0., 0., sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Translate the second simple plane back to the center point.

        call apttris (1, xcen(n), ycen(n), zcen(n), sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Find the center of the second simple plane (nearest origin).

        vnor  = sx**2 + sy**2 + sz**2
        cenx2 = -sc * sx / vnor
        ceny2 = -sc * sy / vnor
        cenz2 = -sc * sz / vnor

 8360 format ('Second parallel simple plane:' /
     &        '  Center     (xyz)',1p3e20.12 /
     &        '  qc = -N dot P   ',1pe20.12  /
     &        '    qx,  qy,  qz  ',1p3e20.12 /
     &        '  Norm vect  (xyz)',1p3e20.12 )

        write (   6, 8360) cenx2, ceny2, cenz2, sc, sx, sy, sz,
     &    sx, sy, sz
        write (iout, 8360) cenx2, ceny2, cenz2, sc, sx, sy, sz,
     &    sx, sy, sz

      endif                           ! Two parallel simple planes.

      if (ntype .eq. 3) then          ! Two intersecting simple planes.

        fx  = sqrt ( axx / (axx - ayy))
        fy  = sqrt (-ayy / (axx - ayy))

        sc  =  0.0
        sx  =  fx
        sy  =  fy
        sz  =  0.0
        sxy =  0.0
        syz =  0.0
        szx =  0.0
        sxx =  0.0
        syy =  0.0
        szz =  0.0

c....   Rotate the first plane back to the original axes.

        call aptrois (rotq, 1, 0., 0., 0., sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Translate the first plane back to the center point.

        call apttris (1, xcen(n), ycen(n), zcen(n), sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Find the center of the first plane (nearest origin).

        vnor  = sx**2 + sy**2 + sz**2
        cenx1 = -sc * sx / vnor
        ceny1 = -sc * sy / vnor
        cenz1 = -sc * sz / vnor

 8370   format ('First  intersecting simple plane:' /
     &          '  Center     (xyz)',1p3e20.12 /
     &          '  qc = -N dot P   ',1pe20.12  /
     &          '    qx,  qy,  qz  ',1p3e20.12 /
     &          '  Norm vect  (xyz)',1p3e20.12 )

        write (   6, 8370) cenx1, ceny1, cenz1, sc, sx, sy, sz,
     &    sx, sy, sz
        write (iout, 8370) cenx1, ceny1, cenz1, sc, sx, sy, sz,
     &    sx, sy, sz

c....   Find the coefficients of the second plane.

        sc  =  0.0
        sx  =  fx
        sy  = -fy 
        sz  =  0.0

c....   Rotate the second plane back to the original axes.

        call aptrois (rotq, 1, 0., 0., 0., sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Translate the second plane back to the center point.

        call apttris (1, xcen(n), ycen(n), zcen(n), sc, sx, sy, sz,
     &                sxy, syz, szx, sxx, syy, szz, tol, nerrs)

c....   Find the center of the second plane (nearest origin).

        vnor  = sx**2 + sy**2 + sz**2
        cenx2 = -sc * sx / vnor
        ceny2 = -sc * sy / vnor
        cenz2 = -sc * sz / vnor

 8380   format ('Second intersecting simple plane:' /
     &          '  Center     (xyz)',1p3e20.12 /
     &          '  qc = -N dot P   ',1pe20.12  /
     &          '    qx,  qy,  qz  ',1p3e20.12 /
     &          '  Norm vect  (xyz)',1p3e20.12 )

        write (   6, 8380) cenx2, ceny2, cenz2, sc, sx, sy, sz,
     &    sx, sy, sz
        write (iout, 8380) cenx2, ceny2, cenz2, sc, sx, sy, sz,
     &    sx, sy, sz

      endif                           ! Two intersecting simple planes.

c=======================================================================********

c.... Find foci of x-z parabolic surfaces, in standard and original form.

 8310 format ('  z  focus (r = 0)',1p3e20.12)
 8311 format ('  z'' focus (r''= 0)',1p3e20.12)
 8312 format ('  ", original xyz ',1p3e20.12)
 8320 format ('  z  focus (y = 0)',1p3e20.12)
 8321 format ('  z'' focus (y''= 0)',1p3e20.12)
 8322 format ('  ", original xyz ',1p3e20.12)
 8330 format ('  z  focus (x = 0)',1p3e20.12)
 8331 format ('  z'' focus (x''= 0)',1p3e20.12)
 8332 format ('  ", original xyz ',1p3e20.12)

      if (az .ne. 0.0) then

        errxy = tol * (abs (axx) + abs (ayy))

        if (ayy .eq. 0.0) then
          xf = 0.0
          yf = 0.0
          zf = -0.25 * az / (axx + small)

          if (kstd .eq. 1) then
            write (   6, 8320) xf, yf, zf
            write (iout, 8320) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8321) xf, yf, zf
            write (iout, 8321) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8322) xf, yf, zf
            write (iout, 8322) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

        elseif (abs (axx - ayy) .le. errxy) then
          xf = 0.0
          yf = 0.0
          zf = -0.25 * az / (axx + small)

          if (kstd .eq. 1) then
            write (   6, 8310) xf, yf, zf
            write (iout, 8310) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8311) xf, yf, zf
            write (iout, 8311) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8312) xf, yf, zf
            write (iout, 8312) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

        else

          xf = 0.0
          yf = 0.0
          zf = -0.25 * az / (axx + small)

          if (kstd .eq. 1) then
            write (   6, 8320) xf, yf, zf
            write (iout, 8320) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8321) xf, yf, zf
            write (iout, 8321) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8322) xf, yf, zf
            write (iout, 8322) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = 0.0
          yf = 0.0
          zf = -0.25 * az / (ayy + small)

          if (kstd .eq. 1) then
            write (   6, 8330) xf, yf, zf
            write (iout, 8330) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8331) xf, yf, zf
            write (iout, 8331) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8332) xf, yf, zf
            write (iout, 8332) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

        endif
      endif

c=======================================================================********

c.... Find foci of x-y parabolic surfaces, in standard and original form.

 8610 format ('  y  focus (r = 0)',1p3e20.12)
 8611 format ('  y'' focus (r''= 0)',1p3e20.12)
 8612 format ('  ", original xyz ',1p3e20.12)
 8640 format ('  x  focus (z = 0)',1p3e20.12)
 8641 format ('  x'' focus (z''= 0)',1p3e20.12)
 8620 format ('  y  focus (z = 0)',1p3e20.12)
 8621 format ('  y'' focus (z''= 0)',1p3e20.12)
 8622 format ('  ", original xyz ',1p3e20.12)
 8630 format ('  y  focus (x = 0)',1p3e20.12)
 8631 format ('  y'' focus (x''= 0)',1p3e20.12)
 8632 format ('  ", original xyz ',1p3e20.12)

      if (ay .ne. 0.0) then

        errxz = tol * (abs (axx) + abs (azz))

        if (azz .eq. 0.0) then
          xf = 0.0
          yf = -0.25 * ay / (axx + small)
          zf = 0.0

          if (kstd .eq. 1) then
            write (   6, 8620) xf, yf, zf
            write (iout, 8620) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8621) xf, yf, zf
            write (iout, 8621) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8622) xf, yf, zf
            write (iout, 8622) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

        elseif (abs (axx - azz) .le. errxz) then
          xf = 0.0
          yf = -0.25 * ay / (axx + small)
          zf = 0.0

          if (kstd .eq. 1) then
            write (   6, 8610) xf, yf, zf
            write (iout, 8610) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8611) xf, yf, zf
            write (iout, 8611) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8612) xf, yf, zf
            write (iout, 8612) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

        else

          xf = 0.0
          yf = -0.25 * ay / (axx + small)
          zf = 0.0

          if (kstd .eq. 1) then
            write (   6, 8620) xf, yf, zf
            write (iout, 8620) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8621) xf, yf, zf
            write (iout, 8621) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8622) xf, yf, zf
            write (iout, 8622) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = 0.0
          yf = -0.25 * ay / (azz + small)
          zf = 0.0

          if (kstd .eq. 1) then
            write (   6, 8630) xf, yf, zf
            write (iout, 8630) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8631) xf, yf, zf
            write (iout, 8631) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8632) xf, yf, zf
            write (iout, 8632) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

        endif
      endif

c=======================================================================********

c.... Ellipsoid:            -1 + qxx*x**2 + qyy*y**2 + qzz*z**2 = 0  (type 15).

c.... Find foci of ellipsoids, in standard and original form.

 8410 format ('  x = 0 focus     ',1p3e20.12)
 8411 format ('  x''= 0 focus     ',1p3e20.12)
 8420 format ('  ", original xyz ',1p3e20.12)

      if (ntype .eq. 15) then         ! Ellipsoid.    

        erryz = tol * (abs (ayy) + abs (azz))

        if ((azz - ayy) .gt. erryz) then  ! x = 0:  focus is on y axis.
          xf = 0.0
          yf = sqrt (ac / (azz + small) - ac / (ayy + small))
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8410) xf, yf, zf
            write (iout, 8410) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8411) xf, yf, zf
            write (iout, 8411) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8420) xf, yf, zf
            write (iout, 8420) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = 0.0
          yf = -sqrt (ac / (azz + small) - ac / (ayy + small))
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8410) xf, yf, zf
            write (iout, 8410) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8411) xf, yf, zf
            write (iout, 8411) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8420) xf, yf, zf
            write (iout, 8420) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif
        elseif ((ayy - azz) .gt. erryz) then  ! x = 0:  focus on z axis.
          xf = 0.0
          yf = 0.0
          zf = sqrt (ac / (ayy + small) - ac / (azz + small))
          if (kstd .eq. 1) then
            write (   6, 8410) xf, yf, zf
            write (iout, 8410) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8411) xf, yf, zf
            write (iout, 8411) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8420) xf, yf, zf
            write (iout, 8420) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = 0.0
          yf = 0.0
          zf = -sqrt (ac / (ayy + small) - ac / (azz + small))
          if (kstd .eq. 1) then
            write (   6, 8410) xf, yf, zf
            write (iout, 8410) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8411) xf, yf, zf
            write (iout, 8411) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8420) xf, yf, zf
            write (iout, 8420) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif
        endif

 8430 format ('  y = 0 focus     ',1p3e20.12)
 8431 format ('  y''= 0 focus     ',1p3e20.12)
 8440 format ('  ", original xyz ',1p3e20.12)

        errzx = tol * (abs (azz) + abs (axx))

        if ((axx - azz) .gt. errzx) then  ! y = 0:  focus is on z axis.
          xf = 0.0
          yf = 0.0
          zf = sqrt (ac / (axx + small) - ac / (azz + small))
          if (kstd .eq. 1) then
            write (   6, 8430) xf, yf, zf
            write (iout, 8430) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8431) xf, yf, zf
            write (iout, 8431) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8440) xf, yf, zf
            write (iout, 8440) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = 0.0
          yf = 0.0
          zf = -sqrt (ac / (axx + small) - ac / (azz + small))
          if (kstd .eq. 1) then
            write (   6, 8430) xf, yf, zf
            write (iout, 8430) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8431) xf, yf, zf
            write (iout, 8431) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8440) xf, yf, zf
            write (iout, 8440) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif
        elseif ((azz - axx) .gt. errzx) then  ! y = 0:  focus on x axis.
          xf = sqrt (ac / (azz + small) - ac / (axx + small))
          yf = 0.0
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8430) xf, yf, zf
            write (iout, 8430) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8431) xf, yf, zf
            write (iout, 8431) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8440) xf, yf, zf
            write (iout, 8440) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = -sqrt (ac / (azz + small) - ac / (axx + small))
          yf = 0.0
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8430) xf, yf, zf
            write (iout, 8430) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8431) xf, yf, zf
            write (iout, 8431) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8440) xf, yf, zf
            write (iout, 8440) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif
        endif

 8450 format ('  z = 0 focus     ',1p3e20.12)
 8451 format ('  z''= 0 focus     ',1p3e20.12)
 8460 format ('  ", original xyz ',1p3e20.12)

        errxy = tol * (abs (axx) + abs (ayy))

        if ((ayy - axx) .gt. errxy) then  ! z = 0:  focus is on x axis.
 
          xf = sqrt (ac / (ayy + small) - ac / (axx + small))
          yf = 0.0
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8450) xf, yf, zf
            write (iout, 8450) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8451) xf, yf, zf
            write (iout, 8451) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8460) xf, yf, zf
            write (iout, 8460) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = -sqrt (ac / (ayy + small) - ac / (axx + small))
          yf = 0.0
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8450) xf, yf, zf
            write (iout, 8450) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8451) xf, yf, zf
            write (iout, 8451) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8460) xf, yf, zf
            write (iout, 8460) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif
        elseif ((axx - ayy) .gt. errxy) then  ! z = 0:  focus on y axis.
          xf = 0.0
          yf = sqrt (ac / (axx + small) - ac / (ayy + small))
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8450) xf, yf, zf
            write (iout, 8450) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8451) xf, yf, zf
            write (iout, 8451) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8460) xf, yf, zf
            write (iout, 8460) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif

          xf = 0.0
          yf = -sqrt (ac / (axx + small) - ac / (ayy + small))
          zf = 0.0
          if (kstd .eq. 1) then
            write (   6, 8450) xf, yf, zf
            write (iout, 8450) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          else
            write (   6, 8451) xf, yf, zf
            write (iout, 8451) xf, yf, zf
            call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
            write (   6, 8460) xf, yf, zf
            write (iout, 8460) xf, yf, zf
            call plotpt (xf, yf, zf, nerrg)
          endif
        endif
      endif                           ! Ellipsoid.

c=======================================================================********

c.... Hyperboloid of 1 sheet:   -1+qxx*x**2+qyy*y**2-|qzz|*z**2 = 0  (type 13).

c.... Find foci of hyperboloids of one sheet, in standard and original form.

      if (ntype .eq. 13) then         ! Hyperboloid of one sheet.

c....   Find foci on x axis, in plane z = 0.
 
        xf = sqrt (ac / (azz + small) - ac / (axx + small))
        yf = 0.0
        zf = 0.0
        if (kstd .eq. 1) then
          write (   6, 8640) xf, yf, zf
          write (iout, 8640) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        else
          write (   6, 8641) xf, yf, zf
          write (iout, 8641) xf, yf, zf
          call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
          write (   6, 8460) xf, yf, zf
          write (iout, 8460) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        endif

        xf = -sqrt (ac / (azz + small) - ac / (axx + small))
        yf = 0.0
        zf = 0.0
        if (kstd .eq. 1) then
          write (   6, 8640) xf, yf, zf
          write (iout, 8640) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        else
          write (   6, 8641) xf, yf, zf
          write (iout, 8641) xf, yf, zf
          call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
          write (   6, 8460) xf, yf, zf
          write (iout, 8460) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        endif

c.... Find foci on y axis, in plane z = 0.

        xf = 0.0
        yf = sqrt (ac / (azz + small) - ac / (ayy + small))
        zf = 0.0
        if (kstd .eq. 1) then
          write (   6, 8620) xf, yf, zf
          write (iout, 8620) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        else
          write (   6, 8621) xf, yf, zf
          write (iout, 8621) xf, yf, zf
          call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
          write (   6, 8460) xf, yf, zf
          write (iout, 8460) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        endif

        xf = 0.0
        yf = -sqrt (ac / (azz + small) - ac / (ayy + small))
        zf = 0.0
        if (kstd .eq. 1) then
          write (   6, 8620) xf, yf, zf
          write (iout, 8620) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        else
          write (   6, 8621) xf, yf, zf
          write (iout, 8621) xf, yf, zf
          call aptmopv (rotq, 1, 0., 0., 0., xf, yf, zf, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf, yf, zf, 1, tol, nerra)
          write (   6, 8460) xf, yf, zf
          write (iout, 8460) xf, yf, zf
          call plotpt (xf, yf, zf, nerrg)
        endif

      endif                           ! Hyperboloid of 1 sheet.

c=======================================================================********

c.... Cylinder, hyperbolic:      (+/-)1 + qxx*x**2 - |qyy|*y**2 = 0  (type  5).

c.... Find two foci on a hyperbolic cylinder.

      if (ntype .eq. 5) then          ! Hyperbolic cylinder.
        zf1 = 0.0
        zf2 = 0.0
        if (ac .gt. 0.0) then
          xf1 = 0.0
          yf1 =  sqrt (ac / (axx + small) - ac / (ayy + small))
          xf2 = 0.0
          yf2 = -sqrt (ac / (axx + small) - ac / (ayy + small))
        else
          xf1 =  sqrt (ac / (ayy + small) - ac / (axx + small))
          yf1 = 0.0
          xf2 = -sqrt (ac / (ayy + small) - ac / (axx + small))
          yf2 = 0.0
        endif

        if (kstd .eq. 1) then

          write (   6, 8450) xf1, yf1, zf1
          write (iout, 8450) xf1, yf1, zf1
          call plotpt (xf1, yf1, zf1, nerrg)

          write (   6, 8450) xf2, yf2, zf2
          write (iout, 8450) xf2, yf2, zf2
          call plotpt (xf2, yf2, zf2, nerrg)

        else

          write (   6, 8451) xf1, yf1, zf1
          write (iout, 8451) xf1, yf1, zf1
          call aptmopv (rotq, 1, 0., 0., 0., xf1, yf1, zf1, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf1, yf1, zf1, 1, tol, nerra)
          write (   6, 8460) xf1, yf1, zf1
          write (iout, 8460) xf1, yf1, zf1
          call plotpt (xf1, yf1, zf1, nerrg)

          write (   6, 8451) xf2, yf2, zf2
          write (iout, 8451) xf2, yf2, zf2
          call aptmopv (rotq, 1, 0., 0., 0., xf2, yf2, zf2, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf2, yf2, zf2, 1, tol, nerra)
          write (   6, 8460) xf2, yf2, zf2
          write (iout, 8460) xf2, yf2, zf2
          call plotpt (xf2, yf2, zf2, nerrg)

        endif
      endif                           ! Hyperbolic cylinder.

c=======================================================================********
 
c.... Cylinder, elliptic:              -1 + qxx*x**2 + qyy*y**2 = 0  (type  6).

c.... Find two foci and latera recta on an elliptic cylinder.

      if (ntype .eq. 6) then          ! Elliptic cylinder.
        xf1 = 0.0
        yf1 = -sqrt (ac / (axx + small) - ac / (ayy + small))
        zf1 = 0.0
        xf2 = 0.0
        yf2 = -yf1
        zf2 = 0.0

        dlatrec = 2.0 * sqrt (-ac * ayy) / (axx + small)

        if (kstd .eq. 1) then

          write (   6, 8450) xf1, yf1, zf1
          write (iout, 8450) xf1, yf1, zf1
          call plotpt (xf1, yf1, zf1, nerrg)

          write (   6, 8450) xf2, yf2, zf2
          write (iout, 8450) xf2, yf2, zf2
          call plotpt (xf2, yf2, zf2, nerrg)

        else

          write (   6, 8451) xf1, yf1, zf1
          write (iout, 8451) xf1, yf1, zf1
          call aptmopv (rotq, 1, 0., 0., 0., xf1, yf1, zf1, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf1, yf1, zf1, 1, tol, nerra)
          write (   6, 8460) xf1, yf1, zf1
          write (iout, 8460) xf1, yf1, zf1
          call plotpt (xf1, yf1, zf1, nerrg)

          write (   6, 8451) xf2, yf2, zf2
          write (iout, 8451) xf2, yf2, zf2
          call aptmopv (rotq, 1, 0., 0., 0., xf2, yf2, zf2, 1, tol,
     &                  nerra)
          call apttran (dx, dy, dz, xf2, yf2, zf2, 1, tol, nerra)
          write (   6, 8460) xf2, yf2, zf2
          write (iout, 8460) xf2, yf2, zf2
          call plotpt (xf2, yf2, zf2, nerrg)

        endif
      endif                           ! Elliptic cylinder.

c=======================================================================********

c.... Hyperboloid of 2 sheets:   1+qxx*x**2+qyy*y**2-|qzz|*z**2 = 0  (type 14).

c.... Find foci on a hyperboloid of two sheets.

      if (ntype .eq. 14) then          ! Hyperboloid of two sheets.

        errxy = tol * (abs (axx) + abs (ayy))
        if (abs (axx - ayy) .le. errxy) then  ! Circular

          xf1 = 0.0
          yf1 = 0.0
          zf1 =  sqrt (ac / (axx + small) - ac / (azz + small))

          xf2 = 0.0
          yf2 = 0.0
          zf2 = -sqrt (ac / (axx + small) - ac / (azz + small))
  
          if (kstd .eq. 1) then
  
            write (   6, 8310) xf1, yf1, zf1
            write (iout, 8310) xf1, yf1, zf1
            write (   6, 8310) xf2, yf2, zf2
            write (iout, 8310) xf2, yf2, zf2
  
          else
  
            write (   6, 8311) xf1, yf1, zf1
            write (iout, 8311) xf1, yf1, zf1
            call aptmopv (rotq, 1, 0., 0., 0., xf1, yf1, zf1, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf1, yf1, zf1, 1, tol, nerra)
            write (   6, 8312) xf1, yf1, zf1
            write (iout, 8312) xf1, yf1, zf1
  
            write (   6, 8311) xf2, yf2, zf2
            write (iout, 8311) xf2, yf2, zf2
            call aptmopv (rotq, 1, 0., 0., 0., xf2, yf2, zf2, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf2, yf2, zf2, 1, tol, nerra)
            write (   6, 8312) xf2, yf2, zf2
            write (iout, 8312) xf2, yf2, zf2
  
          endif

        else                          ! Not circular.

c....     Find the foci in the plane y = 0.

          xf1 = 0.0
          yf1 = 0.0
          zf1 =  sqrt (ac / (axx + small) - ac / (azz + small))

          xf2 = 0.0
          yf2 = 0.0
          zf2 = -sqrt (ac / (axx + small) - ac / (azz + small))
  
          if (kstd .eq. 1) then
  
            write (   6, 8320) xf1, yf1, zf1
            write (iout, 8320) xf1, yf1, zf1
            write (   6, 8320) xf2, yf2, zf2
            write (iout, 8320) xf2, yf2, zf2
  
          else
  
            write (   6, 8321) xf1, yf1, zf1
            write (iout, 8321) xf1, yf1, zf1
            call aptmopv (rotq, 1, 0., 0., 0., xf1, yf1, zf1, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf1, yf1, zf1, 1, tol, nerra)
            write (   6, 8322) xf1, yf1, zf1
            write (iout, 8322) xf1, yf1, zf1
  
            write (   6, 8321) xf2, yf2, zf2
            write (iout, 8321) xf2, yf2, zf2
            call aptmopv (rotq, 1, 0., 0., 0., xf2, yf2, zf2, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf2, yf2, zf2, 1, tol, nerra)
            write (   6, 8322) xf2, yf2, zf2
            write (iout, 8322) xf2, yf2, zf2

          endif

c....     Find the foci in the plane x = 0.

          xf1 = 0.0
          yf1 = 0.0
          zf1 =  sqrt (ac / (ayy + small) - ac / (azz + small))

          xf2 = 0.0
          yf2 = 0.0
          zf2 = -sqrt (ac / (ayy + small) - ac / (azz + small))
  
          if (kstd .eq. 1) then
  
            write (   6, 8330) xf1, yf1, zf1
            write (iout, 8330) xf1, yf1, zf1
            write (   6, 8330) xf2, yf2, zf2
            write (iout, 8330) xf2, yf2, zf2
  
          else
  
            write (   6, 8331) xf1, yf1, zf1
            write (iout, 8331) xf1, yf1, zf1
            call aptmopv (rotq, 1, 0., 0., 0., xf1, yf1, zf1, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf1, yf1, zf1, 1, tol, nerra)
            write (   6, 8332) xf1, yf1, zf1
            write (iout, 8332) xf1, yf1, zf1
  
            write (   6, 8331) xf2, yf2, zf2
            write (iout, 8331) xf2, yf2, zf2
            call aptmopv (rotq, 1, 0., 0., 0., xf2, yf2, zf2, 1, tol,
     &                    nerra)
            call apttran (dx, dy, dz, xf2, yf2, zf2, 1, tol, nerra)
            write (   6, 8332) xf2, yf2, zf2
            write (iout, 8332) xf2, yf2, zf2

          endif

        endif                         ! Tested for circular.

      endif                           ! hyperboloid of two sheets.

c=======================================================================********

c.... Find the distance between two parallel planes.

 8500 format ('  Distance betw pl',1pe20.12 )

      if (aqtype(n) .eq. aqtypes(2)) then
        dist = 2.0 / sqrt (axx)
        write (   6, 8500) dist
        write (iout, 8500) dist
      endif

c=======================================================================********

c.... Find the invariants of the quadric surface.

      trace = 2.0 * (qxx(n) + qyy(n) + qzz(n))
      errtr = 2.0 * tol * (abs (qxx(n)) + abs (qyy(n)) + abs (qzz(n)))
      if (abs (trace) .le. errtr) trace = 0.0

      dsum  = 4.0 * (qxx(n) * qyy(n) +
     &               qyy(n) * qzz(n) +
     &               qzz(n) * qxx(n) -
     &               (qxy(n)**2 + qyz(n)**2 + qzx(n)**2))
      errds = 4.0 * tol * (abs (qxx(n) * qyy(n)) +
     &                     abs (qyy(n) * qzz(n)) +
     &                     abs (qzz(n) * qxx(n)) +
     &                    (qxy(n)**2 + qyz(n)**2 + qzx(n)**2))
      if (abs (dsum) .le. errds) dsum = 0.0

      det   = 2.0 * (4.0 * qxx(n) * qyy(n) * qzz(n) +
     &                     qxy(n) * qyz(n) * qzx(n) -
     &                    (qxx(n) * qyz(n)**2 +
     &                     qyy(n) * qzx(n)**2 +
     &                     qzz(n) * qxy(n)**2))
      errdt = 2.0 * tol * (4.0 * abs (qxx(n) * qyy(n) * qzz(n)) +
     &                           abs (qxy(n) * qyz(n) * qzx(n)) +
     &                          (abs (qxx(n)) * qyz(n)**2 +
     &                           abs (qyy(n)) * qzx(n)**2 +
     &                           abs (qzz(n)) * qxy(n)**2))
      if (abs (det) .le. errdt) det = 0.0

 8501 format ('  trace,dsum,det= ',1p3e20.12)


      write (   6, 8501) trace, dsum, det
      write (iout, 8501) trace, dsum, det

c=======================================================================********

c.... Find radius, area and volume of a sphere.

 8502 format ('  Sphere area     ',1pe20.12 /
     &        '  Sphere volume   ',1pe20.12 )

      if (aqtype(n) .eq. aqtypes(16)) then
        radius = 1.0 / sqrt (axx)
        area   = 4.0 * pi / (axx + small)
        volume = 4.0 * pi * radius**3 / 3.0
        write (   6, 8502) area, volume
        write (iout, 8502) area, volume
      endif

c=======================================================================********

c.... Find area (if circular) and volume of an ellipsoid.

 8503 format ('  Ellipsoid area  ',1pe20.12 )
 8504 format ('  Ellipsoid volume',1pe20.12 )

      if (aqtype(n) .eq. aqtypes(15)) then
        if (asym(n) .eq. 'axisymmetric') then             ! Spheroid.
          eccen = sqrt (1.0 - azz / (axx + small))
          if (abs (semix - semiy) .le. tol * semix) then  ! Prolate.
            area = 2.0 * pi * semix**2 *
     &             (1.0 + (semiz / semix) * asin (eccen) / eccen)
          else                                            ! Oblate.
            area = 2.0 * pi * semiz**2 *
     &             (1.0 + 0.5 * ((semix / semiz)**2 / eccen) *
     &             log ((1.0 + eccen) / (1.0 - eccen)))
          endif
          write (   6, 8503) area
          write (iout, 8503) area
        endif
        volume = 4.0 * pi * semix * semiy * semiz / 3.0
        write (   6, 8504) volume
        write (iout, 8504) volume
      endif

c=======================================================================********

c.... Find radius, circumference and circular area of a circular cylinder.

 8505 format ('  Circumference   ',1pe20.12 /
     &        '  Area of circle  ',1pe20.12 )

      if (aqtype(n) .eq. aqtypes(7)) then
        radius = 1.0 / sqrt (axx)
        circum = 2.0 * pi * radius
        area   = pi / (axx + small)
        write (   6, 8505) circum, area
        write (iout, 8505) circum, area
      endif

c=======================================================================********

c.... Find zones defined by this surface.

 8510 format ('    Defines zone     ',a)

      do 520 nz = 1, nzones
        if (azone(nz) .eq. ' ') go to 520
        do 510 nq = 1, nzquads(nz)
          if ((azquad(nq,nz) .eq. aquad(n))  .or.
     &        (azquad(nq,nz) .eq. aquad(n))) then
            write (   6, 8510) azone(nz)
            write (iout, 8510) azone(nz)
          endif
  510   continue
  520 continue

      endif                           ! Printed maximum amount of output.

c=======================================================================********

  120 continue                        ! End of loop over quadric surfaces.

      return

c.... End of subroutine quout.        (+1 line)
      end
