      subroutine qustore (ntype, nrep, n)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  qustore  Updated     1994 July 20  9:40.
c               qustore  Originated  1994 July 20  9:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the index in the quadric surface array in which to
c               store the quadric surface being created.
c
c     Logic:    Entry points:  qustore.
c qustore       Called by:  cone, cylinder, ellipsoid, hyperb, plane, quadric,
c               qumove, quoper, sphere.
c
c qustore       Calls:  quout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     n         Output   0 if quadric surface array is full.  No room for a new
c                          quadric surface.  Positive if the index where the new
c                          quadric surface is to be stored.
c
c     nrep      Output   0 if quadric surface name is new, 1 if an old quadric
c                          surface with the same name is being replaced.
c
c     ntype     Input    The type of the new quadric surface.
c                          -1 to ignore.
c                           0 for a plane.
c                           7 for a circular cylinder.
c                          12 for a circular cone.
c                          16 for a sphere.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      n = 0

c.... See if the quadric surface name already exists.

 8110 format ('  WARNING:  replacing existing quadric:  ',a)
 8120 format ('  qustore WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric surface being replaced is not of same type.')

      nrep = 0

      do 110 nq = 1, nquads
        if (aword(2) .eq. aquad(nq)) then
          nrep = 1
          n    = nq
          write (   6, 8110) aquad(nq)
          write (iout, 8110) aquad(nq)
          if (ntype .ne. -1) then
            if (aqtype(nq) .ne. aqtypes(ntype)) then
              write (   6, 8120) ainput, ninput
              write (iout, 8120) ainput, ninput
            endif
          endif
          call quout (nq, nq, 'min', nerrq)
          go to 999
        endif
  110 continue

c.... See if a null quadric name can be replaced.

      do 120 nq = 1, nquads
        if (aquad(nq) .eq. ' ') then
          n = nq
          go to 999
        endif
  120 continue

c.... See if the maximum number of quadric surfaces has been exceeded.

 8130 format ('  qustore ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of quadric surfaces exceeds limit ',i5)

      if ((nquads + 1) .gt. nquadm) then
        write (   6, 8130) ainput, ninput, nquadm
        write (iout, 8130) ainput, ninput, nquadm
        n = 0
        go to 999
      endif

c.... Add a new quadric surface at the end of the list.

      nquads = nquads + 1

      n = nquads

c=======================================================================********

  999 return

c.... End of subroutine qustore.      (+1 line)
      end
