      subroutine rootf

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  rootf    Updated     1997 September 26 13:00.
c               rootf    Originated  1997 August 18 13:10
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To factor a real root out of a polynomial equation with real
c               coefficients C0, C1, C2, C3, C4, ...:
c               C0 + C1 * x + C2 * x**2 + C3 * x**3 + C4 * x**4 + ... = 0.
c
c     Logic:    Entry points:  rootf.
c rootf         Called by:  helpall, main.
c
c rootf         Calls:  aptnewt, aptpolf, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension a(0:64)               ! Coefficients of polynomial equation.
      dimension b(0:64)               ! Coefficients of factored equation.
      dimension berr(0:64)            ! Error in calculating b (first method).
      dimension c(0:64)               ! Coefficients of factored equation.
      dimension cerr(0:64)            ! Error in calculating c (second method).

c=======================================================================********

c.... Input line:  rootf [help]

 8110 format ('  Command "rootf":  acceptable input lines include:' /
     &  '  "rootf [help]"           Display this message.' //
     &  '  "rootf XROOT A(0) A(1) A(2) A(3) ... A(N)"' /
     &  '     Factor the term (x - XROOT) out of the polynomial' /
     &  '     equation P(x) = sum(n=0,N) {A(n) * x**n} = 0.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  rootf XROOT A(0) A(1) A(2) A(3) ... A(N)

      nerr = 0

 8120 format ('  rootf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not enough words on "rootf" line:  ', i3)

      if (nword .lt. 4) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the input data.

      call varfind (2, nword)

 8130 format ('  rootf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      do 140 nw = 2, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  140 continue

c.... Find the root.

 8140 format ('  rootf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Root XROOT is zero.')

      xroot = fword(2)

      if (nerr .ne. 0) go to 999

c.... Find the order of the polynomial equation.

      norda = nword - 3

c.... Find the coefficients of the polynomial equation.

      do 150 n = 0, norda
        a(n) = fword(n+3)
  150 continue

  350 call aptpolf (xroot, a, norda, tol, nz, b, nb, nerra)

      nax = nb + 1

 8350 format ('Factored out ',i2,' roots at zero.')

      if (nz .ne. 0) then
        write (   6, 8350) nz
        write (iout, 8350) nz
      endif

 8360 format ('  rootf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Removal of zeros reduced order to less than 2:  ',i3)
 8160 format ('  rootf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  XROOT is not a root of the polynomial.')

      if (nerra .eq. 1) then          ! Insufficient order.
        write (   6, 8360) ainput, ninput, nax
        write (iout, 8360) ainput, ninput, nax
        go to 999
      elseif (nerra .eq. 2) then      ! Root is zero.
        write (   6, 8140) ainput, ninput
        write (iout, 8140) ainput, ninput
        go to 999
      elseif (nerra .eq. 3) then      ! Root is not a root.
        write (   6, 8160) ainput, ninput
        write (iout, 8160) ainput, ninput

        if (xroot .eq. 0.0) then
          xmin = -1.0
          xmax =  1.0
        else
          xmin = xroot - abs (xroot)
          xmax = xroot + abs (xroot)
        endif

        nax   = nb + 1
        nzero = norda - nax

        call aptnewt (a(nzero), nax, xroot, xmin, xmax, 100, tol,
     &                xroota, nerra)

        if (nerra .ne. 0) go to 999

 8170   format ('  Root found by Newtonian iteration:' /
     &          '  x (root)        ',1pe20.12)     

        xroot = xroota
        write (   6, 8170) xroot
        write (iout, 8170) xroot
        go to 350

      endif                           ! Error in aptpolf.

 8330 format ('  set   b',i1,'      = ',1pe20.12,'  ! Cut and paste.')
 8340 format ('  set   b',i2,'     = ',1pe20.12,'  ! Cut and paste.')

      do 340 n = 0, nb
        if (n .lt. 10) then
          write (   6, 8330) n, b(n)
          write (iout, 8330) n, b(n)
        else
          write (   6, 8340) n, b(n)
          write (iout, 8340) n, b(n)
        endif
  340 continue
    
  999 return

c.... End of subroutine rootf.        (+1 line)
      end
