      subroutine roots

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  roots    Updated     2005 December 20 14:40.
c               roots    Originated  1994 November 23 17:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the roots (real and/or complex) of the polynomial
c               equation with real coefficients A0, A1, A2, A3 and A4:
c               A0 + A1 * z + A2 * z**2 + A3 * z**3 + A4 * z**4 = 0.
c               which may be linear (A1 nonzero, A2 = A3 = A4 = 0),
c               quadratic (A2 nonzero, A3 = A4 = 0), cubic (A3 nonzero, A4 = 0)
c               or quartic (A4 nonzero).
c
c     Logic:    Entry points:  roots.
c roots         Called by:  helpall, main.
c
c roots         Calls:  aptcubs, aptqrts, aptquar, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /lroots/ xroot(4)        ! Real part of a root.
      common /lroots/ yroot(4)        ! Imaginary part of a root.
      common /lroots/ artype(4)       ! The type of root.
      character*8     artype          ! The type of root.

c=======================================================================********

c.... Input line:  roots [help]

 8110 format ('  Command "roots":  acceptable input lines include:' /
     &  '  "roots [help]"           Display this message.' //
     &  '  "roots A0 A1 [A2 [A3 [A4]]]"' /
     &  '   Find the real and/or any complex roots of the' /
     &  '     polynomial equation:' /
     &  '     A0 + A1*z + A2*z**2 + A3*z**3 + A4*z**4 = 0.' //
     &  '  "roots = X1 [Y1 X2 Y2 [X3 Y3 [X4 Y4]]]"' /
     &  '   Find the coefficients of the polynomial equation',
     &  ' with the roots' /
     &  '     X1 + Y1 * i, [X2 + Y2 * i,',
     &  ' [X3 + Y3 * i, [X4 + Y4 * i]]].' /
     &  '   Specify any real roots first, with the imaginary part',
     &  ' zero,' /
     &  '   followed by any complex roots, in conjugate pairs.' )

 8112 format (/ '  "roots value X Y A0 A1 [A2|0] [A3|0] A4|0]"' /
     &  '   For z = X + i*Y, find the value of the polynomial',
     &  ' equation:' /
     &  '     A0 + A1*z + A2*z**2 + A3*z**3 + A4*z**4 = 0.' /
     &  '     and use Newton''s iteration to search for a root.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c=======================================================================********

c.... Input line:  roots = X1 Y1 X2 Y2 [X3 Y3 [X4 Y4]]

 8010 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Less than 3 or more than 10 words on "roots =" line.' )
 8020 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Complex numbers are not conjugate on "roots =" line.' )

      if (aword(2) .eq. '=') then

        nerr = 0

        if ((nword .lt. 3) .or. (nword .gt. 10)) then
          write (   6, 8010) ainput, ninput
          write (iout, 8010) ainput, ninput
          nerr = 1
        endif

c....   Find numerical values of roots.

        call varfind (3, nword)

c....   Verify that all are integer or floating point.

        do 110 nw = 3, nword
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
            nerr = 1
          endif
  110   continue

 8030 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Single complex root on "roots =" line.' )

        if ((nword .eq. 4)       .and.
     &      (fword(4) .ne. 0.0)) then
          write (   6, 8030) ainput, ninput
          write (iout, 8030) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

 8040   format ('Coefficients of the equation for the roots:' /
     &          '  set a0         =',1pe20.12,'  ! Cut and paste.' /
     &          '  set a1         =',1pe20.12,'  ! Cut and paste.' /
     &          '  set a2         =',1pe20.12,'  ! Cut and paste.' /
     &          '  set a3         =',1pe20.12,'  ! Cut and paste.' /
     &          '  set a4         =',1pe20.12,'  ! Cut and paste.' )

c....   Find the coefficients of a linear equation.

        if ((nword .eq. 3) .or. (nword .eq. 4)) then

          a0 = -fword(3)
          a1 = 1.0
          a2 = 0.0
          a3 = 0.0
          a4 = 0.0
          write (   6, 8040) a0, a1, a2, a3, a4
          write (iout, 8040) a0, a1, a2, a3, a4
          go to 999

        endif

c....   Find the coefficients of a quadratic equation.

        if (nword .eq. 6) then        ! Quadratic equation.

          if ((fword(4) + fword(6)) .ne. 0.0) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          if ((fword(4) .ne. 0.0) .and. (fword(5) .ne. fword(3))) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          a0 = fword(3) * fword(5) + fword(4) * fword(6)
          a1 = -(fword(3) + fword(5))
          a2 = 1.0
          a3 = 0.0
          a4 = 0.0
          write (   6, 8040) a0, a1, a2, a3, a4
          write (iout, 8040) a0, a1, a2, a3, a4
          go to 999
        endif

c....   Find the coefficients of a cubic equation.

        if (nword .eq. 8) then        ! Cubic equation.

          if (fword(4) .ne. 0.0) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          if ((fword(6) + fword(8)) .ne. 0.0) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          if ((fword(6) .ne. 0.0) .and. (fword(7) .ne. fword(5))) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          a0 = -fword(3) * (fword(5) * fword(7) - fword(6) * fword(8))
          a1 =  fword(3) * fword(5) + fword(5) * fword(7)  +
     &          fword(7) * fword(3) - fword(6) * fword(8)
          a2 = -(fword(3) + fword(5) + fword(7))
          a3 = 1.0
          a4 = 0.0
          write (   6, 8040) a0, a1, a2, a3, a4
          write (iout, 8040) a0, a1, a2, a3, a4
          go to 999
        endif

c....   Find the coefficients of a quartic equation.

        if (nword .eq. 10) then       ! Quartic equation.

          if ((fword(4) + fword(6)) .ne. 0.0) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          if ((fword(4) .ne. 0.0) .and. (fword(5) .ne. fword(3))) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          if ((fword(8) + fword(10)) .ne. 0.0) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          if ((fword(8) .ne. 0.0) .and. (fword(9) .ne. fword(7))) then
            write (   6, 8020) ainput, ninput
            write (iout, 8020) ainput, ninput
            go to 999
          endif

          a0 =   fword(3) *  fword(5) *  fword(7) * fword(9)   -
     &          (fword(3) *  fword(5) *  fword(8) * fword(10)   +
     &           fword(5) *  fword(7) *  fword(10) * fword(4)   +
     &           fword(7) *  fword(9) *  fword(4) * fword(6)   +
     &           fword(9) *  fword(3) *  fword(6) * fword(8)   +
     &           fword(3) *  fword(7) *  fword(6) * fword(10)   +
     &           fword(5) *  fword(9) *  fword(4) * fword(8))  +
     &           fword(4) *  fword(6) *  fword(8) * fword(10)
          a1 = -(fword(3) *  fword(5) * (fword(7) + fword(9))  +
     &           fword(7) *  fword(9) * (fword(3) + fword(5))) +
     &           fword(3) * (fword(6) * (fword(8) + fword(10))  +
     &                       fword(8) *  fword(10)           )  +
     &           fword(5) * (fword(8) * (fword(10) + fword(4))  +
     &                       fword(10) *  fword(4)           )  +
     &           fword(7) * (fword(10) * (fword(4) + fword(6))  +
     &                       fword(4) *  fword(6)           )  +
     &           fword(9) * (fword(4) * (fword(6) + fword(8))  +
     &                       fword(6) *  fword(8)              )
          a2 =   fword(3) *  fword(5) +  fword(5) * fword(7)   +
     &           fword(7) *  fword(9) +  fword(9) * fword(3)   +
     &           fword(3) *  fword(7) +  fword(5) * fword(9)   -
     &          (fword(4) *  fword(6) +  fword(6) * fword(8)   +
     &           fword(8) *  fword(10) +  fword(10) * fword(4)   +
     &           fword(4) *  fword(8) +  fword(6) * fword(10)   )
          a3 = -(fword(3) +  fword(5)  +  fword(7)  + fword(9))
          a4 = 1.0
          write (   6, 8040) a0, a1, a2, a3, a4
          write (iout, 8040) a0, a1, a2, a3, a4

          go to 999
        endif

 8050 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of arguments is not 3, 6, 8 or 10.' )

        write (   6, 8050) ainput, ninput
        write (iout, 8050) ainput, ninput

        go to 999
      endif

c=======================================================================********

c.... Input line:  roots value X Y A0 A1 [A2|0] [A3|0] [A4|0]

 8510 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Less than 6 or more than 9 words on "roots value" line.' )

      if (aword(2) .eq. 'value') then ! Test root, use Newton's iteration.

        nerr = 0

  540   if ((nword .lt. 6) .or. (nword .gt. 9)) then
          write (   6, 8510) ainput, ninput
          write (iout, 8510) ainput, ninput
          nerr = 1
        endif

c....   Find the numerical values of the arguments.

        call varfind (3, nword)

c....   Verify that all are integer or floating point.

        do 510 nw = 3, nword
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
            nerr = 1
          endif
  510   continue

        if (nerr .ne. 0) go to 999

        if (fword(nword) .eq. 0.0) then
          nword = nword - 1
          go to 540
        endif

        if (fword(nword) .eq. 0.0) then
          nword = nword - 1
        endif

        if (nword .lt. 9) then
          do 560 nw = nword + 1, 9
            fword(nw) = 0.0
  560     continue
        endif

c....   Find complex argument, and coefficients of polynomial.

        niter     = 0
        xroot(1)  = fword (3)
        yroot(1)  = fword (4)

        a0 = fword(5) / fword(nword)
        a1 = fword(6) / fword(nword)
        a2 = fword(7) / fword(nword)
        a3 = fword(8) / fword(nword)
        a4 = fword(9) / fword(nword)

c....   Find a nonzero value of y that will make the imaginary part zero.

        fun1    = a1 + xroot(1) * (2.0 * a2 + xroot(1) * (3.0 * a3 +
     &            xroot(1) * (4.0 * a4)))
        fun2    = a3 + 4.0 * a4 * xroot(1)
        fun2err = tol * (abs (a3) + 4.0 * abs (a4 * xroot(1)))
        if (abs (fun2) .le. fun2err) fun2 = 0.0

 8571   format (/ '  Try x, y       =',1p2e20.12,
     &    '  (real, imaginary)')

        if ((fun2 .ne. 0.0)  .and.
     &      (fun1 .ne. 0.0)) then
          arg     = fun1 / fun2
          if (arg .gt. 0.0) then
            yzero   = sqrt (arg)
            if (abs (yzero) .ne. abs (yroot(1))) then
              write (iout, 8571) xroot(1), yzero
              write (   6, 8571) xroot(1), yzero
            endif
          endif
        endif

 8572   format ('  Iterate  ',i4,'  =',1p2e20.12,'  (real, imaginary)')

        write (iout, 8572) niter, xroot(1), yroot(1)
        write (   6, 8572) niter, xroot(1), yroot(1)
        xroots = xroot(1)
        yroots = yroot(1)
        xslops = 0.0
        yslops = 0.0
        frress = 1.e99
        firess = 1.e99

c....   Start and return here for each Newton's iteration.

  570   niter = niter + 1             ! Number of Newton's iterations.

c....   Find polynomial value at specified argument, and allowed error in root.

        frres = a4 * xroot(1)**4 + a3 * xroot(1)**3 + a2 * xroot(1)**2 +
     &          a1 * xroot(1) + a0 + yroot(1)**2 *
     &          (a4 * yroot(1)**2 - 6.0 * a4 * xroot(1)**2 -
     &          3.0 * a3 * xroot(1) - a2)

        frerr = tol * (abs (a4) * xroot(1)**4 + abs (a3 * xroot(1)**3) +
     &          abs (a2) * xroot(1)**2 + abs (a1 * xroot(1)) +
     &          abs (a0) + yroot(1)**2 *
     &          (abs (a4) * yroot(1)**2 + 6.0 * abs (a4) * xroot(1)**2 +
     &          3.0 * abs (a3 * xroot(1)) + abs (a2)))


        fires = yroot(1) * (4.0 * a4 * xroot(1) *
     &          (xroot(1)**2 - yroot(1)**2) +
     &          a3 * (3.0 * xroot(1)**2 - yroot(1)**2) +
     &          2.0 * a2 * xroot(1) + a1)

        fierr = tol * (abs (yroot(1)) * (4.0 * abs (a4 * xroot(1)) *
     &          (xroot(1)**2 + yroot(1)**2) +
     &          abs (a3) * (3.0 * xroot(1)**2 + yroot(1)**2) +
     &          2.0 * abs (a2 * xroot(1)) + abs (a1)))

        write (   6, 8395) frres, fires, frerr, fierr
        write (iout, 8395) frres, fires, frerr, fierr

        frress = frres
        firess = fires

c....   Find slopes.

        xslop = a1 + 2.0 * a2 * xroot(1) + 3.0 * a3 * xroot(1)**2 +
     &          4.0 * a4 * xroot(1)**3 -
     &          3.0 * yroot(1)**2 * (a3 + 4.0 * a4 * xroot(1))

        xserr = tol * (abs (a1) + 2.0 * abs (a2 * xroot(1)) +
     &          3.0 * abs (a3) * xroot(1)**2 +
     &          4.0 * a4 * xroot(1)**3 +
     &          3.0 * yroot(1)**2 *
     &          (abs (a3) + 4.0 * abs (a4 * xroot(1))))

        yslop = 4.0 * a4 * yroot(1)**3 - 2.0 * yroot(1) *
     &          (a2 + 3.0 * a3 * xroot(1) + 6.0 * a4 * xroot(1)**2)

        yserr = tol * (4.0 * abs (a4 * yroot(1)**3) +
     &          2.0 * abs (yroot(1)) *
     &          (abs (a2) + 3.0 * abs (a3 * xroot(1)) +
     &          6.0 * abs (a4) * xroot(1)**2))

        if (abs (xslop) .le. xserr) xslop = 0.0
        if (abs (yslop) .le. yserr) yslop = 0.0

 8574   format ('    Slope        =',1p2e20.12,'  (real, imaginary)' )

        write (iout, 8574) xslop, yslop
        write (   6, 8574) xslop, yslop

c....   Stop iterating if polynomial value is less than allowed error.

        if ((abs (frres) .le. frerr)  .and.
     &      (abs (fires) .le. fierr)) go to 580

c....   Stop iterating if slope is zero.

 8575   format ('Stopped Newton''s iterations. Slope is zero.')

        slsq = xslop**2 + yslop**2
        if (slsq .eq. 0.0) then
          write (   6, 8576)
          write (iout, 8576)
          xroot(1) = xroots
          yroot(1) = yroots
          go to 580
        endif

c....   Stop iterating if slope rotates more than 90 degrees.

 8576   format ('Stopped Newton''s iterations.  Slope changed sign.')

        if ((xslop * xslops + yslop * yslops) .lt. 0.0) then
          write (   6, 8576)
          write (iout, 8576)
          xroot(1) = xroots
          yroot(1) = yroots
          go to 580
        endif

        xslops = xslop
        yslops = yslop

c....   Find corrections to x and y.

        xdel  =  (fires * yslop - frres * xslop) / slsq
        xderr =  tol * ((abs (fires * yslop) +
     &                   abs (frres * xslop))) / slsq
        if (abs (xdel) .le. xderr) xdel = 0.0

        ydel  = -(frres * yslop + fires * xslop) / slsq
        yderr =  tol * ((abs (frres * yslop) +
     &                   abs (fires * xslop))) / slsq
        if (abs (ydel) .le. yderr) ydel = 0.0

 8630   format ('    Delta x, y   =',1p2e20.12,'  (real, imaginary)' )

        write (iout, 8630) xdel, ydel
        write (   6, 8630) xdel, ydel

c....   Find next Newton's iterate.

        xroots   = xroot(1)
        yroots   = yroot(1)

        xroot(1) = xroot(1) + xdel
        yroot(1) = yroot(1) + ydel

        write (iout, 8572) niter, xroot(1), yroot(1)
        write (   6, 8572) niter, xroot(1), yroot(1)

c....   Do up to 100 iterations.

        if (niter .lt. 100) go to 570 ! Do another iteration.

c....   Stop iterating if no convergence after 100 iterations.

 8580   format ('Failed to converge on root in 100 Newton''s',
     &    ' iterations.')

          write (   6, 8580)
          write (iout, 8580)

c....   Stopped iterating.

  580   write (iout, 8572) niter, xroot(1), yroot(1)
        write (   6, 8572) niter, xroot(1), yroot(1)
        go to 999

      endif                           ! Input line "roots value".

c=======================================================================********

c.... Input line:  roots A0 A1 [A2|0]
c.... Input line:  roots A0 A1 A2 A3
c.... Input line:  roots A0 A1 A2 A3 A4

      nerr = 0

 8115 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Only two words on "roots" line.' )

      if (nword .eq. 2) then

        call varfind (2, 2)

        write (   6, 8115) ainput, ninput
        write (iout, 8115) ainput, ninput
        nerr = 1

      endif

c.... Test for the wrong number of words on "roots" line.

 8120 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words on "roots" line:  ', i3)

      if (nword .gt. 6) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the coefficients of the polynomial equation.

 8130 format ('  roots ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (2, nword)

      do 140 nw = 2, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  140 continue

      if (nerr .ne. 0) go to 999

c.... Find the order of the polynomial equation.

      if (nword .eq. 3) then
        nword    = 4
        fword(4) = 0.0
        mtype(4) = 3
      endif

      if ((nword .eq. 6) .and. (fword(6) .eq. 0.0)) then
        nword = 5
      endif

      if ((nword .eq. 5) .and. (fword(5) .eq. 0.0)) then
        nword = 4
      endif

c=======================================================================********

c.... Input line:  roots A0 A1 A2

      if (nword .eq. 4) then          ! Quadratic equation.

        a0 = fword(2)
        a1 = fword(3)
        a2 = fword(4)

 8240   format ('  A0,A1,A2(input) ',1p3e20.12)        

        write (   6, 8240) a0, a1, a2
        write (iout, 8240) a0, a1, a2

c....   Find any extrema.

        d0 = fword(3)
        d1 = fword(4) * 2.0

 8234   format (/ 'Quadratic equation has a ',a7,' at' /
     &            '  z (real)       =',1pe20.12 /
     &            '  P(z)           =',1pe20.12 )

        if (d1 .ne. 0.0) then
          xextr = -d0 / d1
          frres = a0 + a1 * xextr + a2 * xextr**2
          frerr = tol * ( abs (a0) + abs (a1 * xextr) +
     &            abs (a2) * xextr**2)
          if (abs (frres) .le. frerr) frres = 0.0
          if (d1 .lt. 0.0) then
            artype(1) = 'maximum'
          else
            artype(1) = 'minimum'
          endif
          write (   6, 8234) artype(1), xextr, frres
          write (iout, 8234) artype(1), xextr, frres
        endif

c....   Find the real or complex roots of the quadratic equation.

        call aptqrts (0, a2, a1, a0, qq, tol,
     &                nroots, xroot(1), xroot(2), itrun)

 8250   format (/ '  Root 1 (complex)',1p2e20.12,'  (real, imaginary)' /
     &          '  Root 2 (complex)',1p2e20.12,'  (real, imaginary)' )
 8260   format (/ '  Root 1 (single )',1pe20.12)        
 8265   format (/ '  Root 1 (double )',1pe20.12)        
 8270   format (/ '  Root 1 (single )',1pe20.12 /        
     &          '  Root 2 (single )',1pe20.12 )        
 8280   format (/ '  Null or bad equation.  A1 and A2 are zero.  A0 = ',
     &    1pe20.12)

        if (nroots .eq. 0) then
          yroot(1) = -xroot(2)
          xroot(2) =  xroot(1)
          yroot(2) = -yroot(1)
          write (   6, 8250) xroot(1), yroot(1), xroot(2), yroot(2)
          write (iout, 8250) xroot(1), yroot(1), xroot(2), yroot(2)
        elseif (nroots .eq. 1) then
          if ((a2 .ne. 0.0)  .and.
     &        (a1 .eq. 0.0)  .and.
     &        (a0 .eq. 0.0)) then
            write (   6, 8265) xroot(1)
            write (iout, 8265) xroot(1)
          elseif (itrun .eq. 0) then
            write (   6, 8260) xroot(1)
            write (iout, 8260) xroot(1)
          else
            write (   6, 8265) xroot(1)
            write (iout, 8265) xroot(1)
          endif
        elseif (nroots .eq. 2) then
          write (   6, 8270) xroot(1), xroot(2)
          write (iout, 8270) xroot(1), xroot(2)
        else
          write (   6, 8280) a0
          write (iout, 8280) a0
        endif

        go to 999
      endif                           ! Quadratic equation.

c=======================================================================********

c.... Input line:  roots A0 A1 A2 A3

      if (nword .eq. 5) then          ! Cubic equation.

 8140   format ('  A0,A1,A2(input) ',1p3e20.12 /
     &          '  A3      (input) ',1pe20.12 )

        write (   6, 8140) fword(2), fword(3), fword(4), fword(5)
        write (iout, 8140) fword(2), fword(3), fword(4), fword(5)

c....   Find the coefficients of the cubic equation.

        a0 = fword(2)
        a1 = fword(3)
        a2 = fword(4)
        a3 = fword(5)

c....   Find any real extrema.

        d0 = fword(3)
        d1 = fword(4) * 2.0
        d2 = fword(5) * 3.0

 8142   format (/ 'Cubic equation has no extrema.' )
 8144   format (/ 'Cubic equation has a ',a7,' at' /
     &            '  z (real)       =',1pe20.12 /
     &            '  P(z)           =',1pe20.12 )

        call aptqrts (0, d2, d1, d0, qq, tol,
     &                nroots, xroot(1), xroot(2), itrun)

        if (nroots .eq. 0) then
          write (   6, 8142)
          write (iout, 8142)
        else
          do 8145 n = 1, nroots
            frres = a0 + xroot(n) * (a1 + xroot(n) *
     &             (a2 + xroot(n) * a3))
            xabs  = abs (xroot(n))
            frerr = tol * ( abs (a0) + xabs * (abs (a1) + xabs *
     &             (abs (a2) + xabs * abs (a3))))
            if (abs (frres) .le. frerr) frres = 0.0
            if ((d1 + 2.0 * d2 * xroot(n)) .lt. 0.0) then
              artype(n) = 'maximum'
            elseif ((d1 + 2.0 * d2 * xroot(n)) .gt. 0.0) then
              artype(n) = 'minimum'
            elseif ((d1 + 2.0 * d2 * xroot(n)) .eq. 0.0) then
              artype(n) = 'min/max'
            endif
            write (   6, 8144) artype(n), xroot(n), frres
            write (iout, 8144) artype(n), xroot(n), frres
 8145     continue
        endif

c....   Find any inflection points.

        e0 = fword(4) * 2.0
        e1 = fword(5) * 6.0

 8154   format (/ 'Cubic equation has an inflection point at' /
     &            '  z (real)       =',1pe20.12 /
     &            '  P(z)           =',1pe20.12 )

        xinfl = -e0 / e1
        xabs  = abs (xinfl)
        frres = a0 + xinfl * (a1 + xinfl * (a2 + xinfl * a3))
        frerr = tol * (abs (a0) + xabs * (abs (a1) + xabs *
     &         (abs (a2) + xabs * abs (a3))))
        if (abs (frres) .le. frerr) frres = 0.0
        write (   6, 8154) xinfl, frres
        write (iout, 8154) xinfl, frres

c....   Normalize the coefficient of z**3 to 1.

        a0 = fword(2) / fword(5)
        a1 = fword(3) / fword(5)
        a2 = fword(4) / fword(5)

c....   Find any roots of the cubic equation.

        call aptcubs (a0, a1, a2, tol,
     &                nroots, xroot, yroot, artype, itrun)

 8350   format (/ '  No roots.  Method failed in aptcubs.' /
     &          '  Try using a different value of tol.' )
 8354   format (/ '  Found ',i1,' unique real root.')
 8355   format (/ '  Found ',i1,' unique real roots.')

        if (nroots .eq. 0) then
          write (   6, 8350)
          write (iout, 8350)
        elseif (nroots .eq. 1) then
          write (   6, 8354) nroots
          write (iout, 8354) nroots
        else
          write (   6, 8355) nroots
          write (iout, 8355) nroots
        endif

 8356   format ('  Minor numerical truncation occurred, less than tol.')
 8358   format ('  Major numerical truncation occurred,',
     &    ' more than tol.' /
     &          '  Try increasing tol.' )

        if (itrun .eq. 2) then
          write (   6, 8358)
          write (iout, 8358)
        elseif (itrun .eq. 1) then
          write (   6, 8356)
          write (iout, 8356)
        endif

 8360   format ('  Root ',i1,' (',a7,')',1pe20.12)
 8365   format ('  Root ',i1,' (',a7,')',1p2e20.12,
     &    '  (real, imaginary)')
 8390   format ('    P(x+i*y)     =',1pe20.12 /
     &          '    Max allowed  =',1pe20.12 )
 8395   format ('    P(x+i*y)     =',1p2e20.12,'  (real, imaginary)' /
     &          '    Max allowed  =',1p2e20.12,'  (real, imaginary)' )
 8398   format ('    1 / Root ',i1,'   =',1pe20.12 )
 8399   format ('    1 / Root ',i1,'   =',1p2e20.12,
     &    '  (real, imaginary)' )

        do 190 n = 1, 3
          if (artype(n) .eq. 'unknown') go to 200

          frres = fword(5) * (xroot(n)**3 + a2 * xroot(n)**2 +
     &            a1 * xroot(n) + a0 -
     &            yroot(n)**2 * (3.0 * xroot(n) + a2))

          frerr = tol * abs (fword(5)) * (abs (xroot(n)**3) +
     &            abs (a2) * xroot(n)**2 +
     &            abs (a1 * xroot(n)) + abs (a0) +
     &            yroot(n)**2 * (3.0 * abs (xroot(n)) + abs (a2)))

          fires = fword(5) * (yroot(n) * (3.0 * xroot(n)**2 -
     &            yroot(n)**2 + 2.0 * a2 * xroot(n) + a1))

          fierr = tol * abs (fword(5)) * (abs (yroot(n)) *
     &           (3.0 * xroot(n)**2 +
     &            yroot(n)**2 + 2.0 * abs (a2 * xroot(n)) + abs (a1)))

          if (artype(n) .ne. 'complex') then
            write (   6, 8360) n, artype(n), xroot(n)
            write (iout, 8360) n, artype(n), xroot(n)
            if (abs (frres) .gt. frerr) then
              write (   6, 8390) frres, frerr
              write (iout, 8390) frres, frerr
            endif
            xsq = xroot(n)**2
            if (xsq .ne. 0.0) then
              rreal =  xroot(n) / xsq
            else
              rreal = -1.e99
            endif
            write (iout, 8398) n, rreal
            write (   6, 8398) n, rreal
          else
            write (   6, 8365) n, artype(n), xroot(n), yroot(n)
            write (iout, 8365) n, artype(n), xroot(n), yroot(n)
            if ((abs (frres) .gt. frerr)  .or.
     &          (abs (fires) .gt. fierr)) then
              write (   6, 8395) frres, fires, frerr, fierr
              write (iout, 8395) frres, fires, frerr, fierr
            endif
            xsq = xroot(n)**2 + yroot(n)**2
            if (xsq .ne. 0.0) then
              rreal =  xroot(n) / xsq
              rimag = -yroot(n) / xsq
            else
              rreal = -1.e99
              rimag = -1.e99
            endif
            write (iout, 8399) n, rreal, rimag
            write (   6, 8399) n, rreal, rimag
          endif

          if ((abs (frres) .gt. frerr)  .or.
     &        (abs (fires) .gt. fierr)) then
            write (iout, 8650)
            write (   6, 8650)
          endif

  190   continue                      ! End of loop over roots.

  200   continue                      ! Done with all roots.

c....   Reconstruct coefficients.

        if (nroots .eq. 1) then
          if (artype(1) .eq. 'single') then
            a2p = -(xroot(1) + xroot(2) + xroot(3))
            a1p = xroot(1) * (xroot(2) + xroot(3)) +
     &            xroot(2) * xroot(3) + yroot(2)**2
            a0p = -xroot(1) * (xroot(2) * xroot(3) + yroot(2)**2)
          else
            a2p = -3.0 * xroot(1)
            a1p = 3.0 * xroot(1)**2
            a0p = -xroot(1)**3
          endif
        elseif (nroots .eq. 2) then
          if (artype(1) .eq. 'single') then
            a2p = -(xroot(1) + 2.0 * xroot(2))
            a1p = 2.0 * xroot(1) * xroot(2) + xroot(2)**2
            a0p = -xroot(1) * xroot(2)**2
          else
            a2p = -(xroot(2) + 2.0 * xroot(1))
            a1p = 2.0 * xroot(1) * xroot(2) + xroot(1)**2
            a0p = -xroot(2) * xroot(1)**2
          endif
        elseif (nroots .eq. 3) then
          a2p = -(xroot(1) + xroot(2) + xroot(3))
          a1p = xroot(1) * (xroot(2) + xroot(3)) + xroot(2) * xroot(3)
          a0p = -xroot(1) * xroot(2) * xroot(3)
        endif

        a0p = fword(5) * a0p
        a1p = fword(5) * a1p
        a2p = fword(5) * a2p
        a0err = tol * (abs (fword(2)) + abs (a0p))
        a1err = tol * (abs (fword(3)) + abs (a1p))
        a2err = tol * (abs (fword(4)) + abs (a2p))

 8410   format ('  A0,A1,A2(input) ',1p3e20.12 /
     &          '  A0,A1,A2(recalc)',1p3e20.12 )

        if ((abs (fword(2) - a0p) .gt. a0err)  .or.
     &      (abs (fword(3) - a1p) .gt. a1err)  .or.
     &      (abs (fword(4) - a2p) .gt. a2err)) then
          write (   6, 8410) fword(2), fword(3), fword(4),
     &      a0p, a1p, a2p
          write (iout, 8410) fword(2), fword(3), fword(4),
     &      a0p, a1p, a2p
        endif

        go to 999

      endif                           ! Cubic equation.

c=======================================================================********

c.... Input line:  roots A0 A1 A2 A3 A4

      if (nword .eq. 6) then          ! Quartic equation.

 8430   format ('  A0,A1,A2(input) ',1p3e20.12 /
     &          '  A3,A4   (input) ',1p2e20.12 )

        write (   6, 8430) fword(2), fword(3), fword(4),
     &    fword(5), fword(6)
        write (iout, 8430) fword(2), fword(3), fword(4),
     &    fword(5), fword(6)

c....   Find the coefficients.

        a0 = fword(2)
        a1 = fword(3)
        a2 = fword(4)
        a3 = fword(5)
        a4 = fword(6)

c....   Find any real extrema.

        d0 = 0.25 * fword(3) / fword(6)
        d1 = 0.50 * fword(4) / fword(6)
        d2 = 0.75 * fword(5) / fword(6)
        d3 = 1.0

 8432   format (/ 'Quartic equation has no extrema.' )
 8434   format (/ 'Quartic equation has a ',a7,' at' /
     &            '  z (real)       =',1pe20.12 /
     &            '  P(z)           =',1pe20.12 )

        call aptcubs (d0, d1, d2, tol,
     &                nroots, xroot, yroot, artype, itrun)

        if (nroots .eq. 0) then
          write (   6, 8432)
          write (iout, 8432)
        else
          do 8435 n = 1, nroots
            frres = a0 + xroot(n) * (a1 + xroot(n) * (a2 + xroot(n) *
     &             (a3 + xroot(n) * a4)))
            xabs  = abs (xroot(n))
            frerr = tol * (abs (a0) + xabs * (abs (a1) + xabs *
     &              (abs (a2) + xabs * (abs (a3) + xabs * abs (a4)))))
            if (abs (frres) .le. frerr) frres = 0.0
            xslop = d1 + xroot(n) * (2.0 * d2 + xroot(n) * 3.0 * d3)
            if (xslop .lt. 0.0) then
              artype(n) = 'maximum'
            elseif (xslop .gt. 0.0) then
              artype(n) = 'minimum'
            elseif (xslop .eq. 0.0) then
              artype(n) = 'min/max'
            endif
            write (   6, 8434) artype(n), xroot(n), frres
            write (iout, 8434) artype(n), xroot(n), frres
 8435     continue
        endif

c....   Find any real inflection points.

        e0 = fword(4) * 2.0
        e1 = fword(5) * 6.0
        e2 = fword(6) * 12.0

 8442   format (/ 'Quartic equation has no inflection points.' )
 8444   format (/ 'Quartic equation has an inflection point  at' /
     &            '  z (real)       =',1pe20.12 /
     &            '  P(z)           =',1pe20.12 )

        call aptqrts (0, e2, e1, e0, qq, tol,
     &                nroots, xroot(1), xroot(2), itrun)

        if (nroots .eq. 0) then
          write (   6, 8442)
          write (iout, 8442)
        elseif (nroots .ge. 1) then
          do 8445 n = 1, nroots
            frres = a0 + xroot(n) * (a1 + xroot(n) * (a2 + xroot(n) *
     &             (a3 + xroot(n) * a4)))
            xabs  = abs (xroot(n))
            frerr = tol * (abs (a0) + xabs * (abs (a1) + xabs *
     &              (abs (a2) + xabs * (abs (a3) + xabs * abs (a4)))))
            if (abs (frres) .le. frerr) frres = 0.0
            write (   6, 8444) xroot(n), frres
            write (iout, 8444) xroot(n), frres
 8445     continue
        endif

c....   Find any points where P'''(z) = 0.

        f0 = fword(5)
        f1 = fword(6) * 4.0

 8448   format (/ 'Quartic equation has a symmetry point at' /
     &            '  z (real)       =',1pe20.12 /
     &            '  P(z)           =',1pe20.12 )

        xthrd = -f0 / f1
        xabs  = abs (xthrd)
        frres = a0 + xthrd * (a1 + xthrd * (a2 + xthrd *
     &         (a3 + xthrd * a4)))
        frerr = tol * (abs (a0) + xabs * (abs (a1) + xabs *
     &          (abs (a2) + xabs * (abs (a3) + xabs * abs (a4)))))
        if (abs (frres) .le. frerr) frres = 0.0
        write (   6, 8448) xthrd, frres
        write (iout, 8448) xthrd, frres

c....   Normalize the coefficient of z**4 to 1.

        a0 = fword(2) / fword(6)
        a1 = fword(3) / fword(6)
        a2 = fword(4) / fword(6)
        a3 = fword(5) / fword(6)

c....   Find any roots of the quartic equation.

        call aptquar (a0, a1, a2, a3, tol,
     &                nroot, nrooti, xroot, yroot, artype, itrun)

 8450   format ('  No roots.  Method failed in aptquar.' /
     &          '  Try using a different value of tol.' )

        if ((nroot .eq. 0) .and. (nrooti .eq. 0)) then
          write (   6, 8450)
          write (iout, 8450)
        endif

 8451   format (/ '  Found no unique real root.')
 8452   format (/ '  Found ',i1,' unique real root.')
 8453   format (/ '  Found ',i1,' unique real roots.')
 8454   format (/ '  Found no complex roots.')
 8455   format (/ '  Found ',i1,' double complex roots',
     &  ' (in conjugate pairs).')
 8456   format (/ '  Found ',i1,' complex roots (in conjugate pairs).')

        if (nroot .eq. 0) then
          write (   6, 8451)
          write (iout, 8451)
        elseif (nroot .eq. 1) then
          write (   6, 8452) nroot
          write (iout, 8452) nroot
        else
          write (   6, 8453) nroot
          write (iout, 8453) nroot
        endif

        if (nrooti .eq. 0) then
          write (   6, 8454)
          write (iout, 8454)
        else
          if (nroot .eq. 0) then
            if (nrooti .eq. 2) then
              write (   6, 8455) nrooti
              write (iout, 8455) nrooti
            else
              write (   6, 8456) nrooti
              write (iout, 8456) nrooti
            endif
          else
            write (   6, 8456) nrooti
            write (iout, 8456) nrooti
          endif
        endif

 8457   format ('  Minor numerical truncation occurred, less than tol.')
 8458   format ('  Major numerical truncation occurred,',
     &    ' more than tol.' /
     &          '  Try increasing tol.' )

        if (itrun .eq. 2) then
          write (   6, 8458)
          write (iout, 8458)
        elseif (itrun .eq. 1) then
          write (   6, 8457)
          write (iout, 8457)
        endif

 8650   format ('    WARNING:  this root is not accurate.')

        do 390 n = 1, 4
          if (artype(n) .eq. 'unknown') go to 400

          frres = xroot(n)**4 + a3 * xroot(n)**3 + a2 * xroot(n)**2 +
     &            a1 * xroot(n) + a0 +
     &            yroot(n)**2 * (yroot(n)**2 - 6.0 * xroot(n)**2 -
     &            3.0 * a3 * xroot(n) - a2)

          frerr = tol * (xroot(n)**4 + abs (a3 * xroot(n)**3) +
     &            abs (a2) * xroot(n)**2 + abs (a1 * xroot(n)) +
     &            abs (a0) +
     &            yroot(n)**2 * (yroot(n)**2 + 6.0 * xroot(n)**2 +
     &            3.0 * abs (a3 * xroot(n)) + abs (a2)))


          fires = yroot(n) * (4.0 * xroot(n) *
     &            (xroot(n)**2 - yroot(n)**2) +
     &            a3 * (3.0 * xroot(n)**2 - yroot(n)**2) +
     &            2.0 * a2 * xroot(n) + a1)

          fierr = tol * (abs (yroot(n)) * (4.0 * abs (xroot(n)) *
     &            (xroot(n)**2 + yroot(n)**2) +
     &            abs (a3) * (3.0 * xroot(n)**2 + yroot(n)**2) +
     &            2.0 * abs (a2 * xroot(n)) + abs (a1)))

          write (iout, 8365) n, artype(n), xroot(n), yroot(n)
          write (   6, 8365) n, artype(n), xroot(n), yroot(n)
          if ((abs (frres) .gt. frerr)  .or.
     &        (abs (fires) .gt. fierr)) then
            write (iout, 8395) frres, fires, frerr, fierr
            write (   6, 8395) frres, fires, frerr, fierr
          endif

          xsq = xroot(n)**2 + yroot(n)**2
          if (xsq .ne. 0.0) then
            rreal =  xroot(n) / xsq
            rimag = -yroot(n) / xsq
          else
            rreal = -1.e99
            rimag = -1.e99
          endif
          write (iout, 8399) n, rreal, rimag
          write (   6, 8399) n, rreal, rimag

          if ((abs (frres) .gt. frerr)  .or.
     &        (abs (fires) .gt. fierr)) then
            write (iout, 8650)
            write (   6, 8650)
          endif

  390   continue

  400   continue

c....   Reconstruct the coefficients.

        a0p =   xroot(1) *  xroot(2) *  xroot(3) * xroot(4)   -
     &         (xroot(1) *  xroot(2) *  yroot(3) * yroot(4)   +
     &          xroot(2) *  xroot(3) *  yroot(4) * yroot(1)   +
     &          xroot(3) *  xroot(4) *  yroot(1) * yroot(2)   +
     &          xroot(4) *  xroot(1) *  yroot(2) * yroot(3)   +
     &          xroot(1) *  xroot(3) *  yroot(2) * yroot(4)   +
     &          xroot(2) *  xroot(4) *  yroot(1) * yroot(3))  +
     &          yroot(1) *  yroot(2) *  yroot(3) * yroot(4)

        a1p = -(xroot(1) *  xroot(2) * (xroot(3) + xroot(4))  +
     &          xroot(3) *  xroot(4) * (xroot(1) + xroot(2))) +
     &          xroot(1) * (yroot(2) * (yroot(3) + yroot(4))  +
     &                      yroot(3) *  yroot(4)           )  +
     &          xroot(2) * (yroot(3) * (yroot(4) + yroot(1))  +
     &                      yroot(4) *  yroot(1)           )  +
     &          xroot(3) * (yroot(4) * (yroot(1) + yroot(2))  +
     &                      yroot(1) *  yroot(2)           )  +
     &          xroot(4) * (yroot(1) * (yroot(2) + yroot(3))  +
     &                      yroot(2) *  yroot(3)              )

        a2p =   xroot(1) *  xroot(2) +  xroot(2) * xroot(3)   +
     &          xroot(3) *  xroot(4) +  xroot(4) * xroot(1)   +
     &          xroot(1) *  xroot(3) +  xroot(2) * xroot(4)   -
     &         (yroot(1) *  yroot(2) +  yroot(2) * yroot(3)   +
     &          yroot(3) *  yroot(4) +  yroot(4) * yroot(1)   +
     &          yroot(1) *  yroot(3) +  yroot(2) * yroot(4)   )

        a3p = -(xroot(1) +  xroot(2)  +  xroot(3)  + xroot(4))

        a0p = a0p * fword(6)
        a1p = a1p * fword(6)
        a2p = a2p * fword(6)
        a3p = a3p * fword(6)
        a0err = tol * (abs (fword(2)) + abs (a0p))
        a1err = tol * (abs (fword(3)) + abs (a1p))
        a2err = tol * (abs (fword(4)) + abs (a1p))
        a3err = tol * (abs (fword(5)) + abs (a1p))

 8660   format (/ 'Coefficient check:' /
     &          '  A0,A1(input)   =',1p2e20.12 /
     &          '  A0,A1(recalc)  =',1p2e20.12 //
     &          '  A2,A3(input)   =',1p2e20.12 /
     &          '  A2,A3(recalc)  =',1p2e20.12 )

        if ((abs (fword(2) - a0p) .gt. a0err)  .or.
     &      (abs (fword(3) - a1p) .gt. a1err)  .or.
     &      (abs (fword(4) - a2p) .gt. a2err)  .or.
     &      (abs (fword(5) - a3p) .gt. a3err)) then
          write (iout, 8660) fword(2), fword(3), a0p, a1p,
     &      fword(4), fword(5), a2p, a3p
          write (   6, 8660) fword(2), fword(3), a0p, a1p,
     &      fword(4), fword(5), a2p, a3p
        endif

        go to 999

      endif                           ! Quartic equation.

c=======================================================================********

  999 return

c.... End of subroutine roots.        (+1 line)
      end
