      subroutine search

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  search   Updated     2006 June 2 14:30.
c               search   Originated  2003 April 19 17:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To search aliases, markers, variables, big integers, vectors,
c               operators, points, clusters, lines, triangles, annular disks,
c               regular polygons, planes, spheres, circular cylinders,
c               circular cones, hyperbolic paraboloids, ellipsoids, axisymmetric
c               quadric surfaces, general quadric surfaces, regular polyhedrons,
c               tetrahedrons, bricks, zones, bins, probability distribution
c               functions (pdfs) and symbols, to find a specififed name, and
c               to search big integers, points, variables and vectors to find a
c               specified value.
c
c     Logic:    Entry points:  search.
c search        Called by:  helpall, main.
c
c search        Calls:  alout, aptbtod, aptcsyv, aptsubr, bigout, binout, brout,
c               clout, dkout, lout, markout, opout, pdfout, pgout, phout, pout,
c               ptcoord, quout, symbdisp, tetout, trout, varout, vout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*1  afirst             ! First character of object name.
      character*1  afirstr            ! First character of object text.
      character*80 astemr             ! Stem of subscripted object text.
      character*8  amount             ! Amount of output from bigout.

c.... Input line:  search [help]

 8110 format ('  Command "search":  acceptable input lines include:' /
     &  '  "search [help]"   Display this message.' //
     &  '  "search OBJNAME1 OBJNAME2 OBJNAME3 ..."' /
     &  '    Search for objects with names OBJNAME1, OBJNAME2,',
     &  ' OBJNAME3, ...,' /
     &  '    or beginning with single character OBJNAME1, OBJNAME2,'
     &  ' OBJNAME3, ...' //
     &  '    Search for variables, points or vectors with values' /
     &  '    OBJNAME1, OBJNAME2, OBJNAME3, ...,' //
     &  '    Search for big integers with integer or floating point',
     &  ' values' /
     &  '    or base 10 logs OBJNAME1, OBJNAME2, OBJNAME3, ...,' //
     &  '  "search all"      Display all objects.' //
     &  '  Synonyms:  [help, h], [search, find].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  search OBJNAME1 OBJNAME2 OBJNAME3 ...
c.... Input line:  search all

      do 400 nw = 2, nword            ! Loop over object names and values.
        numobj = 0

c===============================================================================

c.... See if alias exists.

      do 110 n = 1, nalias
        afirst  = alias1(n)
        afirstr = alias2(n)

        call aptsubr (alias1(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        call aptsubr (alias2(n), 1, 72, 80, astemr, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. alias1(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. afirstr)   .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. astemr)    .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call alout (n, n)
          go to 110
        endif

        call aptsubm (alias1(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call alout (n, n)
              go to 110
            endif
          enddo
        endif


        call aptsubm (alias2(n), 1, 72, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call alout (n, n)
              go to 110
            endif
          enddo
        endif

  110 continue

c===============================================================================

c.... See if big number exists.

      do n = 1, 26
        if ((aword(nw) .eq. abig(n)) .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          amount = 'maximum'
          call bigout (n, n, amount)
        endif
      enddo

c===============================================================================

c.... Find any numerical values among big numbers.

      if (mtype(nw) .eq. 2) then      ! Search for integer values.

        do n = 1, 26
          call aptbtod (10, idig(1,n), ndig(n), idmax, iemax,
     &                  ideca, fdeca, floga, nerra)
          if (iword(nw) .eq. ideca) then
            numobj = numobj + 1
            amount = 'maximum'
            call bigout (n, n, amount)
          endif
        enddo

      endif

      if (mtype(nw) .eq. 3) then      ! Search for floating point values.

        do n = 1, 26
          call aptbtod (10, idig(1,n), ndig(n), idmax, iemax,
     &                  ideca, fdeca, floga, nerra)
          ferr = tol * (abs (fword(nw)) + abs (fdeca))
          if (abs (fword(nw) - fdeca) .le. ferr) then
            numobj = numobj + 1
            amount = 'maximum'
            call bigout (n, n, amount)
          endif
          ferr = tol * (abs (fword(nw)) + abs (floga))
          if (abs (fword(nw) - floga) .le. ferr) then
            numobj = numobj + 1
            amount = 'maximum'
            call bigout (n, n, amount)
          endif
        enddo

      endif                           ! Search for integer values.

c===============================================================================

c.... See if bin exists.

      do 140 n = 1, nbins
        afirst = abin(n)

        call aptsubr (abin(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. abin(n)) .or.
     &      (aword(nw) .eq. afirst)  .or.
     &      (aword(nw) .eq. astem)   .or.
     &      (aword(nw) .eq. 'all'))  then
          numobj = numobj + 1
          call binout (n, n, 'min')
          go to 140
        endif

        call aptsubm (abin(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call binout (n, n, 'min')
              go to 140
            endif
          enddo
        endif

  140 continue

c===============================================================================

c.... See if brick exists.

      do 150 n = 1, nbricks
        afirst = abrick(n)

        call aptsubr (abrick(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. abrick(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call brout (n, n, 'min')
          go to 150
        endif

        call aptsubm (abrick(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call brout (n, n, 'min')
              go to 150
            endif
          enddo
        endif
  150 continue

c===============================================================================

c.... See if cluster exists.

      do 160 n = 1, nclusts
        afirst = aclust(n)

        call aptsubr (aclust(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. aclust(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call clout (n, n, 'min')
          go to 160
        endif

        call aptsubm (aclust(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call clout (n, n, 'min')
              go to 160
            endif
          enddo
        endif
  160 continue

c===============================================================================

c.... See if annular disk exists.

      do 165 n = 1, ndisks
        afirst = adisk(n)

        call aptsubr (adisk(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. adisk(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call dkout (n, n)
          go to 165
        endif

        call aptsubm (adisk(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call dkout (n, n)
              go to 165
            endif
          enddo
        endif
  165 continue

c===============================================================================

c.... See if line exists.

      do 170 n = 1, nlines
        afirst = aline(n)

        call aptsubr (aline(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. aline(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call lout (n, n)
          go to 170
        endif

        call aptsubm (aline(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call lout (n, n)
              go to 170
            endif
          enddo
        endif
  170 continue

c===============================================================================

c.... See if marker exists.

      do 180 n = 1, nmarks
        afirst  = amark(n)
        afirstr = amarkt(n)

        call aptsubr (amark(n), 1, 1, 80, astem, lstem,
     &    asub, lsub, nerra)

        call aptsubr (amarkt(n), 1, 72, 80, astemr, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. amark(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. afirstr)  .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. astemr)   .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call markout (n, n)
          go to 180
        endif

        call aptsubm (amarkt(n), 1, 72, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call markout (n, n)
              go to 180
            endif
          enddo
        endif
  180 continue

c===============================================================================

c.... See if operator exists.

      do 190 n = 1, nopers
        afirst = aoper(n)

        call aptsubr (aoper(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. aoper(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call opout (n, n, 'min')
          go to 190
        endif

        call aptsubm (aoper(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call opout (n, n, 'min')
              go to 190
            endif
          enddo
        endif
  190 continue

c===============================================================================

c.... See if pdf exists.

      do 200 n = 1, npdfs
        afirst = apdf(n)

        call aptsubr (apdf(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. apdf(n)) .or.
     &      (aword(nw) .eq. afirst)  .or.
     &      (aword(nw) .eq. astem)   .or.
     &      (aword(nw) .eq. 'all'))  then
          numobj = numobj + 1
          call pdfout (n, n, 'min')
          go to 200
        endif

        call aptsubm (apdf(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call pdfout (n, n, 'min')
              go to 200
            endif
          enddo
        endif
  200 continue

c===============================================================================

c.... See if point exists.

      do 210 n = 1, npoints
        afirst = apoint(n)

        call aptsubr (apoint(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. apoint(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call pout (n, n, 'min')
          go to 210
        endif

        call aptsubm (apoint(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call pout (n, n, 'min')
              go to 210
            endif
          enddo
        endif
  210 continue

c===============================================================================

c.... Find any numerical values among coordinates of points,
c....   in current coordinate system.

      if (mtype(nw) .ne. 1) then      ! Look for numerical values.

 8119 format ('coord  ',a,5x,a,4x,a,4x,a)

      write (   6, 8119) asys, acoordu, acoordv, acoordw
      write (iout, 8119) asys, acoordu, acoordv, acoordw

      do n = 1, npoints

        call ptcoord (asys, angunit, x(n), y(n), z(n),
     &                u, v, w, vpos, nerra)

        ferru  = tol * (abs (fword(nw)) + abs (u))
        ferrv  = tol * (abs (fword(nw)) + abs (v))
        ferrw  = tol * (abs (fword(nw)) + abs (w))
        ferrvp = tol * (abs (fword(nw)) + abs (vpos))
        if ((abs (fword(nw) - u) .le. ferru)      .or.
     &      (abs (fword(nw) - v) .le. ferrv)      .or.
     &      (abs (fword(nw) - w) .le. ferrw)      .or.
     &      (abs (fword(nw) - vpos) .le. ferrvp)) then
          numobj = numobj + 1
          call pout (n, n, 'min')
        endif
      enddo

      endif                           ! Looked for numerical values.

c===============================================================================

c.... See if polygon exists.

      do 230 n = 1, npolygs
        afirst = apolyg(n)

        call aptsubr (apolyg(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. apolyg(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call pgout (n, n, 'min')
          go to 230
        endif

        call aptsubm (apolyg(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call pgout (n, n, 'min')
              go to 230
            endif
          enddo
        endif
  230 continue

c===============================================================================

c.... See if polyhedron exists.

      do 240 n = 1, npolyhs
        afirst = apolyh(n)

        call aptsubr (apolyh(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. apolyh(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call phout (n, n, 'min')
          go to 240
        endif

        call aptsubm (apolyh(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call phout (n, n, 'min')
              go to 240
            endif
          enddo
        endif
  240 continue

c===============================================================================

c.... See if quadric exists.

      do 250 n = 1, nquads
        afirst = aquad(n)

        call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. aquad(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call quout (n, n, 'min', nerrq)
          go to 250
        endif

        call aptsubm (aquad(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call quout (n, n, 'min', nerrq)
              go to 250
            endif
          enddo
        endif
  250 continue

c===============================================================================

c.... See if symbol exists.

      do 260 n = 1, nsymbs
        afirst  = asymbl(n)
        afirstr = asymbr(n)

        call aptsubr (asymbl(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        call aptsubr (asymbr(n), 1, 24, 80, astemr, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. asymbl(n))  .or.
     &      (aword(nw) .eq. asymbr(n))  .or.
     &      (aword(nw) .eq. afirst)     .or.
     &      (aword(nw) .eq. afirstr)    .or.
     &      (aword(nw) .eq. astem)      .or.
     &      (aword(nw) .eq. astemr)     .or.
     &      (aword(nw) .eq. 'all'))     then
          numobj = numobj + 1
          call symbdisp (n)
          go to 260
        endif

        call aptsubm (asymbl(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call symbdisp (n)
              go to 260
            endif
          enddo
        endif

        call aptsubm (asymbr(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call symbdisp (n)
              go to 260
            endif
          enddo
        endif
  260 continue

c===============================================================================

c.... Find any numerical values among symbols.

      if (mtype(nw) .eq. 2) then      ! Search for integer values.

        do n = 1, nsymbs
          if ((msymb(n) .eq. 2)           .and.
     &      ((iword(nw) .eq. isymb(n))    .or.
     &        (fword(nw) .eq. fsymb(n)))) then
            numobj = numobj + 1
            call symbdisp (n)
          endif
        enddo

      endif                           ! Search for integer values.

      if (mtype(nw) .eq. 3) then      ! Search for floating point values.

        do n = 1, nsymbs
          ferr = tol * (abs (fword(nw)) + abs (fsymb(n)))
          if (abs (fword(nw) - fsymb(n)) .le. ferr) then
            numobj = numobj + 1
            call symbdisp (n)
          endif
        enddo

      endif                           ! Search for floating point values.

c===============================================================================

c.... See if tetrahedron exists.

      do 290 n = 1, ntets
        afirst = atetra(n)

        call aptsubr (atetra(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. atetra(n)) .or.
     &      (aword(nw) .eq. afirst)    .or.
     &      (aword(nw) .eq. astem)     .or.
     &      (aword(nw) .eq. 'all'))    then
          numobj = numobj + 1
          call tetout (n, n, 'min')
          go to 290
        endif

        call aptsubm (atetra(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call tetout (n, n, 'min')
              go to 290
            endif
          enddo
        endif
  290 continue

c===============================================================================

c.... See if triangle exists.

      do 300 n = 1, ntrigs
        afirst = atrig(n)

        call aptsubr (atrig(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. atrig(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call trout (n, n, 'min')
          go to 300
        endif

        call aptsubm (atrig(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call trout (n, n, 'min')
              go to 300
            endif
          enddo
        endif
  300 continue

c===============================================================================

c.... See if variable exists.

      do 310 n = 1, nvars
        afirst  = avarbl(n)
        afirstr = avartag(n)

        call aptsubr (avarbl(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        call aptsubr (avartag(n), 1, 48, 80, astemr, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. avarbl(n))   .or.
     &      (aword(nw) .eq. avartag(n))  .or.
     &      (aword(nw) .eq. afirst)      .or.
     &      (aword(nw) .eq. afirstr)     .or.
     &      (aword(nw) .eq. astem)       .or.
     &      (aword(nw) .eq. astemr)      .or.
     &      (aword(nw) .eq. 'all'))      then
          numobj = numobj + 1
          call varout (n, n)
          go to 310
        endif

        call aptsubm (avarbl(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call varout (n, n)
              go to 310
            endif
          enddo
        endif
  310 continue

c===============================================================================

c.... Find any numerical values among variables.

      if (mtype(nw) .eq. 2) then      ! Search for integer values.

        do n = 1, nvars
          if ((mvar(n) .eq. 2)           .and.
     &      ((iword(nw) .eq. ivar(n))    .or.
     &        (fword(nw) .eq. fvar(n)))) then
            numobj = numobj + 1
            call varout (n, n)
          endif
        enddo

      endif                           ! Search for integer values.

      if (mtype(nw) .eq. 3) then      ! Search for floating point values.

        do n = 1, nvars
          ferr = tol * (abs (fword(nw)) + abs (fvar(n)))
          if (abs (fword(nw) - fvar(n)) .le. ferr) then
            numobj = numobj + 1
            call varout (n, n)
          endif
        enddo

      endif                           ! Search for floating point values.

c===============================================================================

c.... See if vector exists.

      do 340 n = 1, nvectors
        afirst = avector(n)

        call aptsubr (avector(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. avector(n)) .or.
     &      (aword(nw) .eq. afirst)     .or.
     &      (aword(nw) .eq. astem)      .or.
     &      (aword(nw) .eq. 'all'))     then
          numobj = numobj + 1
          call vout (n, n)
          go to 340
        endif

        call aptsubm (avector(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call vout (n, n)
              go to 340
            endif
          enddo
        endif
  340 continue

c===============================================================================

c.... Find any numerical values among components or bound points of vectors,
c....   in current coordinate system.

      if (mtype(nw) .ne. 1) then      ! Look for numerical values.

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      do n = 1, nvectors

        du = xcom(n)
        dv = ycom(n)
        dw = zcom(n)

        u  = xbnd(n)
        v  = ybnd(n)
        w  = zbnd(n)

c....   Convert units, if necessary, for output.

        if     (asys .eq. 'cyl') then
          call aptcsyv (0, 1, iunit, u, v, w, du, dv, dw, 1, tol, nerra)
        elseif (asys .eq. 'sph') then
          call aptcsyv (0, 2, iunit, u, v, w, du, dv, dw, 1, tol, nerra)
        endif

        ferru  = tol * (abs (fword(nw)) + abs (u))
        ferrv  = tol * (abs (fword(nw)) + abs (v))
        ferrw  = tol * (abs (fword(nw)) + abs (w))
        ferrdu = tol * (abs (fword(nw)) + abs (du))
        ferrdv = tol * (abs (fword(nw)) + abs (dv))
        ferrdw = tol * (abs (fword(nw)) + abs (dw))
        if ((abs (fword(nw) - u)  .le. ferru)   .or.
     &      (abs (fword(nw) - v)  .le. ferrv)   .or.
     &      (abs (fword(nw) - w)  .le. ferrw)   .or.
     &      (abs (fword(nw) - du) .le. ferrdu)  .or.
     &      (abs (fword(nw) - dv) .le. ferrdv)  .or.
     &      (abs (fword(nw) - dw) .le. ferrdw)) then
          numobj = numobj + 1
          call vout (n, n)
        endif
      enddo

c===============================================================================

c.... See if zone exists.

      do 360 n = 1, nzones
        afirst = azone(n)

        call aptsubr (azone(n), 1, 24, 80, astem, lstem,
     &    asub, lsub, nerra)

        if ((aword(nw) .eq. azone(n)) .or.
     &      (aword(nw) .eq. afirst)   .or.
     &      (aword(nw) .eq. astem)    .or.
     &      (aword(nw) .eq. 'all'))   then
          numobj = numobj + 1
          call zout (n, n, 'min')
          go to 360
        endif

        call aptsubm (azone(n), 1, 24, iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .ge. 1) then
          do nn = 1, nsub
            afirst = asub(nn)
            if ((aword(nw) .eq. asub(nn))  .or.
     &          (aword(nw) .eq. afirst  )  .or.
     &         ((mtype(nw) .eq. 2) .and. (msub(nn) .eq. 2) .and.
     &          (iword(nw) .eq. isub(nn)))  .or.
     &         ((mtype(nw) .eq. 3) .and. (msub(nn) .eq. 3) .and.
     &          (abs (fword(nw) - fsub(nn)) .le.
     &           tol * (abs (fword(nw)) + abs (fsub(nn)))))) then
              numobj = numobj + 1
              call zout (n, n, 'min')
              go to 360
            endif
          enddo
        endif
  360 continue

c===============================================================================

      endif                           ! Looked for numerical values.

c.... Display number of objects found.

 8250 format (/ 'Found    no objects with name, tag or value:  ',a)
 8260 format (/ 'Found',i6,' object  with name, tag or value:  ',a)
 8270 format (/ 'Found',i6,' objects with name, tag or value:  ',a)

      if (numobj .eq. 0) then
        write (   6, 8250) aword(nw)
        write (iout, 8250) aword(nw)
      elseif (numobj .eq. 1) then
        write (   6, 8260) numobj, aword(nw)
        write (iout, 8260) numobj, aword(nw)
      else
        write (   6, 8270) numobj, aword(nw)
        write (iout, 8270) numobj, aword(nw)
      endif

  400 continue                        ! End of loop over names and values.

  999 return

c.... End of subroutine search.       (+1 line)
      end
