      subroutine tetra

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  tetra    Updated     2005 April 5 17:45.
c               tetra    Originated  1994 January 31 11:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create tetrahedrons by specifying four points.
c
c     Logic:    Entry points:  tetra.
c tetra         Called by:  helpall, main.
c
c tetra         Calls:  aptsubr, incrsubs, namechk, tetout, tetstore, tettest,
c               varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /cctet/ lapointx(4)      ! Number of characters in apointx.
      common /cctet/ np               ! Index in table of points.
      common /cctet/ incp(4)          ! Increment in a vertex point name.
      common /cctet/ apointx(4)       ! Temporary name of a vertex point.
      character*24   apointx          ! Temporary name of a vertex point.

      character*8    amount           ! Amount of output to write in tetout.
      character*24   anamex           ! Temporary name of a tetrahedron.

c.... Input line:  tetrahedron [help]

 8110 format (
     &  '  Command "tetrahedron":  acceptable input lines include:' //
     &  '  "tetrahedron [help]"  Display this message.' /
     &  '  "tetrahedron all"     Display all tetrahedrons.' /
     &  '  "tetrahedron list [TETNAME1 TETNAME2 TETNAME3 ...]"' /
     &  '     Display all listed tetrahedrons (default is all',
     &  ' tetrahedrons).' /
     &  '  "tetrahedron TETNAME"  Display tetrahedron TETNAME.' //
     &  '  "tetrahedron TETNAME = PNAME1 PNAME2 PNAME3 PNAME4"' /
     &  '     Create tetrahedron TETNAME through the 4',
     &  ' points PNAME1, PNAME2,' /
     &  '     PNAME3 and PNAME4.' )
 8112 format (/
     &  '  "tetrahedron TET02 increment NUMTET INC TET01 IP1 [IP2',
     &  ' [IP3 [IP4]]]"' /
     &  '     Create a family of NUMTET tetrahedrons TET02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by incrementing the names of the four vertex',
     &  ' points of' /
     &  '     the preceding tetrahedron by the number of characters',
     &  ' indicated' /
     &  '     by the integers IP1, IP2, IP3, IP4, starting from',
     &  ' tetrahedron TET01.' /
     &  '     Unspecifed values of IP2 - IP4 default to zero.' //
     &  '  Note:  use +/- for TETNAME or TET02 to incr/decr the last' /
     &  '    tetrahedron name:  ',a //
     &  '  Synonyms:  [help, h], [increment, incr],',
     &  ' [tetrahedron, tetra, tet].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112) atetras
        write (iout, 8112) atetras
        go to 999
      endif

c===============================================================================

c.... Input line:  tetrahedron all
c.... Input line:  tetrahedron list

 8120 format (/ 'Number of tetrahedrons = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        ntetx = 0
        do 120 n = 1, ntets
          if (atetra(n) .eq. ' ') go to 120
          ntetx = ntetx + 1
  120   continue
        if (ntetx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call tetout (1, ntets, amount)
        write (   6, 8120) ntetx
        write (iout, 8120) ntetx
        go to 999
      endif

c===============================================================================

c.... Input line:  tetrahedron list TETNAME1 TETNAME2 TETNAME3 ...

 8130 format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named tetrahedron does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, ntets
            call aptsubr (atetra(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. atetra(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              call tetout (n, n, 'min')
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  tetrahedron TETNAME

c.... See if user wants one tetrahedron displayed.

      if (nword .eq. 2) then
        do 130 n = 1, ntets
          if (aword(2) .eq. atetra(n)) then
            amount = 'maximum'
            call tetout (n, n, amount)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

 8136 format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of tetrahedron not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last tetrahedron name should be incremented and used.

 8128 format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last tetrahedron can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = atetras
        lword(2) = latetras
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, atetras
          write (iout, 8128) ainput, ninput, atetras
          go to 999
        endif
      endif

 8129 format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last tetrahedron can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = atetras
        lword(2) = latetras
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, atetras
          write (iout, 8129) ainput, ninput, atetras
          go to 999
        endif
      endif

 8905 format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of tetrahedron too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  tetrahedron TET02 incr NUMTET INC TET01 IP1 [IP2 [IP3 [IP4]]]
c....   Note:  "incr" is an abbreviation for "increment".

      if ((aword(3) .eq. 'increment') .or.
     &    (aword(3) .eq. 'incr'))     then

        nerr   = 0

 8505   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Too many words in input line:  ',i3)

        if (nword .gt. 10) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          go to 999
        endif

 8508   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Too few words in input line:  ',i3)

        if (nword .lt. 7) then
          write (   6, 8508) ainput, ninput, nword
          write (iout, 8508) ainput, ninput, nword
          go to 999
        endif

 8520   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMTET is not a positive integer:  ',a)
 8525   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMTET is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numtet = iword(4)
          if (numtet .lt. 1) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            go to 999
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          go to 999
        endif

 8510   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
            go to 999
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          go to 999
        endif

c....   Find the base tetrahedron.

        do 530 nt = 1, ntets
          if (aword(6) .eq. atetra(nt)) then
            nbase = nt
            call tetout (nbase, nbase, 'min')
            go to 540
          endif
  530   continue
        write (   6, 8130) ainput, ninput, aword(6)
        write (iout, 8130) ainput, ninput, aword(6)
        go to 999

c....   Find the increments to the vertex point names.

  540   do 550 npt = 1, 4
          incp(npt) = 0
  550   continue

 8560   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not an integer or integer variable:  ',a)

        call varfind (7, nword)

        do 560 nw = 7, nword
          if (mtype(nw) .eq. 2) then
            incp(nw-6) = iword(nw)
          else
            write (   6, 8560) ainput, ninput, aword(nw)
            write (iout, 8560) ainput, ninput, aword(nw)
            go to 999
          endif
  560   continue

c....   See if the tetrahedron names can be generated uniquely.

 8570   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' tetrahedron names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numtet - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8570) ainput, ninput, numtet, aword(2)
          write (iout, 8570) ainput, ninput, numtet, aword(2)
          go to 999
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          go to 999
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) go to 999

c....   Create the new tetrahedrons.

        anamex  = aword(2)            ! Initial tetrahedron name.
        lanamex = lword(2)

        do 590 ntt = 1, numtet        ! Loop over the new tetrahedrons.

c....     Find the index in the tetrahedron list for the new tetrahedron.

          aword(2) = anamex
          lword(2) = lanamex

          call tetstore (nrep, n)

          if (n .eq. 0) go to 999

          do 580 nv = 1, 4

            apointx(nv)  = atetpt(nv,nbase)
            lapointx(nv) = latetpt(nv,nbase)

            if (incp(nv) .eq. 0) go to 580

            call incrsubs (incp(nv), apointx(nv), 1, lapointx(nv),
     &                     nerra)

 8580       format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Can not increment point name:  ',a)

            if (nerra .ne. 0) then
              write (   6, 8580) ainput, ninput, atetpt(nv,nbase)
              write (iout, 8580) ainput, ninput, atetpt(nv,nbase)
              go to 999
            endif

  580     continue

          call tettest (apointx(1), apointx(2), apointx(3), apointx(4),
     &                  n1, n2, n3, n4, nerr)

          if (nerr .ne. 0) go to 999
    
          atetras   = anamex
          latetras  = lanamex

          atetra(n) = anamex

          do 585 nv = 1, 4
            atetpt(nv,n)  = apointx(nv)
            latetpt(nv,n) = lapointx(nv)
  585     continue

          voltet(n) = 0.0

          call tetout (n, n, 'min')

          nbase     = n

c....     Find the next tetrahedron name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

  590   continue                      ! End of loop over the new tetrahedrons.

        go to 999

      endif

c===============================================================================

c.... Find the index in the tetrahedron array at which to store this
c....   tetrahedron.
c....   Set nrep = 1 if replacing an existing tetrahedron.

      call tetstore (nrep, n)

      if (n .eq. 0) go to 999

c===============================================================================

c.... Input line:  tetrahedron TETNAME = PNAME1 PNAME2 PNAME3 PNAME4

      if (aword(3) .eq. '=') then

 8135   format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words on "tetrahedron" line not 7:  ',i3)
        if (nword .ne. 7) then
          write (   6, 8135) ainput, ninput, nword
          write (iout, 8135) ainput, ninput, nword
          go to 999
        endif

c....   See if the points are unique.

        call tettest (aword(4), aword(5), aword(6), aword(7),
     &                n1, n2, n3, n4, nerrtet)

        if (nerrtet .ne. 0) then
          if (nrep .eq. 0) then
            atetra(n) = ' '
            if (n .eq. ntets) ntets = ntets - 1
          endif
          go to 999
        endif

c....   Store and display the tetrahedron specifications.

        atetras      = aword(2)
        latetras     = lword(2)

        atetra(n)    = aword(2)

        atetpt(1,n)  = apoint(n1)
        atetpt(2,n)  = apoint(n2)
        atetpt(3,n)  = apoint(n3)
        atetpt(4,n)  = apoint(n4)
        latetpt(1,n) = lapoint(n1)
        latetpt(2,n) = lapoint(n2)
        latetpt(3,n) = lapoint(n3)
        latetpt(4,n) = lapoint(n4)
        voltet(n)    = 0.0             ! Calculate later in tetout.

        amount = 'maximum'
        call tetout (n, n, amount)

        go to 999

      endif                           ! Created a tetrahedron.

c===============================================================================

 8440 format ('  tetra ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8440) ainput, ninput
      write (iout, 8440) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine tetra.        (+1 line)
      end
