      subroutine trout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  trout    Updated     2005 December 20 20:00.
c               trout    Originated  1993 November 29 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the coordinates of the three points dcut3ining
c               triangles n1 to n2, and related data.
c
c     Logic:    Entry points:  trout.
c trout         Called by:  copy, delete, distance, plane, point, renamea,
c               renamex, search, track, triangle, trig.
c
c trout         Calls:  aptblsq, aptcsys, aptcsyv, aptrcut, aptripp, aptsver,
c               apttrig, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      dimension ex(3)                 ! Cooordinate x of cut line point E.
      dimension ey(3)                 ! Cooordinate y of cut line point E.
      dimension ez(3)                 ! Cooordinate z of cut line point E.
      dimension fx(3)                 ! Cooordinate x of cut line point F.
      dimension fy(3)                 ! Cooordinate y of cut line point F.
      dimension fz(3)                 ! Cooordinate z of cut line point F.
      dimension dcut1(3)              ! Distance of E on first edge.
      dimension dcut2(3)              ! Distance of F on second edge.
      dimension dcut3(3)              ! Length of cutting line.
      dimension fcut1(3)              ! Fractional dist of E on first edge.
      dimension fcut2(3)              ! Fractional dist of F on second edge.

      character*8   amount            ! Amount of output to write.
      character*8   ans               ! User's response.
      character*1   acut1(3)          ! Names of edges or vertices for point E.
      character*1   acut2(3)          ! Names of edges or vertices for point F.

      character*32  awordx(32)        ! Temporary space for output line.
      character*1   asink(160)        ! Temporary space for output line.

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. ntrigs) nn2 = ntrigs

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

 8225 format ('triangle ',160a1)
 8325 format ('symbol atript1 = ',a /
     &        'symbol atript2 = ',a /
     &        'symbol atript3 = ',a /
     &        'triangle ',a,' = atript1 atript2 atript3')
 8135 format ('coord  ',a,5x,a,4x,a,4x,a)
 8141 format ('  trout ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Vertex point does not exist:  ',a)

      do 120 n = nn1, nn2

        if (atrig(n) .eq. ' ') go to 120

        awordx(1) = atrig(n)
        awordx(2) = '='
        awordx(3) = atript1(n)
        awordx(4) = atript2(n)
        awordx(5) = atript3(n)

        call aptblsq (awordx, 1, 160, asink, 1, lsink, nerra)

        if (lsink .le. 70) then
          write (   6, 8225) (asink(i), i = 1, lsink)
          write (iout, 8225) (asink(i), i = 1, lsink)
        else
          write (   6, 8325) atript1(n), atript2(n), atript3(n),
     &                       atrig(n)
          write (iout, 8325) atript1(n), atript2(n), atript3(n),
     &                       atrig(n)
        endif


        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

          write (   6, 8135) asys, acoordu, acoordv, acoordw
          write (iout, 8135) asys, acoordu, acoordv, acoordw

          do 141 np = 1, npoints
            if (apoint(np) .eq. atript1(n)) then
              ax = x(np)
              ay = y(np)
              az = z(np)
              call pout (np, np, 'min')
              go to 142
            endif
  141     continue
          write (   6, 8141) ainput, ninput, atript1(n)
          write (iout, 8141) ainput, ninput, atript1(n)
          go to 120

  142     do 143 np = 1, npoints
            if (apoint(np) .eq. atript2(n)) then
              bx = x(np)
              by = y(np)
              bz = z(np)
              call pout (np, np, 'min')
              go to 144
            endif
  143     continue
          write (   6, 8141) ainput, ninput, atript2(n)
          write (iout, 8141) ainput, ninput, atript2(n)
          go to 120

  144     do 145 np = 1, npoints
            if (apoint(np) .eq. atript3(n)) then
              cx = x(np)
              cy = y(np)
              cz = z(np)
              call pout (np, np, 'min')
              go to 146
            endif
  145     continue
          write (   6, 8141) ainput, ninput, atript3(n)
          write (iout, 8141) ainput, ninput, atript3(n)
          go to 120

c....     Find the geometric data for the triangle.

  146     call apttrig (ax, ay, az, bx, by, bz, cx, cy, cz, tol,
     &                  abx, aby, abz, dab, bcx, bcy, bcz, dbc,
     &                  cax, cay, caz, dca, anga, angb, angc,
     &                  vnx, vny, vnz, areatr,
     &                  vmax, vmay, vmaz, dma, pmax, pmay, pmaz,
     &                  vmbx, vmby, vmbz, dmb, pmbx, pmby, pmbz,
     &                  vmcx, vmcy, vmcz, dmc, pmcx, pmcy, pmcz,
     &                  cgx, cgy, cgz,
     &                  viax, viay, viaz, dvia, anax, anay, anaz,
     &                  vibx, viby, vibz, dvib, anbx, anby, anbz,
     &                  vicx, vicy, vicz, dvic, ancx, ancy, ancz,
     &                  cix, ciy, ciz, radinsc,
     &                  vqabx, vqaby, vqabz, dqab,
     &                  vqbcx, vqbcy, vqbcz, dqbc,
     &                  vqcax, vqcay, vqcaz, dqca, ccx, ccy, ccz,
     &                  radcirc,
     &                  vhax, vhay, vhaz, dha, alax, alay, alaz,
     &                  vhbx, vhby, vhbz, dhb, albx, alby, albz,
     &                  vhcx, vhcy, vhcz, dhc, alcx, alcy, alcz,
     &                  cox, coy, coz, nerra)

c....     Convert the normal vector to the current coordinate system,
c....       assuming it is bound at vertex "a" of the triangle.

          u = ax
          v = ay
          w = az

          du = vnx
          dv = vny
          dw = vnz

          if     (asys .eq. 'cyl') then

            call aptcsyv (0, 1, iunit, u, v, w, du, dv, dw, 1,
     &                    tol, nerra)
            call aptcsys (0, 1, iunit, ccx, ccy, ccz, 1, tol, nerra)
            call aptcsys (0, 1, iunit, cix, ciy, ciz, 1, tol, nerra)
            call aptcsys (0, 1, iunit, cgx, cgy, cgz, 1, tol, nerra)
            call aptcsys (0, 1, iunit, cox, coy, coz, 1, tol, nerra)

          elseif (asys .eq. 'sph') then

            call aptcsyv (0, 2, iunit, u, v, w, du, dv, dw, 1,
     &                    tol, nerra)
            call aptcsys (0, 2, iunit, ccx, ccy, ccz, 1, tol, nerra)
            call aptcsys (0, 2, iunit, cix, ciy, ciz, 1, tol, nerra)
            call aptcsys (0, 2, iunit, cgx, cgy, cgz, 1, tol, nerra)
            call aptcsys (0, 2, iunit, cox, coy, coz, 1, tol, nerra)

          endif

c....     Write out the normal vector and the triangle area.

 8130     format ('  Normal vector   ',1p3e20.12 /
     &            '  Triangle area   ',1pe20.12)

          write (   6, 8130) du, dv, dw, areatr
          write (iout, 8130) du, dv, dw, areatr

c....     Write out the three edge lengths.

 8128     format ('  Edge lengths abc',1p3e20.12 )

          write (   6, 8128) dbc, dca, dab 
          write (iout, 8128) dbc, dca, dab 

c....     Write out the perimeter.

          per = dab + dbc + dca

 8129     format ('  Perimeter       ',1p3e20.12 )

          write (   6, 8129) per
          write (iout, 8129) per

c....     Write out the three vertex angles.

 8140     format ('  Angles ABC (deg)',1p3e20.12 )
 8142     format ('  Angles ABC (rad)',1p3e20.12 )

          if (iunit .eq. 0) then
            write (   6, 8140) anga, angb, angc
            write (iout, 8140) anga, angb, angc
          else
            anga = anga * pi / 180.0
            angb = angb * pi / 180.0
            angc = angc * pi / 180.0
            write (   6, 8142) anga, angb, angc
            write (iout, 8142) anga, angb, angc
          endif

c....     Write out the center and radius of the circumscribed circle.

 8132     format ('  Center circumscr',1p3e20.12 /
     &            '  Radius circumscr',1pe20.12)

          write (   6, 8132) ccx, ccy, ccz, radcirc
          write (iout, 8132) ccx, ccy, ccz, radcirc

c....     Write out the center and radius of the inscribed circle.

 8134     format ('  Center inscribed',1p3e20.12 /
     &            '  Radius inscribed',1pe20.12)

          write (   6, 8134) cix, ciy, ciz, radinsc
          write (iout, 8134) cix, ciy, ciz, radinsc

c....     Write out the center of gravity (centroid).

 8136     format ('  Centroid        ',1p3e20.12 )

          write (   6, 8136) cgx, cgy, cgz
          write (iout, 8136) cgx, cgy, cgz

c....     Write out the orthocenter (intersection of altitudes).

 8138     format ('  Orthocenter     ',1p3e20.12 )

          write (   6, 8138) cox, coy, coz
          write (iout, 8138) cox, coy, coz

c=======================================================================********

 8150     format (/ 'Type <return> for more data.' )
 8155     format (a)

          if (ainput .eq. 'tty') then
            write (   6, 8150)
            write (iout, 8150)
            read  (   5, 8155) ans
          endif

c=======================================================================********

c....     Write out the radii of the three tangent circles at the vertices.

 8181     format (/ '  Tangent circles at vertices:' /
     &            '  Radii           ',1p3e20.12)
 8182     format ('  Tangent pt (xyz)',1p3e20.12 /
     &            '  Tangent pt (xyz)',1p3e20.12 /
     &            '  Tangent pt (xyz)',1p3e20.12 )

          ra = 0.5 * (dca + dab - dbc)
          rb = 0.5 * (dab + dbc - dca)
          rc = 0.5 * (dbc + dca - dab)

          write (   6, 8181) ra, rb, rc
          write (iout, 8181) ra, rb, rc

c....     Write out the coordinates of the tangent points.

          xab = (rb * ax + ra * bx) / (dab + 1.e-99)
          yab = (rb * ay + ra * by) / (dab + 1.e-99)
          zab = (rb * az + ra * bz) / (dab + 1.e-99)

          xbc = (rc * bx + rb * cx) / (dbc + 1.e-99)
          ybc = (rc * by + rb * cy) / (dbc + 1.e-99)
          zbc = (rc * bz + rb * cz) / (dbc + 1.e-99)

          xca = (ra * cx + rc * ax) / (dca + 1.e-99)
          yca = (ra * cy + rc * ay) / (dca + 1.e-99)
          zca = (ra * cz + rc * az) / (dca + 1.e-99)

          write (   6, 8182) xab, yab, zab, xbc, ybc, zbc,
     &                       xca, yca, zca
          write (iout, 8182) xab, yab, zab, xbc, ybc, zbc,
     &                       xca, yca, zca

c....     Write out the radii of the two central circles tangent to the three
c....       vertex circles.

          rra = 1.0 / (ra + 1.e-99)
          rrb = 1.0 / (rb + 1.e-99)
          rrc = 1.0 / (rc + 1.e-99)

          fsum  = rra + rrb + rrc
          froot = 2.0 * sqrt (rra * rrb + rrb * rrc + rrc * rra)

          rr1   = fsum + froot
          rr2   = fsum - froot

          r1    = 1.0 / (rr1 + 1.e-99)
          r2    = 1.0 / (rr2 + 1.e-99)

 8195     format ('  Circles tangent to vertex  circles:' /
     &            '  Radii           ',1p2e20.12)

          write (   6, 8195) r1, r2
          write (iout, 8195) r1, r2

c=======================================================================********

c....     Find lines that cut triangle into two figures with equal areas
c....       and perimeters.

          call aptrcut (ax, ay, az, bx, by, bz,
     &                  cx, cy, cz, tol,
     &                  ncut, acut1, fcut1, dcut1, ex, ey, ez,
     &                  acut2, fcut2, dcut2, fx, fy, fz, dcut3, nerra)

          if (nerra .ne. 0) go to 430

 8165     format (/ '  Lines that cut triangle (with edges labeled',
     &     ' a, b, and c, and opposite' /
     &     '    vertices labeled A, B and C) into figures with equal',
     &     ' perimeters and areas:' )

          write (   6, 8165)
          write (iout, 8165)

 8170     format ('  From ',a1,5x,'(xyz)',1p3e20.12 /
     &            '    to ',a1,5x,'(xyz)',1p3e20.12 )
 8175     format ('  Frac dist 1,2 = ',1p2e20.12)
 8180     format ('  Dist 1, 2, cut= ',1p3e20.12)

          do 420 nn = 1, ncut

            write (   6, 8170) acut1(nn), ex(nn), ey(nn), ez(nn),
     &        acut2(nn), fx(nn), fy(nn), fz(nn)
            write (iout, 8170) acut1(nn), ex(nn), ey(nn), ez(nn),
     &        acut2(nn), fx(nn), fy(nn), fz(nn)

            write (   6, 8175) fcut1(nn), fcut2(nn)
            write (iout, 8175) fcut1(nn), fcut2(nn)

            write (   6, 8180) dcut1(nn), dcut2(nn), dcut3(nn)
            write (iout, 8180) dcut1(nn), dcut2(nn), dcut3(nn)

  420     continue

  430     continue

c=======================================================================********

c....     Find the Steiner vertix.

          call aptsver (ax, ay, az, bx, by, bz, cx, cy, cz, tol,
     &                  sa, sb, sc, sum, wa, wb, wc, sx, sy, sz, nerr)

 8530     format (/ '  Steiner pt (xyz)',1p3e20.12 /
     &            '  Lengths sa,sb,sc',1p3e20.12 /
     &            '  Sum sa + sb + sc',1pe20.12 /
     &            '  Weights wa,wb,wc',1p3e20.12 / )

          write (   6, 8530) sx, sy, sz, sa, sb, sc, sum, wa, wb, wc
          write (iout, 8530) sx, sy, sz, sa, sb, sc, sum, wa, wb, wc

c=======================================================================********

c....     Find the equilateral triangle that projects onto this one.

          call aptripp (ax, ay, az, bx, by, bz, cx, cy, cz, tol,
     &                  rx, ry, rz, sx, sy, sz, tx, ty, tz, edge,
     &                  ux, uy, uz, nerr)

 8535     format (/ '  Equilateral triangle that projects onto'
     &              ' this one.' /
     &              '  point  e1       = ',1p3e20.12 /
     &              '  point  e2       = ',1p3e20.12 /
     &              '  point  e3       = ',1p3e20.12 /
     &              '  side            = ',1pe20.12  / )

          write (   6, 8535) rx, ry, rz, sx, sy, sz, tx, ty, tz, edge
          write (iout, 8535) rx, ry, rz, sx, sy, sz, tx, ty, tz, edge

c=======================================================================********

c....     See if triangle has zero area (vertices colinear).

8145      format ('  trout ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Points are colinear.')

          if (areatr .eq. 0.0) then
            write (   6, 8145) ainput, ninput
            write (iout, 8145) ainput, ninput
          endif

        endif                         ! Displayed maximum amount of data.

  120 continue

      return

c.... End of subroutine trout.        (+1 line)
      end
