      subroutine twist

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  twist    Updated     2004 November 23 13:30.
c               twist    Originated  1995 May 2 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To twist one or all points or a cluster around a specified
c               axis, depending on the axial distance from a point on the
c               axis, or depending on the radial distance from the axis,
c               with a specified pitch.
c
c     Logic:    Entry points:  twist.
c twist         Called by:  helpall, main.
c
c twist         Calls:  aptwirl, aptwist, clout, pout, ptfind, varfind,
c               vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /ltwist/ amount          ! Amount of output data.
      character*8     amount

c.... Input line:  twist [help]

 8110 format ('  Command "twist":  acceptable input lines include:' //
     &  '  "twist [help]"   Display this message.' //
     &  '  "twist point   PNAME  PAXIS VAXIS axial PITCH"' /
     &  '     Twist point   PNAME  around the axis through',
     &  ' point PAXIS' /
     &  '     in the direction of vector VAXIS, with one',
     &  ' clockwise rotation' /
     &  '     per axial distance PITCH, along the axis',
     &  ' from point PAXIS.' //
     &  '  "twist point   all    PAXIS VAXIS axial PITCH"' /
     &  '     As above, for all points.' //
     &  '  "twist cluster CLNAME PAXIS VAXIS axial PITCH"' /
     &  '     As above, for all points in cluster CLNAME.' )

 8111 format (/
     &  '  "twist point   PNAME  PAXIS VAXIS radial PITCH RINV"' /
     &  '     Twist point   PNAME  around the axis through',
     &  ' point PAXIS' /
     &  '     in the direction of vector VAXIS, with one',
     &  ' clockwise rotation' /
     &  '     per radial distance PITCH from radius RINV.' //
     &  '  "twist point   all    PAXIS VAXIS radial PITCH RINV"' /
     &  '     As above, for all points.' //
     &  '  "twist cluster CLNAME PAXIS VAXIS radial PITCH RINV"' /
     &  '     As above, for all points in cluster CLNAME.' )

 8112 format (/
     &  '  Synonyms:  [cluster, cl], [help, h],',
     &  ' [point, pnt, pt, p],' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then

        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8111)
        write (iout, 8111)
        write (   6, 8112)
        write (iout, 8112)
        go to 999

      endif

c===============================================================================

c.... Initialize.

      nerr = 0

c===============================================================================

c.... Test for recognized command options.

 8115 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Option not recognized:  ',a)

      if ((aword(6) .ne. 'axial')   .and.
     &    (aword(6) .ne. 'radial')) then
        write (   6, 8115) ainput, ninput, aword(6)
        write (iout, 8115) ainput, ninput, aword(6)
        nerr = 1
      endif

c===============================================================================

c.... Test for correct number of words in command.

 8120 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Wrong number of words in command:  ',i3)

      if ((aword(6) .eq. 'axial') .and.
     &    (nword .ne. 7))         then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

      if ((aword(6) .eq. 'radial') .and.
     &    (nword .ne. 8))          then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

c===============================================================================

c.... Find the axis point.

      call ptfind (4, np)

      if (np .eq. 0) then
        nerr = 1
      else
        ax = x(np)
        ay = y(np)
        az = z(np)
      endif

c===============================================================================

c.... Find the axis vector.

      call vectfind (5, nv)

      if (nv .eq. 0) then
        nerr = 1
      else

c....   See if the axis vector has a real length.

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8130) ainput, ninput, aword(5)
          write (iout, 8130) ainput, ninput, aword(5)
          nerr = 1
        else
          vx = xcom(nv)
          vy = ycom(nv)
          vz = zcom(nv)
        endif
      endif

c===============================================================================

c.... Find the pitch of the twist.

 8130 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)
 8135 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  PITCH is zero.  This is not allowed.')

      call varfind (7, 7)

      if ((mtype(7) .ne. 2) .and. (mtype(7) .ne. 3)) then
        write (   6, 8130) ainput, ninput, aword(7)
        write (iout, 8130) ainput, ninput, aword(7)
        nerr = 1
      endif

      if (fword(7) .eq. 0.0) then
        write (   6, 8135) ainput, ninput
        write (iout, 8135) ainput, ninput
        nerr = 1
      endif

      pitch = fword(7)

c===============================================================================

c.... Find the radius at which the twist is zero (option "radial").

 8138 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  RINV is negative.  This is not allowed.')

      if (aword(6) .eq. 'radial') then

        call varfind (8, 8)
  
        if ((mtype(8) .ne. 2) .and. (mtype(8) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(8)
          write (iout, 8130) ainput, ninput, aword(8)
          nerr = 1
        endif
  
        if (fword(8) .lt. 0.0) then
          write (   6, 8135) ainput, ninput
          write (iout, 8135) ainput, ninput
          nerr = 1
        endif
  
        rinv = fword(8)

      endif
c===============================================================================

c.... Input line:  twist point PNAME PAXIS VAXIS axial  PITCH
c.... Input line:  twist point all   PAXIS VAXIS axial  PITCH
c.... Input line:  twist point PNAME PAXIS VAXIS radial PITCH RINV
c.... Input line:  twist point all   PAXIS VAXIS radial PITCH RINV

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

 8140 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no points.')

        if (npoints .eq. 0) then
          write (   6, 8140) ainput, ninput
          write (iout, 8140) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

        if (aword(3) .eq. 'all') then

c....     Operate on all points.

          if (aword(6) .eq. 'axial') then

            call aptwist (ax, ay, az, vx, vy, vz, pitch,
     &                    x, y, z, npoints, tol, nerra)

          else

            call aptwirl (ax, ay, az, vx, vy, vz, pitch, rinv,
     &                    x, y, z, npoints, tol, nerra)

          endif

          amount = 'maximum'
          call pout (1, npoints, amount)

        else

c....     Operate on one point.

          call ptfind (3, np)

          if (np .eq. 0) go to 999

          if (aword(6) .eq. 'axial') then

            call aptwist (ax, ay, az, vx, vy, vz, pitch,
     &                    x(np), y(np), z(np), 1, tol, nerra)

          else

            call aptwirl (ax, ay, az, vx, vy, vz, pitch, rinv,
     &                    x(np), y(np), z(np), 1, tol, nerra)

          endif

          amount = 'maximum'
          call pout (np, np, amount)

        endif                         ! Operated on one or all points.

c===============================================================================

c.... Input line:  twist cluster CLNAME PAXIS VAXIS axial  PITCH
c.... Input line:  twist cluster CLNAME PAXIS VAXIS radial PITCH RINV

      elseif ((aword(2) .eq. 'cluster') .or.
     &        (aword(2) .eq. 'cl'))    then

 8160 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no clusters.')

      if (nclusts .eq. 0) then
        write (   6, 8160) ainput, ninput
        write (iout, 8160) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

      do 180 n = 1, nclusts           ! Loop over clusters.
        if (aword(3) .ne. aclust(n)) go to 180

        call clout (n, n, 'min')

        if (aclust(n) .eq. ' ') go to 190

        do 170 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .ne. aclust(n)) go to 170

          do 160 np = 1, npoints      ! Loop over points.
            if (apoint(np) .ne. apairpt(npr)) go to 160

            amount = 'maximum'
            call pout (np, np, amount)

            if (aword(6) .eq. 'axial') then

              call aptwist (ax, ay, az, vx, vy, vz, pitch,
     &                      x(np), y(np), z(np), 1, tol, nerra)

            else

              call aptwirl (ax, ay, az, vx, vy, vz, pitch, rinv,
     &                      x(np), y(np), z(np), 1, tol, nerra)

            endif

            call pout (np, np, 'min')

            go to 170                 ! Find next point in cluster.
  160     continue                    ! End of loop over points.

 8170     format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named cluster point does not exist:  ',a)

          write (   6, 8170) ainput, ninput, apairpt(npr)
          write (iout, 8170) ainput, ninput, apairpt(npr)
          go to 999

  170   continue                      ! End of loop over cluster pairs.

        go to 999
  180 continue                        ! End of loop over clusters.

 8180 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

  190 write (   6, 8180) ainput, ninput, aword(3)
      write (iout, 8180) ainput, ninput, aword(3)
      go to 999

c===============================================================================

c.... No such object.

      else

 8200 format ('  twist ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Object type is not recognized:  ',a)

        write (   6, 8200) ainput, ninput, aword(2)
        write (iout, 8200) ainput, ninput, aword(2)

      endif                           ! Found object to operate on.

c===============================================================================

  999 return

c.... End of subroutine twist.        (+1 line)
      end
