      subroutine varfind (nw1, nw2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  varfind  Updated     2004 January 9 14:20.
c               varfind  Originated  1994 August 24 14:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find an integer or floating point variable with the same
c               name as an ASCII argument, and replace the mode and value of
c               the argument with the mode and value of the variable, and
c               display the value.
c
c     Logic:    Entry points:  varfind.
c varfind       Called by:  accel, angles, base, bigint, bigtran, bin, brick,
c               cone, contfr, crt, cute, cuts, cylinder, disk, doenddo,
c               ellipsoid, factor, hex, ifendif, incr, input, intcirc, iris,
c               kiss, line, mcvol, mesh, movex, operator, plane, plotx, point,
c               polygon, polyhed, project, prox, ptoper, quadric, qumove,
c               quoper, ratio, root, rootf, roots, sample, slice, sliceq,
c               sphere, spin, symbol, tetra, tolset, triangle, trig, twist,
c               undo, variable, vector, voper, walk, zone.
c
c varfind       Calls:  aptchmv, varout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     aword     Output   ASCII value of argument.
c
c     nw1       Input    First argument to test.  Not more than nw2 or nword.
c
c     nw2       Input    Last argument to test.   Not less than nw1 or 1.
c
c     fword     Output   Floating point value of argument.
c
c     iword     Output   Integer value of argument.
c
c     lword     Output   Number of characters in aword.
c
c     mtype     Output   Mode of argument.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*1  afirst
      character*79 ashift

 8110 format ('  varfind ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Argument number is bad:  ',i5)

      if ((nw1 .lt. 1)      .or.
     &    (nw1 .gt. nw2)    .or.
     &    (nw1 .gt. nword)) then
        write (   6, 8110) ainput, ninput, nw1
        write (iout, 8110) ainput, ninput, nw1
        go to 999
      endif

      if ((nw2 .lt. 1)      .or.
     &    (nw2 .lt. nw1)    .or.
     &    (nw2 .gt. nword)) then
        write (   6, 8110) ainput, ninput, nw1
        write (iout, 8110) ainput, ninput, nw1
        go to 999
      endif

      do 130 nw = nw1, nw2               ! Loop over arguments.
        if (mtype(nw) .ne. 1) go to 130  ! Skip numerical arguments.

        afirst = aword(nw)
        if ((afirst .eq. '+')  .or.
     &      (afirst .eq. '-')) then
          call aptchmv (aword(nw), 2, 79, ashift, 1, nerra)
          aword(nw) = ashift
        endif

        do 120 nvar = 1, nvars           ! Loop over named variables.
          if (aword(nw) .eq. avarbl(nvar)) then
            call varout (nvar, nvar)
            aword(nw) = avar(nvar)
            mtype(nw) = mvar(nvar)
            if (afirst .eq. '-') then
              fword(nw) = -fvar(nvar)
              iword(nw) = -ivar(nvar)
            else
              fword(nw) = fvar(nvar)
              iword(nw) = ivar(nvar)
            endif
            lword(nw) = lvar(nvar)
            go to 130
          endif
  120   continue                         ! End of loop over named variables.

  130 continue                           ! End of loop over arguments.

  999 return

c.... End of subroutine varfind.       (+1 line)
      end
