      subroutine walk

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  walk     Updated     1995 May 1 16:00.
c               walk     Originated  1993 November 4 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To randomly walk a point, with a specified initial position,
c               direction, mean free path, maximum total path length, and
c               maximum distance from the initial position.
c
c     Logic:    Entry points:  walk.
c walk          Called by:  helpall, main.
c
c walk          Calls:  aptrkis, aptsciz, aptvang, aptvdis, aptvusz, pout,
c               ptfind, varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*16 aresult

c.... Input line:  walk [help]

 8110 format ('  Command "walk":  acceptable input lines include:' /
     &  '  "walk [help]"  to get this message.' //
     &  '  "walk PNAME [VNAME,?] DMEAN PATHMAX [DISTMAX|1.e99]"' /
     &  '     Randomly walk point PNAME, with initial direction vector',
     &  ' [VNAME, random],' /
     &  '     mean free path DMEAN, until it walks a total path',
     &  ' length PATHMAX, or' /
     &  '     reaches distance DISTMAX from its initial position,',
     &  ' whichever occurs first.' //
     &  '  Note:  Use command "define ''random walk''" for details.' )

 8112 format (/
     &  '  Note:  if PATHMAX is reached first, the net radial distance',
     &  ' is stored,' /
     &  '    and if DISTMAX is reached first, the total distance',
     &  ' walked is stored,' /
     &  '    for use in command "variable VARNAME = distance".' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c=======================================================================********

c.... Replace missing DISTMAX with 1.e99.

      if (nword .eq. 5) then
        nword = 6
        mtype(6) = 3
        fword(6) = 1.e99
      endif

c=======================================================================********

c.... Test for wrong number of words on "walk" line.

 8120 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 5 or 6 words on "walk" line:  ',i3)

      nerr = 0

      if ((nword .ne. 5) .and. (nword .ne. 6)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

c=======================================================================********

c.... Replace missing DISTMAX with 1.e99.

      if (nword .eq. 5) then
        nword = 6
        mtype(4) = 3
        fword(4) = 1.e99
      endif

c=======================================================================********

c.... Input line:  walk PNAME [VNAME,?] DMEAN PATHMAX DISTMAX

c.... Find the point to walk.

      call ptfind (2, np)

      if (np .eq. 0) nerr = 1

c=======================================================================********

c.... Find the initial direction vector.

      if ((aword(3) .eq. 'random') .or.
     &    (aword(3) .eq. '?'))     then     ! Random. 

        call aptsciz (ux, uy, uz)

 8122   format ('  Init  direction ',1p3e20.12 )

        write (   6, 8122) ux, uy, uz
        write (iout, 8122) ux, uy, uz

      else                            ! Named vector.

        call vectfind (3, nv)

        if (nv .eq. 0) nerr = 1

 8125   format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Initial direction vector has zero length:  ',a)

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8125) ainput, ninput, aword(3)
          write (iout, 8125) ainput, ninput, aword(3)
          nerr = 1
        endif

        call aptvusz (xcom(nv), ycom(nv), zcom(nv), tol,
     &                ux, uy, uz, vlength)

      endif

c=======================================================================********

c.... Find the mean free path.

      call varfind (4, 4)

 8130 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mean free path not integer or floating point value or',
     &  ' variable:  ',a)

      if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
        write (   6, 8130) ainput, ninput, aword(4)
        write (iout, 8130) ainput, ninput, aword(4)
        nerr = 1
      endif

      dmean = fword(4)

 8140 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mean free path not positive:  ',a)

      if (dmean .le. 0.0) then
        write (   6, 8140) ainput, ninput, aword(4)
        write (iout, 8140) ainput, ninput, aword(4)
        nerr = 1
      endif

c=======================================================================********

c.... Find the maximum total path length.

      call varfind (5, 5)

 8150 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  PATHMAX not integer or floating point value or',
     &  ' variable:  ',a)

      if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
        write (   6, 8150) ainput, ninput, aword(5)
        write (iout, 8150) ainput, ninput, aword(5)
        nerr = 1
      endif

      pathmax = fword(5)

 8160 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  PATHMAX not positive:  ',a)

      if (pathmax .le. 0.0) then
        write (   6, 8160) ainput, ninput, aword(5)
        write (iout, 8160) ainput, ninput, aword(5)
        nerr = 1
      endif

c=======================================================================********

c.... Find the maximum distance from the initial position.

      call varfind (6, 6)

 8170 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  DISTMAX not integer or floating point value or',
     &  ' variable:  ',a)

      if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
        write (   6, 8170) ainput, ninput, aword(6)
        write (iout, 8170) ainput, ninput, aword(6)
        nerr = 1
      endif

      distmax = fword(6)

 8180 format ('  walk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  DISTMAX not positive:  ',a)

      if (distmax .le. 0.0) then
        write (   6, 8180) ainput, ninput, aword(6)
        write (iout, 8180) ainput, ninput, aword(6)
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c=======================================================================********

c.... Find the quadric equation of the sphere at the initial point with
c....   radius DISTMAX.

      ac  = x(np)**2 + y(np)**2 + z(np)**2 - distmax**2
      ax  = -2.0 * x(np)
      ay  = -2.0 * y(np)
      az  = -2.0 * z(np)
      axy = 0.0
      ayz = 0.0
      azx = 0.0
      axx = 1.0
      ayy = 1.0
      azz = 1.0

c=======================================================================********

c.... Initialize.

      nwalk   = 0                     ! Number of walk steps.
      pathtot = 0.0                   ! Total path length.
      distout = 0.0                   ! Maximum distance reached.

      xs      = x(np)                 ! Initial x coordinate of point.
      ys      = y(np)                 ! Initial y coordinate of point.
      zs      = z(np)                 ! Initial z coordinate of point.

      uxs     = ux                    ! X component of initial direction vector.
      uys     = uy                    ! Y component of initial direction vector.
      uzs     = uz                    ! Z component of initial direction vector.

      pdist   = -1.e99

      proxvx  = -1.e99
      proxvy  = -1.e99
      proxvz  = -1.e99

c=======================================================================********

c.... Walk the point to census or to the surface of the sphere.

  210 nwalk   = nwalk + 1
      path    = -dmean * log (ranf ()) ! Length of this step.
      pathtot = pathtot + path

      if (pathtot .gt. pathmax) then  ! Exceeded maximum path length.  Back up.

        aresult = 'Reached PATHMAX'
        path    = path + pathmax - pathtot
        pathtot = pathmax

      endif                           ! Reached PATHMAX.

      x(np) = x(np) + path * ux
      y(np) = y(np) + path * uy
      z(np) = z(np) + path * uz

      call aptvdis (xs, ys, zs, x(np), y(np), z(np), 1, tol,
     &              dx, dy, dz, dist, nerra)

      pdist = dist

      if (dist .gt. distmax) then    ! Exceeded maximum distance.  Back up.

        aresult  = 'Reached DISTMAX'

        dintmn = -path * (1.0 + tol)
        dintmx = tol

        call aptrkis (x(np), y(np), z(np), ux, uy, uz, ac, ax, ay, az,
     &                axy, ayz, azx, axx, ayy, azz, dintmn, dintmx, 1,
     &                tol, nints, cx, cy, cz, dint, snx, sny, snz,
     &                cosint, nerra)

        pathtot = pathtot + dint
        pdist   = pathtot
        dist    = distmax

        x(np)   = x(np) + dint * ux
        y(np)   = y(np) + dint * uy
        z(np)   = z(np) + dint * uz

        go to 220

      endif                           ! Reached DISTMAX.

      if (dist .gt. distout) then
        nwalkout = nwalk
        distout  = dist
      endif

      if (pathtot .ge. pathmax) go to 220

c.... Randomly sample a direction vector for the next step.

      call aptsciz (ux, uy, uz)

      go to 210

c=======================================================================********

c.... Reached PATHMAX or DISTMAX.

  220 dnetx = x(np) - xs
      dnety = y(np) - ys
      dnetz = z(np) - zs

      call aptvang (uxs, uys, uzs, ux, uy, uz, 1, tol, costh, nerra)

      angscat = acos (costh)
      if (angunit .eq. 'degrees') angscat = angscat * 180 / pi

      if (aresult .eq. 'Reached PATHMAX') then  ! Reached maximum path length.

 8230   format (2x,a16,1pe20.12)

 8240   format ('  Distance (outer)',1pe20.12,'  Ratio',0pf9.3,4x,
     &    '  Steps',i9 /
     &          '  Distance (final)',1pe20.12,20x,'  Steps',i9 /
     &          '    (expected)    ',1pe20.12,20x,'  Steps',i9 /
     &          '    (std dev)     ',1pe20.12,20x,'  Steps',i9 )

        nwalkexp = 0.5 + pathmax / dmean
        if (nwalkexp .eq. 0) nwalkexp = 1
        nwalkdev = 0.5 + sqrt (pathmax / dmean)
        fact     = 3.0 * pi / 16.0
        distexp  = sqrt (dmean * pathmax / (fact + dmean / pathmax))
        distdev  = sqrt ((2 - fact) * dmean * pathmax)
        distrat  = distout / dist

        write (   6, 8230) aresult, pathmax
        write (iout, 8230) aresult, pathmax

        write (   6, 8240) distout, distrat, nwalkout, dist, nwalk,
     &    distexp, nwalkexp, distdev, nwalkdev
        write (iout, 8240) distout, distrat, nwalkout, dist, nwalk,
     &    distexp, nwalkexp, distdev, nwalkdev

      else                                      ! Reached maximum net distance.

 8250   format (2x,a16,1pe20.12)

 8260   format ('  Tot path length ',1pe20.12,20x,'  Steps',i9 /
     &          '    (expected)    ',1pe20.12,20x,'  Steps',i9 /
     &          '    (std dev)     ',1pe20.12,20x,'  Steps',i9 )

        fact     = distmax / dmean
        pathexp  = distmax * (1.0 + 0.5 * fact)
        pathdev  = sqrt (0.1) * distmax * fact
        nwalkexp = 0.5 + fact * (1.0 + 0.5 * fact)
        if (nwalkexp .eq. 0) nwalkexp = 1
        nwalkdev = 0.5 + sqrt (fact + 0.5 * fact**2) + pathdev / dmean

        write (   6, 8250) aresult, distmax
        write (iout, 8250) aresult, distmax

        write (   6, 8260) pathtot, nwalk, pathexp, nwalkexp, 
     &    pathdev, nwalkdev
        write (iout, 8260) pathtot, nwalk, pathexp, nwalkexp,
     &    pathdev, nwalkdev

      endif

 8270 format ('  Scatter angle   ',1pe20.12,2x,a,
     &        '    Cosine',1pe20.12 /
     &        '  Final direction ',1p3e20.12 /
     &        '  Net dx, dy, dz  ',1p3e20.12 )

      write (   6, 8270) angscat, angunit, costh, ux, uy, uz,
     &   dnetx, dnety, dnetz
      write (iout, 8270) angscat, angunit, costh, ux, uy, uz,
     &   dnetx, dnety, dnetz

      call pout (np, np, 'min')

      proxvx = dnetx
      proxvy = dnety
      proxvz = dnetz

c=======================================================================********

  999 return

c.... End of subroutine walk.         (+1 line)
      end
